import { Bloque } from 'componentes/bloque';
import { Boton } from 'componentes/boton';
import { Campo } from 'componentes/campo';
import { Cortina } from 'componentes/cortina';
import { Pantalla } from 'componentes/pantalla';
import { Tabla } from 'componentes/tabla';
import { DatosTransporte } from 'expedicion/dialogos/js/dialogo-datos-transporte';
import { SeleccionAlbaran } from 'expedicion/dialogos/js/dialogo-seleccion-albaran';
import { Albaran, BultoPedidoPreparado, ExistenciasLote, LineaPedidoPreparado, PedidoPreparado, Preparado, RespuestaResumenBultos } from 'expedicion/modelos/expedicion-tipos';
export type LineaPedido = {
    N_LIN: number;
    GAART_COD: string;
    DESCRIP: string;
    GAART_DESCRIP: string;
    GAALMNOM_COD: string;
    C_PDTE: number;
    C_PDTE2: number;
    GAUBIC_DEF: string;
    GAART_LOTES_SN: 'S' | 'N';
    GAART_UBIC_SN: 'S' | 'N';
    DATOS_DERIVADOS: {
        Articulo: string;
        Almacen: string;
    };
    Existencias: ExistenciasLote[];
};
export type Pedido = {
    GGPEDTIP_COD: string;
    N_PEDIDO: number;
    OBSER: string;
    GGMENVIO_COD: string;
    GGMENVIO_DESCRIP: string;
    GGVEHICULO_COD: string;
    GGVEHICULO_DESCRIP: string;
    GGCLIPR_COD: number;
    GGCLIPR_NOMBRE: string;
    MostrarAlbaranValorado: boolean;
    AlbaranValorado: boolean;
    Albaranes: Albaran[];
    Lineas: LineaPedido[];
    BultosPorLinea: boolean;
    DatosEnvioInicial: boolean;
    PermitirSuperarCantPedido: boolean;
    MostrarUbicDefDetLotes: 'S' | 'N';
};
type DisponibilidadPorArticulo = {
    [GAART_COD: string]: ExistenciasLote[];
};
export declare class PantallaExpedicion extends Pantalla {
    protected botonNuevo: Boton;
    protected botonConfirmar: Boton;
    protected botonPendiente: Boton;
    protected tablaLineas: Tabla;
    protected campoLecturaPedido: Campo;
    protected campoGGCLIPR_COD: Campo;
    protected campoGGCLIPR_NOMBRE: Campo;
    protected campoReferencia: Campo;
    protected bloqueReferencia: Bloque;
    protected campoC_PREP: Campo;
    protected cortina: Cortina;
    protected lecturaPedido: string;
    protected lecturaArticulo: string;
    protected lecturaCantidad: number;
    protected pedido: Pedido;
    protected pedidoPreparado: PedidoPreparado;
    protected disponible: DisponibilidadPorArticulo;
    protected lineaSeleccionada: LineaPedidoPreparado;
    protected _DatosTransporte: DatosTransporte;
    protected ultimoBultoSeleccionado: BultoPedidoPreparado;
    constructor();
    protected inicializaComponentes(): {
        botonNuevo: import("componentes/boton").DefinicionBoton;
        botonConfirmar: import("componentes/boton").DefinicionBoton;
        botonPendiente: import("componentes/boton").DefinicionBoton;
        tablaLineas: any;
        campoLecturaPedido: import("componentes/campo").DefinicionCampo;
        campoGGCLIPR_COD: import("componentes/campo").DefinicionCampo;
        campoGGCLIPR_NOMBRE: import("componentes/campo").DefinicionCampo;
        campoReferencia: import("componentes/campo").DefinicionCampo;
        bloqueReferencia: import("componentes/bloque").DefinicionBloque;
        campoC_PREP: import("componentes/campo").DefinicionCampo;
    };
    protected generaCortina(): import("componentes/cortina").DefinicionCortina;
    actualizaVista(): void;
    protected nuevoPedido(): Promise<void>;
    actualizaModelo(): void;
    protected inicializa(): void;
    protected pideDatosTransporte(p_datosTransporte: DatosTransporte): Promise<DatosTransporte>;
    protected cargaPedido(): Promise<boolean>;
    protected pideAlbaran(p_albaranesPedido?: Albaran[]): Promise<SeleccionAlbaran>;
    protected seleccionaDatosTransporte(p_resultadoDatosTransporte: DatosTransporte): void;
    protected seleccionaAlbaran(p_resultadoSeleccionAlbaran: SeleccionAlbaran): void;
    protected actualizaPreparadoArticuloSinLotesNiUbicaciones(p_valor: any): Promise<void>;
    protected pideYActualizaBultos(p_linea: LineaPedidoPreparado, p_disponible: ExistenciasLote[], p_preparado: Preparado): Promise<void>;
    protected actualizaLinea(p_linea: LineaPedidoPreparado, p_disponible: ExistenciasLote[], p_preparado: Preparado): void;
    protected seleccionaLinea(p_lineaSeleccionada: LineaPedidoPreparado): Promise<void>;
    protected sePuedeConfirmarPedido(): boolean;
    protected actualizaPreparadoConResumenBultos(p_resumenBultos: any, p_opciones: any): void;
    protected confirmaPedido(p_opciones: {
        dejarPendiente: boolean;
    }): Promise<void>;
    protected generaResumenBultos(): Promise<RespuestaResumenBultos>;
    protected pideResumenBultos(p_resumenBultos: RespuestaResumenBultos, p_dejarPendiente: boolean): Promise<unknown>;
    protected generaAlbaran(): Promise<{
        GGALBTIP_COD: string;
        N_ALBARAN: number;
        PREGUNTA_HIST: string;
    }>;
    protected obtenerLinea(): Promise<void>;
    obtenerDatosTransporte(v_pedido: any): Promise<DatosTransporte>;
    mostrarPantallaTransporteCortina(): Promise<void>;
    cambiaEstadoPedido(GGPEDTIP_COD: string, N_PEDIDO: number): Promise<void>;
}
export {};
