define(["require", "exports", "componentes/api", "componentes/boton", "componentes/campo", "componentes/foco", "componentes/lov", "componentes/mensaje", "componentes/notificacion", "componentes/pantalla", "Movimientos/modelos/salidas", "Movimientos/Shared/_BloqueOrigen", "Movimientos/Shared/modelos/DatosAlmacen", "Utilidades/assert"], function (require, exports, api_1, boton_1, campo_1, foco_1, lov_1, mensaje_1, notificacion_1, pantalla_1, salidas_1, _BloqueOrigen_1, DatosAlmacen_1, assert) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PantallaSalidas = void 0;
    class PantallaSalidas extends pantalla_1.Pantalla {
        constructor() {
            super();
            this._model = new salidas_1.SalidasModel();
            const vComponentes = this.inicializaComponentes();
            Object.getOwnPropertyNames(vComponentes).forEach(pNombre => this[pNombre] = vComponentes[pNombre].crea());
            this.inicializaBloques();
            this.ObtenerDatosSalidas().then(vTiposMovimiento => {
                if (vTiposMovimiento) {
                    this._model.TipoMovimiento = vTiposMovimiento.GAMOVTIP[0];
                    this._model.DatosOrigen.AlmacenDefecto = vTiposMovimiento.GAALMNOM_ORI;
                }
            });
            this.actualizaVista();
        }
        inicializaBloques() {
            this._bloqueOrigen = new _BloqueOrigen_1.BloqueOrigen(this);
            this._bloqueOrigen.actualizaBloque();
            this._model.DatosOrigen = this._bloqueOrigen.Datos;
        }
        inicializaComponentes() {
            const vComponentes = {
                _botonNuevo: (0, boton_1.boton)('btnNuevo')
                    .clic(() => {
                    this.Inicializa();
                }),
                _botonGuardar: (0, boton_1.boton)('btnGuardar')
                    .clic(async () => {
                    const resultado = await this.Confirmar();
                    if (resultado)
                        this.Inicializa();
                    this.actualizaVista();
                }),
                _inputArticulo: (0, campo_1.campo)('txtArticulo')
                    .cambio(async (pValor) => {
                    await this.CambioArticulo(pValor);
                    foco_1.foco.enSiguienteA(this._inputArticulo);
                    if (this._bloqueOrigen.Datos.Almacen)
                        foco_1.foco.enSiguienteA(this._bloqueOrigen._inputAlmacen);
                    if (this._bloqueOrigen.Datos.Ubicacion)
                        foco_1.foco.enSiguienteA(this._bloqueOrigen._inputUbicacion);
                }),
                _inputArticuloDescipcion: (0, campo_1.campo)('txtArticuloDescrip'),
                _inputCantidad: (0, campo_1.campo)('txtCantidad')
                    .valida(async () => {
                    await this.CambioCantidad();
                }),
                _botonEstableceCantidad: (0, boton_1.boton)('btnEstableceCantidad')
                    .clic(async () => {
                    await this.establecerCantidad();
                    this.actualizaVista();
                    foco_1.foco.en(this._inputCantidad);
                })
            };
            return vComponentes;
        }
        limpiaCampoCantidad() {
            this._inputCantidad.valor(null);
        }
        Inicializa() {
            this._model = new salidas_1.SalidasModel({
                TipoMovimiento: this._model.TipoMovimiento,
                DatosOrigen: new DatosAlmacen_1.DatosAlmacen({
                    AlmacenDefecto: this._model.DatosOrigen.AlmacenDefecto
                })
            });
            this._bloqueOrigen.Inicializa();
            this.actualizaVista();
            foco_1.foco.en(this._inputArticulo);
        }
        actualizaVista() {
            this._inputArticulo.valor(this._model.Articulo);
            this._inputArticuloDescipcion.valor(this._model.ArticuloDescripcion);
            this._inputCantidad.valor(this._model.Cantidad);
            this._bloqueOrigen.Datos = this._model.DatosOrigen;
            this._bloqueOrigen.actualizaBloque();
            if (this._model.Articulo) {
                this._inputCantidad.habilita();
                const { DatosOrigen: { ControlUbicaciones, ControlLotes, Almacen, Ubicacion, Lote } } = this._model;
                if ((!ControlUbicaciones && !ControlLotes) ||
                    (ControlUbicaciones && !ControlLotes && Almacen && Ubicacion) ||
                    (!ControlUbicaciones && ControlLotes && Almacen && Lote) ||
                    (ControlUbicaciones && ControlLotes && Almacen && Ubicacion && Lote)) {
                    this._botonEstableceCantidad.habilita();
                }
                else {
                    this._botonEstableceCantidad.deshabilita();
                }
            }
            else {
                this._inputCantidad.deshabilita();
            }
            this._botonGuardar.habilitado(this._model.ModeloValido);
        }
        actualizaModelo() {
            this._model.Articulo = this._inputArticulo.valor();
            if (this._bloqueOrigen)
                this._model.DatosOrigen = this._bloqueOrigen.Datos;
            this._model.Cantidad = this._inputCantidad.valor();
        }
        async CambioArticulo(Articulo) {
            this._model = new salidas_1.SalidasModel({ TipoMovimiento: this._model.TipoMovimiento });
            const vArticulo = await this.BuscarCodigosAlternativos(Articulo);
            const resultado = await this.ValidarArticulo(vArticulo);
            if (resultado) {
                this._model.Articulo = resultado.GAART_COD;
                this._model.ArticuloDescripcion = resultado.DESCRIP;
                this._bloqueOrigen.Articulo = resultado.GAART_COD;
                this._bloqueOrigen.ControlUbicaciones = resultado.UBIC_SN;
                this._bloqueOrigen.ControlLotes = resultado.LOTES_SN;
                if (this._bloqueOrigen.Datos.AlmacenDefecto && this._bloqueOrigen.Datos.ControlUbicaciones) {
                    this._bloqueOrigen.Datos.Ubicacion = (await this._bloqueOrigen.UbicacionDefecto()).GAUBIC_DEF;
                }
                this._model.DatosOrigen = this._bloqueOrigen.Datos;
            }
            this.actualizaVista();
        }
        async ValidarArticulo(Articulo) {
            let item;
            try {
                [item] = await (0, api_1.api)("/Movimientos/ArticuloCompleto")
                    .metodo('get')
                    .parametros({
                    empgrupo_cod: this._model.Grupo,
                    emp_cod: this._model.Empresa,
                    lecturaarticulo: Articulo,
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, mensaje_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            assert.esUn(item, "GAART_COD", "DESCRIP");
            return item;
        }
        async CambioCantidad() {
            this.actualizaModelo();
            if (!this._model.DatosOrigen.ModeloValido) {
                await (0, mensaje_1.mensaje)().muestra(Resources.PantallaMovimientos.Error.Datos_Faltantes);
                this._inputCantidad.valor(null);
                foco_1.foco.en(this._inputCantidad);
                this.actualizaVista();
            }
            else {
                this.actualizaVista();
                foco_1.foco.en(this._botonGuardar);
            }
        }
        async ObtenerDatosSalidas() {
            let DatosSalidas;
            [DatosSalidas] = await (0, api_1.api)("RF_Movimientos/InicioSalidas")
                .parametros({
                "empgrupo_cod": this._model.Grupo,
                "emp_cod": this._model.Empresa,
            })
                .invoca();
            assert.esUn(DatosSalidas, "GAMOVTIP");
            return DatosSalidas;
        }
        async Confirmar() {
            this.actualizaModelo();
            if (this._model.ModeloValido) {
                try {
                    let vObjeto, vMensaje;
                    [vObjeto, vMensaje] = await (0, api_1.api)('/Movimientos/DarSalida')
                        .metodo('put')
                        .parametros({
                        "EMPGRUPO_COD": this._model.Grupo,
                        "EMP_COD": this._model.Empresa,
                        "GAMOVTIP_COD": this._model.TipoMovimiento.COD,
                        "GAART_COD": this._model.Articulo,
                        "GAALMNOM_COD": this._model.DatosOrigen.Almacen,
                        "GAUBIC_COD": this._model.DatosOrigen.Ubicacion,
                        "LOTE": this._model.DatosOrigen.Lote,
                        "C_UNID1": this._model.Cantidad,
                        "C_UNID2": 0
                    })
                        .invoca();
                    (0, notificacion_1.notificacion)().muestra(vMensaje);
                    return true;
                }
                catch ([vMensajeError]) {
                    await (0, mensaje_1.mensaje)().error().muestra(vMensajeError);
                }
            }
            else {
                await (0, mensaje_1.mensaje)().muestra(Resources.PantallaMovimientos.Error.Datos_Faltantes);
            }
            this.actualizaVista();
        }
        async BuscarCodigosAlternativos(Articulo) {
            let vArticulos;
            try {
                [vArticulos] = await (0, api_1.api)('/Articulo/Articulos')
                    .parametros({
                    empgrupo_cod: this._model.Grupo,
                    emp_cod: this._model.Empresa,
                    codigoBarras: Articulo,
                    opcion: "T"
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, mensaje_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            if (!vArticulos)
                return Articulo;
            assert.esUnArrayDe(vArticulos, 'GAART_COD', 'GAART_DESCRIP');
            if (vArticulos.length === 1) {
                return vArticulos[0].GAART_COD;
            }
            else {
                const vArticulo = await (0, lov_1.lov)()
                    .titulo(Resources.PantallaRecepcion.CodigosAlternativos.Lov.Titulo)
                    .columnas([
                    { propiedad: 'GAART_COD', titulo: Resources.PantallaRecepcion.CodigosAlternativos.Lov.Tabla.Codigo, ancho: '6em' },
                    { propiedad: 'GAART_DESCRIP', titulo: Resources.PantallaRecepcion.CodigosAlternativos.Lov.Tabla.Descripcion }
                ])
                    .elige(vArticulos);
                if (typeof vArticulo !== 'undefined')
                    return vArticulo.GAART_COD;
            }
        }
        async establecerCantidad() {
            try {
                if (this._model.DatosOrigen.ControlUbicaciones || this._model.DatosOrigen.ControlLotes) {
                    let v_respuesta = null;
                    let v_valorFiltrado = null;
                    [v_respuesta] = await (0, api_1.api)("/Movimientos/LotesArticuloAlmacen")
                        .metodo('get')
                        .parametros({
                        empgrupo_cod: this._model.Grupo,
                        emp_cod: this._model.Empresa,
                        gaalmnom_cod: this._model.DatosOrigen.Almacen,
                        gaart_cod: this._model.Articulo,
                        opcion: 'ORIGEN'
                    })
                        .invoca();
                    if (v_respuesta) {
                        if (this._model.DatosOrigen.Almacen && this._model.DatosOrigen.Ubicacion && this._model.DatosOrigen.Lote) {
                            v_valorFiltrado = v_respuesta.find((v) => v.GAUBIC_COD === this._model.DatosOrigen.Ubicacion && v.LOTE === this._model.DatosOrigen.Lote);
                            this._model.Cantidad = parseInt(v_valorFiltrado?.C_UNID1) || null;
                            return true;
                        }
                    }
                    else {
                        await (0, notificacion_1.notificacion)().error().muestra(Resources.PantallaMovimientos.Error.Obtencion_cantidad);
                        foco_1.foco.en(this._inputCantidad);
                    }
                }
                else {
                    let v_respuesta = null;
                    [v_respuesta] = await (0, api_1.api)("/Movimientos/Articulo")
                        .metodo('get')
                        .parametros({
                        empgrupo_cod: this._model.Grupo,
                        emp_cod: this._model.Empresa,
                        gaalmnom_cod: this._model.DatosOrigen.Almacen,
                        lecturaarticulo: this._model.Articulo
                    })
                        .invoca();
                    if (v_respuesta) {
                        this._model.Cantidad = v_respuesta?.C_EXIST1 || null;
                    }
                    else {
                        await (0, notificacion_1.notificacion)().error().muestra(Resources.PantallaMovimientos.Error.Obtencion_cantidad);
                        foco_1.foco.en(this._inputCantidad);
                    }
                }
            }
            catch ([p_mensajeError]) {
                await (0, mensaje_1.mensaje)().error().muestra(p_mensajeError);
            }
            return false;
        }
    }
    exports.PantallaSalidas = PantallaSalidas;
});
