define(["require", "exports", "Utilidades/utilidades"], function (require, exports, utilidades_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.plantilla = void 0;
    class Renderizado {
        constructor(_documento) {
            this._documento = _documento;
        }
        documento() {
            return this._documento;
        }
        contenido() {
            return this._documento.body.firstElementChild;
        }
        renombra(p_idPrefijo, p_elementos) {
            for (var v_indice = 0; v_indice < p_elementos.length; v_indice++) {
                let v_elemento = p_elementos[v_indice];
                let v_nuevoId;
                if (v_elemento.id.length > 0)
                    v_nuevoId = `${p_idPrefijo}.${v_elemento.id}`;
                else
                    v_nuevoId = p_idPrefijo;
                v_elemento.id = v_nuevoId;
            }
        }
        corrigeAtributoFor(p_idPrefijo, p_etiquetas) {
            for (var v_indice = 0; v_indice < p_etiquetas.length; v_indice++) {
                let v_etiqueta = p_etiquetas[v_indice];
                if (v_etiqueta instanceof HTMLLabelElement) {
                    let v_nuevoFor;
                    if (v_etiqueta.htmlFor.length > 0)
                        v_nuevoFor = `${p_idPrefijo}.${v_etiqueta.htmlFor}`;
                    else
                        v_nuevoFor = p_idPrefijo;
                    v_etiqueta.htmlFor = v_nuevoFor;
                }
            }
        }
        inserta(p_opciones) {
            let { contenedor: v_contenedor, prefijoId: v_prefijoId } = p_opciones || {};
            if (typeof v_contenedor === 'undefined')
                v_contenedor = document.body;
            if (typeof v_prefijoId !== 'undefined') {
                let v_elementosConId = Array.from(this._documento.querySelectorAll('*[id]'));
                if (this._documento.body.firstElementChild !== null && this._documento.body.firstElementChild.id.length === 0)
                    v_elementosConId = [this._documento.body.firstElementChild].concat(v_elementosConId);
                this.renombra(v_prefijoId, v_elementosConId);
                const v_etiquetasConFor = Array.from(this._documento.querySelectorAll('*[for]'));
                this.corrigeAtributoFor(v_prefijoId, v_etiquetasConFor);
            }
            const v_elemento = this._documento.body.firstElementChild;
            if (v_contenedor.nodeType === Node.COMMENT_NODE)
                v_contenedor.replaceWith(v_elemento);
            else
                v_contenedor.appendChild(v_elemento);
            this._documento = null;
            return new Insertado(v_elemento);
        }
    }
    class Insertado {
        constructor(_elemento) {
            this._elemento = _elemento;
        }
        elemento() {
            return this._elemento;
        }
        suprime() {
            this._elemento.remove();
        }
    }
    class DefinicionPlantilla {
        constructor(_valores) {
            this._valores = _valores;
            this._valores = { nombre: null, ..._valores };
        }
        nombre(p_nombre) {
            return new DefinicionPlantilla({ ...this._valores, nombre: p_nombre });
        }
        secciones(p_secciones) {
            return new DefinicionPlantilla({ ...this._valores, secciones: p_secciones });
        }
        async carga(p_valores) {
            if (typeof this._valores.nombre !== 'string')
                throw new Error('No es un texto');
            function dependencia(p_plantilla) {
                const v_dependencia = `text!${p_plantilla.indexOf('.html') === -1 ? p_plantilla + '.html' : p_plantilla}`;
                return v_dependencia;
            }
            const v_nombre = dependencia(this._valores.nombre);
            const v_nombresSeccion = Object.getOwnPropertyNames(this._valores.secciones || {});
            const v_secciones = v_nombresSeccion.reduce((p_secciones, p_nombre) => {
                p_secciones[p_nombre] = dependencia(this._valores.secciones[p_nombre]);
                return p_secciones;
            }, {});
            let v_dependencias = [v_nombre];
            if (typeof v_secciones !== 'undefined')
                v_dependencias = v_dependencias.concat(v_nombresSeccion
                    .map(p_nombre => v_secciones[p_nombre]));
            let v_promesa = new Promise((p_resuelve, p_rechaza) => require(v_dependencias, function () {
                const v_plantillas = Array.from(arguments);
                const v_textosSeccion = v_nombresSeccion.reduce((p_textosSeccion, p_nombreSeccion) => {
                    const v_plantilla = v_secciones[p_nombreSeccion];
                    const v_indice = v_dependencias.indexOf(v_plantilla);
                    const v_texto = v_plantillas[v_indice];
                    p_textosSeccion[p_nombreSeccion] = v_texto;
                    return p_textosSeccion;
                }, {});
                return p_resuelve({ texto: v_plantillas[0], secciones: v_textosSeccion });
            }));
            const v_resultado = await v_promesa;
            const v_texto = (0, utilidades_1.interpola)(v_resultado.texto, p_valores);
            return this.crea(v_texto, v_resultado.secciones);
        }
        crea(p_texto, p_secciones) {
            return new Plantilla(p_texto, p_secciones);
        }
    }
    class Plantilla {
        constructor(_texto, p_secciones) {
            this._texto = _texto;
            this._secciones = p_secciones || {};
        }
        texto() {
            return this._texto;
        }
        renderiza() {
            const v_opciones = arguments.length === 1 ? arguments[0] : {};
            let { prefijoId: v_prefijoId } = v_opciones;
            if (this._texto === null)
                throw new Error('Plantilla sin texto.');
            let v_expresiones = (this._texto.match(/@(?:[a-z_][a-z_0-9]*(?:(?:\.[a-z_][a-z_0-9]*|\[[0-9]*\]))+|\(.*\))/gmi) || [])
                .map(p_expresion => [p_expresion, new Function(`return ${p_expresion.substring(1)};`)()]);
            let v_texto = v_expresiones.reduce((p_texto, [p_expresion, p_valor]) => p_texto.replace(p_expresion, p_valor), this._texto);
            const v_parser = new DOMParser();
            const v_documento = v_parser.parseFromString(v_texto, 'text/html');
            if (v_documento.body.firstElementChild === null) {
                console.log(v_parser.parseFromString(this._texto, 'text/html'));
                throw new Error('No se permiten plantillas vacías.');
            }
            if (typeof this._secciones !== 'undefined') {
                const v_iterador = v_documento.createNodeIterator(v_documento.body, NodeFilter.SHOW_COMMENT, {
                    acceptNode: p_nodo => p_nodo.nodeValue.trimLeft().startsWith('!') ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_REJECT
                });
                const v_marcadores = [];
                let v_nodo;
                while (v_nodo = v_iterador.nextNode()) {
                    const v_json = v_nodo.nodeValue.trimLeft().slice(1);
                    let v_valor;
                    try {
                        v_valor = eval(`({${v_json}})`);
                        v_marcadores.push({ nodo: v_nodo, valor: v_valor });
                    }
                    catch {
                    }
                }
                v_marcadores.forEach(p_marcador => {
                    if (typeof p_marcador.valor.seccion !== 'string')
                        return;
                    const v_textoSeccion = this._secciones[p_marcador.valor.seccion];
                    if (typeof v_textoSeccion === 'undefined')
                        return;
                    const v_plantillaSeccion = plantilla().crea(this._secciones[p_marcador.valor.seccion]);
                    v_plantillaSeccion.inserta({ contenedor: p_marcador.nodo });
                });
            }
            return new Renderizado(v_documento);
        }
        inserta() {
            const v_opciones = arguments.length === 1 ? arguments[0] : {};
            const v_renderizado = this.renderiza(v_opciones);
            return v_renderizado.inserta(v_opciones);
        }
    }
    function plantilla(p_definicion) {
        let v_definicion = new DefinicionPlantilla();
        if (typeof p_definicion === 'string')
            v_definicion = v_definicion.nombre(p_definicion);
        else if (p_definicion !== null && typeof p_definicion === 'object') {
            if (typeof p_definicion.nombre === 'string')
                v_definicion = v_definicion.nombre(p_definicion.nombre);
            if (p_definicion.secciones !== null && typeof p_definicion.secciones === 'object')
                v_definicion = v_definicion.secciones(p_definicion.secciones);
        }
        return v_definicion;
    }
    exports.plantilla = plantilla;
});
