define(["require", "exports", "componentes/--plantilla", "componentes/dialogo", "componentes/boton", "Utilidades/utilidades"], function (require, exports, __plantilla_1, dialogo_1, boton_1, utilidades_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.pregunta = void 0;
    class DialogoPregunta {
        constructor(p_modal, p_texto, p_error, p_opciones, p_opcionPredeterminada, p_opcionCancelar, p_elementosBotones) {
            this._etiquetaTexto = p_modal.etiqueta('texto').valor(p_texto).crea();
            this._botones = p_opciones.map((p_opcion, p_indice) => {
                let v_boton = (0, boton_1.boton)(p_elementosBotones[p_indice].id)
                    .clic(() => p_modal.cierra([p_indice, p_opcion]));
                switch (p_opcion) {
                    case p_opcionPredeterminada:
                        v_boton = v_boton.tecla('Enter');
                        break;
                    case p_opcionCancelar:
                        v_boton = v_boton.tecla('Escape');
                        break;
                }
                v_boton = v_boton.crea();
                return v_boton;
            });
        }
        actualizaVista() {
        }
    }
    class Pregunta {
        constructor(_texto, _error, _opciones, _opcionPredeterminada, _opcionCancelar, _cancelable) {
            this._texto = _texto;
            this._error = _error;
            this._opciones = _opciones;
            this._opcionPredeterminada = _opcionPredeterminada;
            this._opcionCancelar = _opcionCancelar;
            this._cancelable = _cancelable;
        }
        texto(p_texto) {
            if (typeof p_texto !== 'string')
                throw new Error('No es un texto válido');
            return new Pregunta(p_texto, this._error, this._opciones, this._opcionPredeterminada, this._opcionCancelar, this._cancelable);
        }
        ;
        error() {
            return new Pregunta(this._texto, true, this._opciones, this._opcionPredeterminada, this._opcionCancelar, this._cancelable);
        }
        ;
        opciones(p_opciones) {
            return new Pregunta(this._texto, this._error, p_opciones, this._opcionPredeterminada, this._opcionCancelar, this._cancelable);
        }
        opcionPredeterminada(p_opcion) {
            return new Pregunta(this._texto, this._error, this._opciones, p_opcion, this._opcionCancelar, this._cancelable);
        }
        opcionCancelar(p_opcion) {
            return new Pregunta(this._texto, this._error, this._opciones, this._opcionPredeterminada, p_opcion, this._cancelable);
        }
        cancelable(p_cancelable) {
            return new Pregunta(this._texto, this._error, this._opciones, this._opcionPredeterminada, this._opcionCancelar, p_cancelable);
        }
        async muestra(p_texto) {
            const v_texto = typeof p_texto === 'string' ? p_texto : this._texto;
            const v_nombrePlantilla = "componentes/" + (this._error ? 'pregunta-error.pp.html' : 'pregunta-info.pp.html');
            const [v_contenido, v_elementosBotones] = await Promise
                .all([
                (0, __plantilla_1.plantilla)().nombre(v_nombrePlantilla).carga(),
                (0, __plantilla_1.plantilla)().nombre('componentes/boton-pregunta.pp.html').carga()
            ])
                .then(([p_plantillaPregunta, p_plantillaBoton]) => {
                const v_renderizado = p_plantillaPregunta.renderiza();
                const v_elementoOpciones = v_renderizado.documento().getElementById('opciones');
                if (this._opciones.length > 1)
                    v_elementoOpciones.classList.add('varias-opciones');
                let v_elementosBotones = this._opciones.map((p_opcion, p_indice) => {
                    const v_id = `boton-${p_indice}`;
                    const v_htmlBoton = (0, utilidades_1.interpola)(p_plantillaBoton.texto(), { id: v_id, texto: p_opcion });
                    const v_elementoBoton = (0, __plantilla_1.plantilla)().crea(v_htmlBoton).inserta({ contenedor: v_elementoOpciones }).elemento();
                    return v_elementoBoton;
                });
                return [v_renderizado.contenido(), v_elementosBotones];
            });
            let v_dialogo = (0, dialogo_1.dialogo)()
                .titulo(this._error ? 'Error' : 'Info')
                .elemento(v_contenido)
                .construye(p_modal => new DialogoPregunta(p_modal, v_texto, this._error, this._opciones, this._opcionPredeterminada, this._cancelable ? this._opcionCancelar : null, v_elementosBotones))
                .cancelable(this._cancelable);
            if (this._error)
                v_dialogo = v_dialogo.estilos({ titulo: 'error' });
            const v_resultadoPregunta = await v_dialogo.abre();
            return v_resultadoPregunta;
        }
        ;
    }
    function pregunta(p_texto) {
        if (arguments.length === 0) {
            p_texto = '';
        }
        return new Pregunta(p_texto, false, ['Si', 'No'], 'Si', null, true);
    }
    exports.pregunta = pregunta;
});
