define(["require", "exports", "lib/lodash/lodash", "componentes/componentes", "componentes/foco"], function (require, exports, _, componentes_1, foco_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoLotesUbicaciones = void 0;
    class DialogoLotesUbicaciones {
        constructor(modal, lineaOriginal, disponibleOriginal, datos = {
            MostrarUbicDefDetLotes: 'N',
            PermitirSuperarCantPedido: true
        }) {
            this.modal = modal;
            this.lineaOriginal = lineaOriginal;
            this.disponibleOriginal = disponibleOriginal;
            this.datos = datos;
            this.gaart_cod = lineaOriginal.GAART_COD;
            this.gaart_descrip = lineaOriginal.GAART_DESCRIP;
            this.pendiente = lineaOriginal.C_PDTE;
            this.pendienteOPreparado = 'pendiente';
            this.linea = _.cloneDeep(lineaOriginal);
            this.disponible = _.cloneDeep(disponibleOriginal);
            this.gaubic_cod = this.lote = this.cantidad = null;
            this.tieneControlLotes = this.lineaOriginal.GAART_LOTES_SN === 'S';
            this.tieneControlUbic = this.lineaOriginal.GAART_UBIC_SN === 'S';
            this.actualizaVista();
        }
        inicializaComponentes() {
            let v_columnas = [];
            if (this.tieneControlUbic)
                v_columnas = v_columnas.concat([{ propiedad: 'GAUBIC_COD', titulo: Resources.PantallaExpedicion.DialogoLotesUbicaciones.Tabla.Columnas.Ubicacion, ancho: 'auto' }]);
            if (this.tieneControlLotes)
                v_columnas = v_columnas.concat([{ propiedad: 'LOTE', titulo: Resources.PantallaExpedicion.DialogoLotesUbicaciones.Tabla.Columnas.Lote, ancho: 'auto' }]);
            v_columnas = v_columnas.concat([{ propiedad: 'C_UNI1', titulo: Resources.PantallaExpedicion.DialogoLotesUbicaciones.Tabla.Columnas.Disponible, ancho: '20%', numerico: true }]);
            return {
                _inputGaart_cod: this.modal.campo('gaart_cod'),
                _inputGaart_descrip: this.modal.campo('gaart_descrip'),
                _botonPendienteOPreparado: this.modal.boton('pendiente-o-preparado')
                    .clic(this.muestraPendienteOPreparado.bind(this)),
                _inputPendiente: this.modal.campo('pendiente'),
                _tabla: this.modal.tabla('disponible')
                    .datos(this.disponible)
                    .columnas(v_columnas)
                    .seleccionado(p_seleccion => {
                    this.gaubic_cod = p_seleccion?.GAUBIC_COD || null;
                    this.lote = p_seleccion?.LOTE || null;
                    this.cantidad = Math.min(p_seleccion?.C_UNI1, this.pendiente) || null;
                    this.actualizaVista();
                    this._inputCantidad.foco();
                })
                    .formatoFila((p_formato, p_disponible) => {
                    let v_indice = this.disponible.findIndex(p => _.isEqual(p, p_disponible));
                    if (v_indice === -1)
                        throw new Error('????');
                    if (!_.isEqual(this.disponible[v_indice], this.disponibleOriginal[v_indice]))
                        p_formato.fondo('verde');
                }),
                bloqueEdicion: this.modal.bloque('bloque-edicion'),
                _inputGaubic_cod: this.modal.campo('gaubic_cod')
                    .cambio(p_valor => {
                    let v_lineas = this._tabla.buscaYSelecciona({ GAUBIC_COD: p_valor, LOTE: this._inputLote.valor() }, { seleccionaPrimero: true });
                    if (!Array.isArray(v_lineas) || v_lineas.length === 0) {
                        (0, componentes_1.notificacion)().error().muestra(Resources.PantallaExpedicion.DialogoLotesUbicaciones.Error.Ubicacion_No_Valida);
                        this._inputGaubic_cod.foco();
                    }
                }),
                _inputLote: this.modal.campo('lote')
                    .cambio(p_valor => {
                    let v_lineas = this._tabla.buscaYSelecciona({ GAUBIC_COD: this._inputGaubic_cod.valor(), LOTE: p_valor }, { seleccionaPrimero: true });
                    if (!Array.isArray(v_lineas) || v_lineas.length === 0) {
                        (0, componentes_1.notificacion)().error().muestra(Resources.PantallaExpedicion.DialogoLotesUbicaciones.Error.Lote_No_Valido);
                        this._inputGaubic_cod.foco();
                    }
                }),
                _inputCantidad: this.modal.campo('cantidad')
                    .cambio(p_valor => {
                    let v_gaubic_cod = this._inputGaubic_cod.valor();
                    let v_lote = this._inputLote.valor();
                    this.insertaOActualizaLinea(v_gaubic_cod, v_lote, p_valor);
                })
                    .valida(p_evento => {
                    let v_gaubic_cod = this._inputGaubic_cod.valor();
                    let v_lote = this._inputLote.valor();
                    this.insertaOActualizaLinea(v_gaubic_cod, v_lote, p_evento.nuevoValor);
                }),
                _botonAceptar: this.modal.boton('aceptar')
                    .clic(this.acepta.bind(this))
            };
        }
        puedeAceptar() {
            let v_puedeAceptar = !_.isEqual(this.linea, this.lineaOriginal) || (this.gaubic_cod !== null && this.lote !== null && this.cantidad !== null);
            return v_puedeAceptar;
        }
        async insertaOActualizaLinea(p_gaubic_cod, p_lote, p_cantidad) {
            var v_disponible = this.disponible.find(p_disponible => p_disponible.GAUBIC_COD === p_gaubic_cod && p_disponible.LOTE === p_lote);
            if (!this.datos.PermitirSuperarCantPedido)
                if (this.pendiente - p_cantidad < 0) {
                    await (0, componentes_1.mensaje)().error().muestra(Resources.PantallaExpedicion.DialogoLotesUbicaciones.Error.Excedido_Pendiente_Linea);
                    this._inputCantidad.foco();
                    return false;
                }
            if (v_disponible && v_disponible.C_UNI1 - p_cantidad < 0) {
                await (0, componentes_1.mensaje)().error().muestra(Resources.PantallaExpedicion.DialogoLotesUbicaciones.Error.Excedido_Stock_Articulo);
                this._inputCantidad.foco();
                return false;
            }
            if (typeof v_disponible !== 'undefined') {
                v_disponible.C_UNI1 -= p_cantidad;
            }
            this.pendiente -= p_cantidad;
            this.linea.C_PREP += p_cantidad;
            let v_loteUbicPreparado = this.linea.Detalles.find(p_preparado => p_preparado.GAUBIC_COD === p_gaubic_cod && p_preparado.LOTE === p_lote);
            if (typeof v_loteUbicPreparado !== 'undefined') {
                v_loteUbicPreparado.C_UNI1 += p_cantidad;
            }
            else {
                this.linea.Detalles.push({
                    GAUBIC_COD: p_gaubic_cod,
                    LOTE: p_lote,
                    C_UNI1: p_cantidad
                });
            }
            this.gaubic_cod = this.lote = this.cantidad = null;
            this.actualizaVista();
            return true;
        }
        muestraPendienteOPreparado() {
            if (this.pendienteOPreparado === 'pendiente')
                this.pendienteOPreparado = 'preparado';
            else
                this.pendienteOPreparado = 'pendiente';
            this.actualizaVista();
        }
        actualizaVista() {
            if (typeof this._inputGaart_cod === 'undefined') {
                const v_componentes = this.inicializaComponentes();
                Object.getOwnPropertyNames(v_componentes).forEach(p_nombre => this[p_nombre] = v_componentes[p_nombre].crea());
                if (this.datos.MostrarUbicDefDetLotes === 'S' && this.lineaOriginal.GAART_UBIC_SN === "S" && this.lineaOriginal.GAUBIC_DEF) {
                    this.gaubic_cod = this.lineaOriginal.GAUBIC_DEF;
                    setTimeout(() => {
                        foco_1.foco.enSiguienteA(this._inputGaubic_cod);
                    }, 0);
                }
            }
            this._inputGaubic_cod.visible(this.tieneControlUbic);
            this._inputLote.visible(this.tieneControlLotes);
            this._inputGaart_cod.valor(this.gaart_cod);
            this._inputGaart_descrip.valor(this.gaart_descrip);
            if (this.pendienteOPreparado === 'pendiente') {
                this._botonPendienteOPreparado.etiqueta('Pdte.');
                this._inputPendiente.valor(this.pendiente);
            }
            else {
                this._botonPendienteOPreparado.etiqueta('Prep.');
                this._inputPendiente.valor(this.linea.C_PREP);
            }
            this._tabla.datos(this.disponible);
            this.bloqueEdicion.clase('ubicacion', this.linea.GAART_UBIC_SN === 'S');
            this.bloqueEdicion.clase('lote', this.linea.GAART_LOTES_SN === 'S');
            this._inputGaubic_cod.valor(this.gaubic_cod);
            this._inputLote.valor(this.lote);
            this._inputCantidad.valor(this.cantidad);
            this._botonAceptar.habilitado(this.puedeAceptar());
        }
        actualizaModelo() {
            this.gaubic_cod = this._inputGaubic_cod.valor();
            this.lote = this._inputLote.valor();
            this.cantidad = this._inputCantidad.valor();
        }
        async acepta() {
            this.actualizaModelo();
            let vContinuar = true;
            if (typeof this.gaubic_cod !== 'undefined' && typeof this.lote !== 'undefined' && typeof this.cantidad !== 'undefined')
                vContinuar = await this.insertaOActualizaLinea(this.gaubic_cod, this.lote, this.cantidad);
            if (vContinuar)
                this.modal.cierra({
                    disponible: this.disponible,
                    preparado: {
                        C_UNI1: this.linea.C_PREP,
                        Detalles: this.linea.Detalles
                    }
                });
        }
    }
    exports.DialogoLotesUbicaciones = DialogoLotesUbicaciones;
});
