define(["require", "exports", "componentes/api", "componentes/componentes", "expedicion/dialogos/js/dialogo-consulta-bultos"], function (require, exports, api_1, componentes_1, dialogo_consulta_bultos_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoResumenBultos = void 0;
    class DialogoResumenBultos {
        constructor(modal, p_resumenBultos, p_otros) {
            this.modal = modal;
            this.resumenBultos = { N_COPIAS: null, ..._.cloneDeep(p_resumenBultos) };
            const { mostrarAlbaranValorado: v_mostrarAlbaranValorado, mostrarCopias: v_mostrarCopias, detalleBultos: v_detalleBultos, fechaAlbaran: v_fechaAlbaran } = p_otros || {};
            this.bultos = v_detalleBultos;
            this._botonConfirmar = modal.boton('confirmar')
                .clic(this.confirma.bind(this))
                .crea();
            this._botonImprimir = modal.boton('imprimir')
                .clic(this.imprime.bind(this))
                .tecla('Enter')
                .crea();
            this._inputGgmenvio_cod = modal.campo('ggmenvio_cod')
                .cambio(this.validaGgmenvio_cod.bind(this))
                .lov(this.pideGgmenvio_cod.bind(this))
                .crea();
            this._inputGgmenvio_descrip = modal.campo('ggmenvio_descrip')
                .crea();
            this._botonBusquedaGgmenvio_cod = modal.boton('busqueda-ggmenvio_cod')
                .clic(this.pideGgmenvio_cod.bind(this))
                .crea();
            this._inputGgvehiculo_cod = modal.campo('ggvehiculo_cod')
                .cambio(this.validaGgvehiculo_cod.bind(this))
                .lov(this.pideGgvehiculo_cod.bind(this))
                .crea();
            this._inputGgvehiculo_descrip = modal.campo('ggvehiculo_descrip')
                .crea();
            this._botonBusquedaGgvehiculo_cod = modal.boton('busqueda-ggvehiculo_cod')
                .clic(this.pideGgvehiculo_cod.bind(this))
                .crea();
            this._botonConsultaBultos = modal.boton('consulta-bultos')
                .clic(this.consultaBultos.bind(this))
                .crea();
            this._labelEtiquetaBultos = modal.etiqueta('etiqueta-bultos')
                .crea();
            this._inputBultos = modal.campo('bultos').crea();
            this._inputPeso = modal.campo('peso').crea();
            this._inputEtiquetas = modal.campo('etiquetas').crea();
            this._desplegablePlantillas = modal.componente('plantilla')
                .valores(p_resumenBultos.Opciones)
                .crea();
            this._inputCopias = modal.campo('copias').crea();
            this._inputCopias.visible(v_mostrarCopias);
            this._inputAlbaranValorado = modal.campo('albaran-valorado').crea();
            this._inputAlbaranValorado.visible(v_mostrarAlbaranValorado);
        }
        actualizaVista() {
            this._inputGgmenvio_cod.valor(this.resumenBultos.GGMENVIO_COD);
            this._inputGgmenvio_descrip.valor(this.resumenBultos.GGMENVIO_DES);
            this._inputGgvehiculo_cod.valor(this.resumenBultos.GGVEHICULO_COD);
            this._inputGgvehiculo_descrip.valor(this.resumenBultos.GGVEHICULO_DES);
            this._inputBultos.valor(this.resumenBultos.N_BULTOS);
            this._inputPeso.valor(this.resumenBultos.PESO);
            this._inputEtiquetas.valor(this.resumenBultos.N_ETIQUETAS);
            this._desplegablePlantillas.valor(this.resumenBultos.OpcionPredeterminada);
            this._inputCopias.valor(this.resumenBultos.N_COPIAS);
            this._inputAlbaranValorado.valor(this.resumenBultos.AlbaranValorado);
            this._botonConsultaBultos.visible(!!this.bultos);
            this._labelEtiquetaBultos.visible(!this.bultos);
        }
        actualizaModelo() {
            this.resumenBultos.GGMENVIO_COD = this._inputGgmenvio_cod.valor().toUpperCase();
            this.resumenBultos.GGVEHICULO_COD = this._inputGgvehiculo_cod.valor().toUpperCase();
            this.resumenBultos.N_BULTOS = this._inputBultos.valor();
            this.resumenBultos.PESO = this._inputPeso.valor();
            this.resumenBultos.N_ETIQUETAS = this._inputEtiquetas.valor();
            this.resumenBultos.OpcionPredeterminada = this._desplegablePlantillas.valor();
            this.resumenBultos.N_COPIAS = this._inputCopias.valor();
            this.resumenBultos.AlbaranValorado = this._inputAlbaranValorado.valor();
        }
        ;
        async confirma() {
            this.actualizaModelo();
            this.modal.cierra([this.resumenBultos, { confirma: true, imprimeEtiquetas: false }]);
        }
        imprime() {
            this.actualizaModelo();
            this.modal.cierra([this.resumenBultos, { confirma: true, imprimeEtiquetas: true }]);
        }
        async validaGgmenvio_cod() {
            this.actualizaModelo();
            let v_modoEnvio;
            try {
                [v_modoEnvio] = await (0, api_1.api)('/Expedicion/ModoEnvio')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    ggmenvio_cod: this.resumenBultos.GGMENVIO_COD
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            this.resumenBultos.GGMENVIO_COD = v_modoEnvio.COD;
            this.resumenBultos.GGMENVIO_DES = v_modoEnvio.DESCRIP;
            this.resumenBultos.GGVEHICULO_COD = this.resumenBultos.GGVEHICULO_DES = null;
            this.actualizaVista();
        }
        async pideGgmenvio_cod() {
            let v_modosEnvio;
            try {
                [v_modosEnvio] = await (0, api_1.api)('/Expedicion/ModosEnvio')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD")
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            const v_elegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaExpedicion.DialogoResumenBultos.Lov.Transportistas.Titulo)
                .columnas([
                { propiedad: 'COD', titulo: Resources.PantallaExpedicion.DialogoResumenBultos.Lov.Transportistas.Tabla.Columnas.Codigo, ancho: '6em' },
                { propiedad: 'DESCRIP', titulo: Resources.PantallaExpedicion.DialogoResumenBultos.Lov.Transportistas.Tabla.Columnas.Nombre }
            ])
                .elige(v_modosEnvio);
            if (typeof v_elegido !== 'undefined') {
                this.resumenBultos.GGMENVIO_COD = v_elegido.COD;
                this.resumenBultos.GGMENVIO_DES = v_elegido.DESCRIP;
                this.resumenBultos.GGVEHICULO_COD = this.resumenBultos.GGVEHICULO_DES = null;
                this.actualizaVista();
                return true;
            }
        }
        async validaGgvehiculo_cod() {
            this.actualizaModelo();
            var v_vehiculo;
            try {
                [v_vehiculo] = await (0, api_1.api)('/Expedicion/Vehiculo')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    ggmenvio_cod: this.resumenBultos.GGMENVIO_COD,
                    cod: this.resumenBultos.GGVEHICULO_COD
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            this.resumenBultos.GGVEHICULO_COD = v_vehiculo.COD;
            this.resumenBultos.GGVEHICULO_DES = v_vehiculo.DESCRIP;
            this.actualizaVista();
        }
        async pideGgvehiculo_cod() {
            let v_vehiculos;
            try {
                [v_vehiculos] = await (0, api_1.api)('/Expedicion/Vehiculos')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    ggmenvio_cod: this.resumenBultos.GGMENVIO_COD
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            let v_elegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaExpedicion.DialogoResumenBultos.Lov.Transportistas.Titulo)
                .columnas([
                { propiedad: 'COD', titulo: Resources.PantallaExpedicion.DialogoResumenBultos.Lov.Transportistas.Tabla.Columnas.Codigo, ancho: '6em' },
                { propiedad: 'DESCRIP', titulo: Resources.PantallaExpedicion.DialogoResumenBultos.Lov.Transportistas.Tabla.Columnas.Nombre }
            ])
                .elige(v_vehiculos);
            if (typeof v_elegido !== 'undefined') {
                this.resumenBultos.GGVEHICULO_COD = v_elegido.COD;
                this.resumenBultos.GGVEHICULO_DES = v_elegido.DESCRIP;
                this.actualizaVista();
                return true;
            }
        }
        async consultaBultos() {
            await (0, componentes_1.dialogo)()
                .titulo(Resources.PantallaExpedicion.DialogoConsultaBultos.Titulo)
                .plantilla('expedicion/dialogos/plantillas/dialogo-consulta-bultos')
                .construye(p_modal => new dialogo_consulta_bultos_1.DialogoConsultaBultos(p_modal, this.bultos))
                .abre();
        }
    }
    exports.DialogoResumenBultos = DialogoResumenBultos;
});
