define(["require", "exports", "componentes/api", "componentes/componentes", "lib/moment/moment"], function (require, exports, api_1, componentes_1, moment) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoSeleccionAlbaran = void 0;
    class DialogoSeleccionAlbaran {
        constructor(p_modal, p_albaranes) {
            this.modal = p_modal;
            this.albaranes = p_albaranes.map(p_albaran => ({ ...p_albaran, F_ALBARAN: moment(p_albaran.F_ALBARAN).format("DD/MM/YY") }));
            this.lecturaAlbaran = null;
            this.ggalbtip_cod = this.n_albaran = this.f_albaran = null;
            this._inputAlbaran = p_modal.campo('lecturaAlbaran')
                .cambio(this.validaAlbaran.bind(this))
                .crea();
            this._tabla = p_modal.tabla('albaranes')
                .datos(this.albaranes)
                .columnas([
                { propiedad: 'GGALBTIP_COD', titulo: Resources.PantallaExpedicion.DialogoSeleccionAlbaran.Tabla.Columna.Tipo_Albaran, ancho: 'auto' },
                { propiedad: 'N_ALBARAN', titulo: Resources.PantallaExpedicion.DialogoSeleccionAlbaran.Tabla.Columna.N_Albaran, ancho: 'auto', numerico: true },
                { propiedad: 'F_ALBARAN', titulo: Resources.PantallaExpedicion.DialogoSeleccionAlbaran.Tabla.Columna.Fecha, ancho: 'auto', alineacion: 'centro' },
                { propiedad: 'GGESTADO_COD', titulo: Resources.PantallaExpedicion.DialogoSeleccionAlbaran.Tabla.Columna.Estado, ancho: 'auto', alineacion: 'centro' },
            ])
                .seleccionado(p_seleccion => {
                this.ggalbtip_cod = p_seleccion?.GGALBTIP_COD || null;
                this.n_albaran = p_seleccion?.N_ALBARAN || null;
                this.f_albaran = p_seleccion?.F_ALBARAN || null;
                this.actualizaVista();
            })
                .crea();
            this._botonNuevo = p_modal.boton('nuevo')
                .clic(this.nuevo.bind(this))
                .crea();
            this._botonSeleccionar = p_modal.boton('seleccionar')
                .clic(this.seleccionar.bind(this))
                .tecla('Enter')
                .crea();
        }
        actualizaVista() {
            this._inputAlbaran.valor(this.lecturaAlbaran);
            this._botonSeleccionar.habilitado(this.ggalbtip_cod !== null && this.n_albaran !== null && this.f_albaran !== null);
        }
        actualizaModelo() {
            this.lecturaAlbaran = this._inputAlbaran.valor();
        }
        async validaAlbaran(p_valor) {
            if (typeof p_valor !== 'string' || p_valor.length === 0)
                return;
            var v_albaran;
            try {
                [v_albaran] = await (0, api_1.api)('/Expedicion/NumeroAlbaran')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    lectura_albaran: p_valor.toUpperCase()
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
        }
        nuevo() {
            this.ggalbtip_cod = this.n_albaran = this.f_albaran = null;
            this.modal.cierra('nuevo');
        }
        seleccionar() {
            this.actualizaModelo();
            this.modal.cierra({
                GGALBTIP_COD: this.ggalbtip_cod,
                N_ALBARAN: this.n_albaran,
                F_ALBARAN: this.f_albaran
            });
        }
    }
    exports.DialogoSeleccionAlbaran = DialogoSeleccionAlbaran;
});
