define(["require", "exports", "Utilidades/assert", "lib/lodash/lodash", "componentes/componentes", "componentes/api", "Recepcion/dialogos/modelos/dialogo-lotes-ubicaciones-traspasos-modelo", "componentes/foco"], function (require, exports, assert, _, componentes_1, api_1, dialogo_lotes_ubicaciones_traspasos_modelo_1, foco_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoLotesUbicacionesTraspasos = void 0;
    class DialogoLotesUbicacionesTraspasos {
        constructor(init) {
            this._modelo = new dialogo_lotes_ubicaciones_traspasos_modelo_1.LotesUbicacionesTraspasosPantallaModelo({ Linea: init.Linea, Pedido: init.Pedido });
            this._modal = init.modal;
            this._lineaPreparada = new dialogo_lotes_ubicaciones_traspasos_modelo_1.DetallesLineaPreparada();
            this.ConfiguraPantalla();
        }
        ConfiguraPantalla() {
            const columnas = [];
            if (this._modelo.ControlLotes) {
                columnas.push({ propiedad: 'LOTE', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.tabla.columna.Lote, ancho: '6em' });
            }
            else {
                this._modelo.LOTE = '*';
            }
            columnas.push({
                propiedad: 'C_UNI1',
                titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.tabla.columna.C_UNI1,
                ancho: '4em',
                alineacion: 'centro'
            });
            this._tabla = this._modal.tabla('disponible')
                .datos(this._modelo.Linea.Existencias)
                .columnas(columnas)
                .seleccionado(pSeleccion => {
                this._modelo.LOTE = pSeleccion?.LOTE || null;
                this._modelo.C_UNI1 = parseInt(_.toString(pSeleccion?.C_UNI1)) >= this._modelo.Linea.C_PDTE ? this._modelo.Linea.C_PDTE : pSeleccion?.C_UNI1;
                this.actualizaVista();
                if (this._modelo.ControlLotes) {
                    foco_1.foco.en(this._inputCantidad);
                }
            })
                .formatoFila((p_formato, p_disponible) => {
                let v_indice = this._modelo.Linea.Existencias.findIndex(p => _.isEqual(p, p_disponible));
                if (!_.isEqual(v_indice, -1)) {
                    let v_cantidadAlteradaExistencias = this._modelo.Linea.Existencias[v_indice].C_UNI1.toString();
                    let v_cantidadOriginalExistencias = this._modelo.LineaOriginal.Existencias[v_indice].C_UNI1.toString();
                    if (!_.isEqual(v_cantidadAlteradaExistencias, v_cantidadOriginalExistencias))
                        p_formato.fondo('verde');
                }
            })
                .crea();
            this._inputGaartCod = this._modal.campo('gaart_cod').crea();
            this._inputGaartDescrip = this._modal.campo('gaart_descrip').crea();
            this._inputPendiente = this._modal.campo('pendiente').crea();
            this.bloqueEdicion = this._modal.bloque('bloque-edicion').crea();
            this._inputLoteOrigen = this._modal.campo('loteOrigen')
                .cambio(p_valor => {
                let v_lineas = this._tabla.busca({ LOTE: p_valor.toString().toUpperCase() });
                if (!Array.isArray(v_lineas) || v_lineas.length === 0) {
                    (0, componentes_1.notificacion)().error().muestra(Resources.PantallaRecepcion.DialogoLotesUbicaciones.Error.Lote_No_Valido);
                    this._inputLoteOrigen.foco();
                }
                else {
                    this._tabla.buscaYSelecciona({ LOTE: p_valor.toString().toUpperCase() }, { seleccionaPrimero: true });
                    this._inputCantidad.foco();
                }
            })
                .crea();
            this._inputLoteOrigen.visible(this._modelo.ControlLotes);
            this._inputLoteOrigen.habilitado(this._modelo.ControlLotes);
            this._inputLoteDestino = this._modal.campo('loteDestino').crea();
            this._inputCantidad = this._modal.campo('cantidad')
                .valida(p_valor => {
                this._modelo.C_UNI1 = p_valor?.nuevoValor ? p_valor?.nuevoValor : 0;
                foco_1.foco.en(this._inputLoteDestino);
            })
                .crea();
            this._botonConsultaUbicacionDestino = this._modal.boton('consultarUbicaciones')
                .clic(this.ObtenerLotesArticuloAlmacen.bind(this))
                .crea();
            this._inputUbicDestino = this._modal.campo('ubicDestino')
                .valida(async (p_valor) => {
                const validado = await this.validarUbicacionesAlmacen(p_valor);
                if (validado) {
                    const v_loteOrigen = this._inputLoteOrigen.valor();
                    const v_cantidad = this._inputCantidad.valor();
                    this.actualizaModelo();
                    await this.actualizaLinea(v_loteOrigen, v_cantidad);
                }
            })
                .crea();
            this._inputUbicDestino.visible(this._modelo.ControlUbicaciones);
            this._inputUbicDestino.habilitado(this._modelo.ControlUbicaciones);
            this._botonAceptar = this._modal.boton('aceptar')
                .clic(this.acepta.bind(this))
                .crea();
            this.gestionaUbicacionXDefecto();
            this.esLoteInicial = true;
            this.loteInicial = {};
            if (this._modelo.Linea.Existencias.length === 1 && this._modelo.LOTE) {
                const v_loteOrigenClassList = this._inputLoteOrigen.obtenElemento().classList;
                v_loteOrigenClassList.forEach((className) => {
                    if (className.includes('foco-inicial')) {
                        this._inputLoteOrigen.obtenElemento().classList.remove('foco-inicial');
                    }
                });
                this._inputCantidad.obtenElemento().classList.add('foco-inicial');
                this._tabla.buscaYSelecciona({ LOTE: this._modelo.LOTE }, { seleccionaPrimero: true });
            }
        }
        async validarUbicacionesAlmacen(gaubic_cod) {
            let respuesta = null;
            try {
                [respuesta] = await (0, api_1.api)('/Recepcion/ValidaUbicacionAlmacen')
                    .metodo('get')
                    .parametros({
                    "empgrupo_cod": sessionStorage.getItem("EMPRGRUP_COD"),
                    "emp_cod": sessionStorage.getItem("EMPR_COD"),
                    "gaalmnom_cod": this._modelo.Linea.GAALMNOM_COD,
                    "gaubic_cod": gaubic_cod?.nuevoValor ? gaubic_cod.nuevoValor : null
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                this._inputUbicDestino.foco();
                return false;
            }
            return true;
        }
        async ObtenerLotesArticuloAlmacen() {
            let valor = null;
            try {
                [valor] = await (0, api_1.api)('/Movimientos/LotesArticuloAlmacen')
                    .metodo('get')
                    .parametros({
                    "empgrupo_cod": sessionStorage.getItem("EMPRGRUP_COD"),
                    "emp_cod": sessionStorage.getItem("EMPR_COD"),
                    "gaalmnom_cod": this._modelo.Linea.GAALMNOM_COD,
                    "gaart_cod": this._modelo.Linea.GAART_COD,
                    "opcion": 'DESTINO'
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUnArrayDe(valor, 'GAUBIC_COD', 'LOTE', 'C_UNID1');
            const v_loteArticuloSeleccionado = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaRecepcion.DialogoLotesUbicaciones.Lov.AlmacenDestino.Titulo)
                .columnas([
                { propiedad: 'GAUBIC_COD', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.Lov.AlmacenDestino.Tabla.GAUBIC_COD, ancho: '6em' },
                { propiedad: 'LOTE', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.Lov.AlmacenDestino.Tabla.Lote },
                { propiedad: 'C_UNID1', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.Lov.AlmacenDestino.Tabla.C_UNID1 }
            ])
                .elige(valor);
            this._inputUbicDestino.valor(v_loteArticuloSeleccionado?.GAUBIC_COD ? v_loteArticuloSeleccionado.GAUBIC_COD : '');
            this._inputUbicDestino.foco();
            return valor;
        }
        gestionaUbicacionXDefecto() {
            if (this._modelo.Pedido.MostrarUbicDefDetLotes === 'S' && this._modelo.ControlUbicaciones && this._modelo.Linea.GAUBIC_DEF) {
                this._modelo.GAUBIC_DES = this._modelo.Linea.GAUBIC_DEF;
            }
            else {
                this._modelo.GAUBIC_DES = null;
            }
        }
        puedeAceptar() {
            const vPuedeAceptar = this._lineaPreparada.Detalles.length > 0 || this._modelo.Linea.Detalles?.length > 0;
            return vPuedeAceptar;
        }
        async actualizaLinea(p_loteOrigen, p_cantidad) {
            if (p_loteOrigen && p_cantidad) {
                const vDisponible = this._modelo.Linea.Existencias.find(pDisponible => pDisponible.LOTE === p_loteOrigen);
                if (!this.esLoteInicial) {
                    if (p_loteOrigen !== this.loteInicial.LOTE) {
                        if (vDisponible.PR_PRECIO !== this.loteInicial.PR_PRECIO && this._modelo.Linea.C_PDTE > 0) {
                            await (0, componentes_1.mensaje)().error().muestra(Resources.PantallaRecepcion.DialogoLotesUbicaciones.Error.Diferente_Precio_Lote);
                            this._inputLoteOrigen.foco();
                            return false;
                        }
                    }
                }
                if (this._modelo.Linea.C_PDTE - p_cantidad < 0) {
                    await (0, componentes_1.mensaje)().error().muestra(Resources.PantallaExpedicion.DialogoLotesUbicaciones.Error.Excedido_Pendiente_Linea);
                    this._inputCantidad.foco();
                    return false;
                }
                if (vDisponible) {
                    if (parseInt(_.toString(vDisponible?.C_UNI1)) - p_cantidad < 0) {
                        await (0, componentes_1.mensaje)().error().muestra(Resources.PantallaRecepcion.DialogoLotesUbicaciones.Error.Excedido_Stock_Articulo);
                        this._inputCantidad.foco();
                        return false;
                    }
                    if (this.esLoteInicial) {
                        Object.assign(this.loteInicial, this.loteInicial, {
                            LOTE: vDisponible.LOTE,
                            PR_PRECIO: vDisponible.PR_PRECIO
                        });
                        this.esLoteInicial = false;
                    }
                }
                this._tabla.refresca();
                const vLoteUbicPreparado = this._lineaPreparada.Detalles.find(pPreparado => pPreparado.LOTE === p_loteOrigen && pPreparado.LOTE_DES === this._modelo.LOTE_DES && pPreparado.GAUBIC_DES === this._modelo.GAUBIC_DES);
                if (vLoteUbicPreparado) {
                    vDisponible.C_UNI1 = +vDisponible.C_UNI1 - +p_cantidad;
                    this._modelo.Linea.C_PDTE -= p_cantidad;
                    this._lineaPreparada.C_PREP += p_cantidad;
                    vLoteUbicPreparado.C_UNI1 += p_cantidad;
                    this._modelo.LOTE_DES = null;
                    if (this._modelo.Linea.C_PDTE > 0 && this._modelo.Linea.Existencias.length === 1) {
                        this._tabla.buscaYSelecciona({ LOTE: this._modelo.LOTE }, { seleccionaPrimero: true });
                    }
                    else {
                        this.btnAceptarOInputLote();
                    }
                }
                else {
                    const lineaPreparadaLocal = this._lineaPreparada.Detalles.find(pPreparado => pPreparado.LOTE === p_loteOrigen && pPreparado.LOTE_DES !== this._modelo.LOTE_DES);
                    const lineaPreparadaAnterior = this._modelo.Linea.Detalles?.find(pPreparado => pPreparado.LOTE === p_loteOrigen && pPreparado.LOTE_DES !== this._modelo.LOTE_DES);
                    if (!lineaPreparadaLocal && !lineaPreparadaAnterior) {
                        vDisponible.C_UNI1 = +vDisponible.C_UNI1 - +p_cantidad;
                        this._modelo.Linea.C_PDTE -= p_cantidad;
                        this._lineaPreparada.C_PREP += p_cantidad;
                        this._lineaPreparada.Detalles.push(new dialogo_lotes_ubicaciones_traspasos_modelo_1.Detalle({
                            LOTE: p_loteOrigen,
                            GAUBIC_COD: vDisponible.GAUBIC_COD,
                            LOTE_DES: this._modelo.LOTE_DES,
                            GAUBIC_DES: this._modelo.GAUBIC_DES,
                            C_UNI1: p_cantidad
                        }));
                        this.gestionaUbicacionXDefecto();
                        this._modelo.LOTE_DES = null;
                        if (this._modelo.Linea.C_PDTE > 0 && this._modelo.Linea.Existencias.length === 1) {
                            this._tabla.buscaYSelecciona({ LOTE: this._modelo.LOTE }, { seleccionaPrimero: true });
                        }
                        else {
                            this.btnAceptarOInputLote();
                        }
                        return true;
                    }
                    else {
                        const v_respuestaPregunta = await (0, componentes_1.pregunta)()
                            .opciones(['Si', 'No'])
                            .opcionPredeterminada('Si')
                            .muestra(Resources.PantallaRecepcion.DialogoLotesUbicaciones.pregunta.ProhibidoDosLotes);
                        if (typeof v_respuestaPregunta !== 'undefined') {
                            let [v_indiceRespuestaPregunta] = v_respuestaPregunta;
                            if (v_indiceRespuestaPregunta === 0) {
                                vDisponible.C_UNI1 = +vDisponible.C_UNI1 - +p_cantidad;
                                this._modelo.Linea.C_PDTE -= p_cantidad;
                                this._lineaPreparada.C_PREP += p_cantidad;
                                const recuperadaLineaLocal = this._lineaPreparada.Detalles.find(pPreparado => pPreparado.LOTE === p_loteOrigen);
                                if (!recuperadaLineaLocal) {
                                    const recuperadaLineaAnterior = this._modelo.Linea.Detalles.find(pPreparado => pPreparado.LOTE === p_loteOrigen);
                                    recuperadaLineaAnterior.C_UNI1 += p_cantidad;
                                }
                                else {
                                    recuperadaLineaLocal.C_UNI1 += p_cantidad;
                                }
                                this.gestionaUbicacionXDefecto();
                                this._modelo.LOTE_DES = null;
                                if (this._modelo.Linea.C_PDTE > 0 && this._modelo.Linea.Existencias.length === 1) {
                                    this._tabla.buscaYSelecciona({ LOTE: this._modelo.LOTE }, { seleccionaPrimero: true });
                                }
                                else {
                                    this.btnAceptarOInputLote();
                                }
                                return true;
                            }
                            else {
                                this._modelo.LOTE_DES = null;
                                if (this._modelo.Linea.C_PDTE > 0 && this._modelo.Linea.Existencias.length === 1) {
                                    this._tabla.buscaYSelecciona({ LOTE: this._modelo.LOTE }, { seleccionaPrimero: true });
                                }
                                else {
                                    this.btnAceptarOInputLote();
                                }
                            }
                        }
                    }
                }
            }
            return false;
        }
        btnAceptarOInputLote() {
            this._modelo.LOTE = this._modelo.C_UNI1 = null;
            if (this._modelo.Pedido.MostrarUbicDefDetLotes === 'S') {
                this._modelo.GAUBIC_DES = this._modelo.Linea.GAUBIC_DEF || null;
            }
            this.actualizaVista();
            if (this._modelo.Linea.C_PDTE === 0) {
                foco_1.foco.en(this._botonAceptar);
            }
            else {
                foco_1.foco.en(this._inputLoteOrigen);
            }
        }
        actualizaVista() {
            this._inputGaartCod.valor(this._modelo.Linea.GAART_COD);
            this._inputGaartDescrip.valor(this._modelo.Linea.GAART_DESCRIP);
            this._inputPendiente.valor(this._modelo.Linea.C_PDTE);
            this._tabla.datos(null);
            this._tabla.datos(this._modelo.Linea.Existencias);
            this.bloqueEdicion.clase('ubicDestino', this._modelo.Linea.GAART_UBIC_SN === 'S');
            this.bloqueEdicion.clase('lote', this._modelo.Linea.GAART_LOTES_SN === 'S');
            this._inputLoteOrigen.valor(this._modelo.LOTE);
            this._inputCantidad.valor(this._modelo.C_UNI1);
            this._inputLoteDestino.valor(this._modelo.LOTE_DES);
            this._inputUbicDestino.valor(this._modelo.GAUBIC_DES);
            this._botonAceptar.habilitado(this.puedeAceptar());
        }
        actualizaModelo() {
            this._modelo.LOTE = this._inputLoteOrigen.valor();
            this._modelo.C_UNI1 = this._inputCantidad.valor();
            this._modelo.LOTE_DES = this._inputLoteDestino.valor();
            this._modelo.GAUBIC_DES = this._inputUbicDestino.valor();
        }
        async acepta() {
            this.actualizaModelo();
            let vContinuar = true;
            if (vContinuar)
                this._modal.cierra(new dialogo_lotes_ubicaciones_traspasos_modelo_1.LineaPreparada({
                    Disponible: this._modelo.Linea.Existencias,
                    Preparado: this._lineaPreparada
                }));
        }
    }
    exports.DialogoLotesUbicacionesTraspasos = DialogoLotesUbicacionesTraspasos;
});
