define(["require", "exports", "componentes/api", "componentes/componentes", "Recepcion/modelos/albaran"], function (require, exports, api_1, componentes_1, albaran_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoSeleccionAlbaran = void 0;
    class DialogoSeleccionAlbaran {
        constructor(model) {
            this.modal = model.modal;
            this.lecturaAlbaran = null;
            this._inputAlbaran = this.modal.campo('lecturaAlbaran')
                .cambio(this.validaAlbaran.bind(this))
                .crea();
            this._tabla = this.modal.tabla('albaranes')
                .datos(model.albaranes)
                .columnas([
                { propiedad: 'GGALBTIP_COD', titulo: Resources.PantallaRecepcion.DialogoSeleccionAlbaran.tabla.columna.GGALBTIP_COD, ancho: 'auto' },
                { propiedad: 'N_ALBARAN', titulo: Resources.PantallaRecepcion.DialogoSeleccionAlbaran.tabla.columna.N_ALBARAN, ancho: 'auto', numerico: true },
                { propiedad: 'F_ALBARAN', titulo: Resources.PantallaRecepcion.DialogoSeleccionAlbaran.tabla.columna.F_ALBARAN, ancho: 'auto', alineacion: 'centro', tipoDato: Date },
                { propiedad: 'GGESTADO_COD', titulo: Resources.PantallaRecepcion.DialogoSeleccionAlbaran.tabla.columna.GGESTADO_COD, ancho: 'auto', alineacion: 'centro' },
            ])
                .seleccionado(pSeleccion => {
                this.AlbaranSeleccionado = pSeleccion;
                this.actualizaVista();
            })
                .crea();
            this._botonNuevo = this.modal.boton('nuevo')
                .clic(this.nuevo.bind(this))
                .crea();
            this._botonSeleccionar = this.modal.boton('seleccionar')
                .clic(this.seleccionar.bind(this))
                .tecla('Enter')
                .crea();
        }
        actualizaVista() {
            this._inputAlbaran.valor(this.lecturaAlbaran);
            let seleccionado = false;
            if (this.AlbaranSeleccionado)
                seleccionado = true;
            this._botonSeleccionar.habilitado(seleccionado);
        }
        actualizaModelo() {
            this.lecturaAlbaran = this._inputAlbaran.valor();
        }
        async validaAlbaran(pValor) {
            if (typeof pValor !== 'string' || pValor.length === 0)
                return;
            var vAlbaran;
            try {
                [vAlbaran] = await (0, api_1.api)('/Expedicion/NumeroAlbaran')
                    .parametros({
                    EMPGRUPO_COD: sessionStorage.getItem("EMPRGRUP_COD"),
                    EMP_COD: sessionStorage.getItem("EMPR_COD"),
                    LECTURA_ALBARAN: pValor.toUpperCase()
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
        }
        nuevo() {
            this.AlbaranSeleccionado = null;
            this.modal.cierra(new albaran_1.Albaran());
        }
        seleccionar() {
            this.actualizaModelo();
            this.modal.cierra(this.AlbaranSeleccionado);
        }
    }
    exports.DialogoSeleccionAlbaran = DialogoSeleccionAlbaran;
});
