define(["require", "exports", "Utilidades/assert", "componentes/api", "componentes/componentes", "componentes/foco", "componentes/campo", "componentes/boton", "componentes/tabla", "componentes/pantalla", "actualiza-albaran/dialogos/js/dialogo-resumen-bultos", "Utilidades/interpola"], function (require, exports, assert, api_1, componentes_1, foco_1, campo_1, boton_1, tabla_1, pantalla_1, dialogo_resumen_bultos_1, interpola_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PantallaActualizaAlbaran = void 0;
    class PantallaActualizaAlbaran extends pantalla_1.Pantalla {
        constructor() {
            super();
            this.Parametros = {};
            this.inicializa();
            this.lecturaAlbaran = null;
            this.albaran = null;
        }
        inicializa() {
            if (typeof this._botonNuevo === 'undefined') {
                const v_componentes = this.inicializaComponentes();
                Object.getOwnPropertyNames(v_componentes).forEach(p_nombre => this[p_nombre] = v_componentes[p_nombre].crea());
            }
        }
        inicializaComponentes() {
            foco_1.foco.enPrimero(document);
            return {
                _botonNuevo: (0, boton_1.boton)('nuevo')
                    .clic(async () => {
                    this.limpia();
                }),
                _botonActivar: (0, boton_1.boton)('activar')
                    .clic(async () => {
                    this.activaAlbaran();
                }),
                _inputAlbaran: (0, campo_1.campo)('albaran')
                    .cambio(async () => {
                    this.cargaAlbaran();
                }),
                _inputFecha: (0, campo_1.campo)('fecha'),
                _inputCliente: (0, campo_1.campo)('cliente'),
                _tablaLineas: (0, tabla_1.tabla)('lineas')
                    .columnas([
                    { propiedad: 'GAART_COD', titulo: Resources.PantallaActualizaAlbaran.Tabla.Columnas.Articulo, ancho: '6em', orden: 'asc' },
                    { propiedad: 'DESCRIP', titulo: Resources.PantallaActualizaAlbaran.Tabla.Columnas.Descripcion, orden: 'asc' },
                    { propiedad: 'C_UNI1', titulo: Resources.PantallaActualizaAlbaran.Tabla.Columnas.Cantidad, ancho: '4em', numerico: true }
                ]).seleccionado(null)
            };
        }
        actualizaVista() {
            this._inputAlbaran.valor(this.lecturaAlbaran);
            this._inputFecha.valor(this.albaran?.FECHA || null);
            this._inputCliente.valor(this.albaran?.GGCLIPR_NOM || null);
            this._tablaLineas.datos(this.albaran?.Lineas || []);
        }
        limpia() {
            this.lecturaAlbaran = null;
            this.albaran = null;
            this.actualizaVista();
            this._inputAlbaran.foco();
        }
        actualizaModelo() {
            this.lecturaAlbaran = this._inputAlbaran.valor().toUpperCase();
        }
        async cargaAlbaran() {
            this.actualizaModelo();
            let vAlbaran;
            try {
                [vAlbaran] = await (0, api_1.api)('/Expedicion/AlbaranPendiente')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem('EMPRGRUP_COD'),
                    emp_cod: sessionStorage.getItem('EMPR_COD'),
                    lecturaalbaran: this.lecturaAlbaran
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)(v_mensaje).error().muestra();
                this.limpia();
                return;
            }
            assert.esUn(vAlbaran);
            this.albaran = vAlbaran;
            this.actualizaVista();
        }
        ;
        async activaAlbaran() {
            this.actualizaModelo();
            let v_resumenBultos;
            try {
                [v_resumenBultos] = await (0, api_1.api)('/Expedicion/ResumenBultosAlbaranPendiente')
                    .metodo('put')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem('EMPRGRUP_COD'),
                    emp_cod: sessionStorage.getItem('EMPR_COD'),
                    ggalbtip_cod: this.albaran?.GGALBTIP_COD,
                    n_albaran: this.albaran?.N_ALBARAN,
                    lineas: this.albaran?.Lineas
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)(v_mensaje).error().muestra();
                this.limpia();
                return;
            }
            let v_resultadoPideResumenBultos = await this.pideResumenBultos(v_resumenBultos);
            if (typeof v_resultadoPideResumenBultos === 'undefined')
                return;
            let v_opcion;
            [v_resumenBultos, v_opcion] = v_resultadoPideResumenBultos;
            const v_opcion1 = Resources.PantallaActualizaAlbaran.Pregunta.Opciones.Si, v_opcion2 = Resources.PantallaActualizaAlbaran.Pregunta.Opciones.No;
            const [p_indice, p_opcion_seleccionada] = await (0, componentes_1.pregunta)()
                .opciones([v_opcion1, v_opcion2])
                .opcionPredeterminada(v_opcion1)
                .muestra((0, interpola_1.interpola)(Resources.PantallaActualizaAlbaran.Pregunta.Cambio_Estado_Albaran, {
                v_fechaActual: this.albaran?.FECHA,
                v_nuevaFecha: moment(new Date()).format("DD/MM/YY")
            }));
            if (typeof p_opcion_seleccionada === 'undefined')
                return;
            if (p_opcion_seleccionada === v_opcion2)
                return;
            this.intentaCambiarEstadoAlbaran(v_resumenBultos, v_opcion);
        }
        ;
        async intentaCambiarEstadoAlbaran(p_resumenBultos, p_opcion) {
            let v_cambiaEstadoAlbaran;
            try {
                [v_cambiaEstadoAlbaran] = await this.cambiaEstadoAlbaran(p_resumenBultos, p_opcion === 'confirmaEImprime');
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)(v_mensaje).error().muestra();
                this.limpia();
                return;
            }
            this.Parametros = v_cambiaEstadoAlbaran.Parametros;
            if (v_cambiaEstadoAlbaran.Parametros && v_cambiaEstadoAlbaran.Parametros.P_PREGUNTA) {
                const v_opcion1 = Resources.PantallaActualizaAlbaran.Pregunta.Opciones.Si, v_opcion2 = Resources.PantallaActualizaAlbaran.Pregunta.Opciones.No;
                let [v_indice, v_opcion_seleccionada] = await (0, componentes_1.pregunta)()
                    .opciones([v_opcion1, v_opcion2])
                    .opcionPredeterminada(v_opcion1)
                    .muestra(v_cambiaEstadoAlbaran.Parametros.P_PREGUNTA);
                if (typeof v_opcion_seleccionada === 'undefined')
                    return;
                if (v_opcion_seleccionada === v_opcion1) {
                    this.Parametros.P_RESPUESTA = 'S';
                }
                else {
                    this.Parametros.P_RESPUESTA = 'N';
                }
                this.intentaCambiarEstadoAlbaran(p_resumenBultos, p_opcion);
            }
            else {
                (0, componentes_1.notificacion)().muestra(Resources.PantallaActualizaAlbaran.Notificacion.Cambio_Estado_Albaran_Realizado);
                this.limpia();
            }
        }
        async cambiaEstadoAlbaran(p_resumenBultos, p_imprimeEtiquetas) {
            let v_resultado = await (0, api_1.api)('/Expedicion/CambiarEstadoAlbaran')
                .metodo('POST')
                .parametros({
                empgrupo_cod: sessionStorage.getItem('EMPRGRUP_COD'),
                emp_cod: sessionStorage.getItem('EMPR_COD'),
                ggalbtip_cod: this.albaran?.GGALBTIP_COD,
                n_albaran: this.albaran?.N_ALBARAN,
                ggmenvio_cod: p_resumenBultos.GGMENVIO_COD,
                ggvehiculo_cod: p_resumenBultos.GGVEHICULO_COD,
                n_bultos: p_resumenBultos.N_BULTOS,
                peso: p_resumenBultos.PESO,
                n_etiquetas: p_resumenBultos.N_ETIQUETAS,
                OpcionEtiquetas: p_resumenBultos.OpcionPredeterminada,
                ImprimirEtiquetas: p_imprimeEtiquetas,
                ImprimirValorado: p_resumenBultos.AlbaranValorado,
                Parametros: this.Parametros,
            })
                .invoca();
            return v_resultado;
        }
        async pideResumenBultos(p_resumenBultos) {
            return await (0, componentes_1.dialogo)()
                .titulo(Resources.PantallaActualizaAlbaran.DialogoResumenBultos.Titulo)
                .plantilla('actualiza-albaran/dialogos/plantillas/dialogo-resumen-bultos')
                .construye(p_modal => new dialogo_resumen_bultos_1.DialogoResumenBultos(p_modal, Object.assign(p_resumenBultos)))
                .abre();
        }
    }
    exports.PantallaActualizaAlbaran = PantallaActualizaAlbaran;
});
