define(["require", "exports", "componentes/boton", "Utilidades/utilidades"], function (require, exports, boton_1, utilidades_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.cortina = exports.Cortina = exports.DefinicionCortina = void 0;
    class DefinicionOpcion {
        constructor(_valores) {
            this._valores = _valores;
        }
        titulo(p_titulo) {
            return new DefinicionOpcion({ ...this._valores, titulo: p_titulo });
        }
        icono(p_icono) {
            return new DefinicionOpcion({ ...this._valores, icono: p_icono });
        }
        valor(p_valor) {
            return new DefinicionOpcion({ ...this._valores, valor: p_valor });
        }
        iconoValor(p_iconoValor) {
            return new DefinicionOpcion({ ...this._valores, iconoValor: p_iconoValor });
        }
        clic(p_funcion) {
            return new DefinicionOpcion({ ...this._valores, clic: p_funcion });
        }
        crea() {
            const v_id = 'cortina-opcion-' + (0, utilidades_1.generaSecuencia)();
            menuElement(this._valores.titulo, this._valores.icono, '#', this._valores.valor, this._valores.iconoValor, v_id);
            return new Opcion(v_id, this._valores);
        }
    }
    class DefinicionOpcionFecha {
        constructor(_valores) {
            this._valores = _valores;
        }
        titulo(p_titulo) {
            return new DefinicionOpcionFecha({ ...this._valores, titulo: p_titulo });
        }
        icono(p_icono) {
            return new DefinicionOpcionFecha({ ...this._valores, icono: p_icono });
        }
        valor(p_valor) {
            return new DefinicionOpcionFecha({ ...this._valores, valor: p_valor });
        }
        cambio(p_funcion) {
            return new DefinicionOpcionFecha({ ...this._valores, cambio: p_funcion });
        }
        crea() {
            const v_id = 'cortina-opcion-' + (0, utilidades_1.generaSecuencia)();
            menuElement(this._valores.titulo, this._valores.icono, '#', this._valores.valor, 'buscador', v_id);
            menuDatePicker(v_id, this._valores.cambio);
            return new OpcionFecha(v_id, this._valores);
        }
    }
    class DefinicionCortina {
        constructor(_valores) {
            this._valores = _valores;
        }
        opcion(...p_argumentos) {
            let v_nombre;
            let v_funcion;
            switch (p_argumentos.length) {
                case 1:
                    [v_funcion] = p_argumentos;
                    break;
                case 2:
                    [v_nombre, v_funcion] = p_argumentos;
                    break;
            }
            const v_opcion = v_funcion(new DefinicionOpcion({ nombre: v_nombre }));
            return new DefinicionCortina({ ...this._valores, opciones: this._valores.opciones.concat(v_opcion) });
        }
        opcionFecha(...p_argumentos) {
            let v_nombre;
            let v_funcion;
            switch (p_argumentos.length) {
                case 1:
                    [v_funcion] = p_argumentos;
                    break;
                case 2:
                    [v_nombre, v_funcion] = p_argumentos;
                    break;
            }
            const v_opcion = v_funcion(new DefinicionOpcionFecha({ nombre: v_nombre }));
            return new DefinicionCortina({ ...this._valores, opciones: this._valores.opciones.concat(v_opcion) });
        }
        crea() {
            $('#nav-user').find('li:not(:last)').remove();
            const v_opciones = this._valores.opciones
                .map(function (p_opcion) { return p_opcion.crea(); })
                .reduce((p_opciones, p_opcion) => {
                const v_nombre = p_opcion.nombre();
                if (typeof v_nombre === 'string') {
                    p_opciones[v_nombre] = p_opcion;
                }
                return p_opciones;
            }, {});
            return new Cortina(v_opciones);
        }
        activa() {
            return this.crea();
        }
    }
    exports.DefinicionCortina = DefinicionCortina;
    class Opcion {
        constructor(_id, _valores) {
            this._id = _id;
            this._valores = _valores;
            this.elemento = document.getElementById(_id);
            this.boton = (0, boton_1.boton)(this.elemento)
                .clic(function (p_evento) { _valores.clic(this, p_evento); })
                .crea();
            this.boton.habilitado(typeof _valores.clic === 'function');
        }
        nombre() { return this._valores.nombre; }
        titulo() { return this._valores.titulo; }
        icono() { return this._valores.icono; }
        valor(p_valor) {
            if (arguments.length === 0) {
                return this._valores.valor;
            }
            else {
                this._valores.valor = p_valor;
                $('#' + this._id + ' div input').val(p_valor);
            }
        }
        iconoValor() { return this._valores.iconoValor; }
        habilita() {
            return this.boton.habilita();
        }
        deshabilita() {
            return this.boton.deshabilita();
        }
        habilitado(p_valor) {
            if (arguments.length === 0) {
                return this.boton.habilitado();
            }
            else {
                this.boton.habilitado(p_valor);
            }
        }
    }
    class OpcionFecha {
        constructor(_id, _valores) {
            this._id = _id;
            this._valores = _valores;
        }
        nombre() { return this._valores.nombre; }
        titulo() { return this._valores.titulo; }
        icono() { return this._valores.icono; }
        valor(p_valor) {
            if (arguments.length === 0) {
                return this._valores.valor;
            }
            else {
                this._valores.valor = p_valor;
                const v_fecha = p_valor ? moment(p_valor, 'DD/MM/YYYY').toDate() : null;
                updateDatePicker(v_fecha, this._id);
            }
        }
    }
    class Cortina {
        constructor(opciones) {
            this.opciones = opciones;
        }
        abre() {
            openMenu();
        }
        cierra() {
            closeMenu();
        }
    }
    exports.Cortina = Cortina;
    function cortina() {
        return new DefinicionCortina({
            opciones: []
        });
    }
    exports.cortina = cortina;
});
