define(["require", "exports", "componentes/api", "componentes/componentes", "lib/lodash/lodash", "Utilidades/assert"], function (require, exports, api_1, componentes_1, _, assert) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoDatosTransporte = void 0;
    class DialogoDatosTransporte {
        constructor(p_modal, p_datosTransporte) {
            this._modal = p_modal;
            this._datosTransporte = _.cloneDeep(p_datosTransporte);
            this._botonAceptar = p_modal.boton('aceptar')
                .clic(this.acepta.bind(this))
                .tecla('Enter')
                .crea();
            this._inputGgmenvio_cod = p_modal.campo('ggmenvio_cod')
                .valida(this.validaGgmenvio_cod.bind(this))
                .lov(this.pideGgmenvio_cod.bind(this))
                .crea();
            this._inputGgmenvio_descrip = p_modal.campo('ggmenvio_descrip')
                .crea();
            this._botonBusquedaGgmenvio_cod = p_modal.boton('busqueda-ggmenvio_cod')
                .clic(this.pideGgmenvio_cod.bind(this))
                .crea();
            this._inputGgvehiculo_cod = p_modal.componente('ggvehiculo_cod')
                .valida(this.validaGgvehiculo_cod.bind(this))
                .lov(this.pideGgvehiculo_cod.bind(this))
                .crea();
            this._inputGgvehiculo_descrip = p_modal.componente('ggvehiculo_descrip')
                .crea();
            this._botonBusquedaGgvehiculo_cod = p_modal.boton('busqueda-ggvehiculo_cod')
                .clic(this.pideGgvehiculo_cod.bind(this))
                .crea();
        }
        actualizaVista() {
            this._inputGgmenvio_cod.valor(this._datosTransporte.GGMENVIO_COD);
            this._inputGgmenvio_descrip.valor(this._datosTransporte.GGMENVIO_DESCRIP);
            this._inputGgvehiculo_cod.valor(this._datosTransporte.GGVEHICULO_COD);
            this._inputGgvehiculo_descrip.valor(this._datosTransporte.GGVEHICULO_DESCRIP);
        }
        actualizaModelo() {
            this._datosTransporte.GGMENVIO_COD = this._inputGgmenvio_cod.valor().toUpperCase();
            this._datosTransporte.GGMENVIO_DESCRIP = this._inputGgmenvio_descrip.valor().toUpperCase();
            this._datosTransporte.GGVEHICULO_COD = this._inputGgvehiculo_cod.valor().toUpperCase();
            this._datosTransporte.GGVEHICULO_DESCRIP = this._inputGgvehiculo_descrip.valor().toUpperCase();
        }
        ;
        acepta() {
            this.actualizaModelo();
            this._modal.cierra(this._datosTransporte);
        }
        async validaGgmenvio_cod({ valorHaCambiado: p_valorHaCambiado }) {
            this.actualizaModelo();
            if (!p_valorHaCambiado)
                return;
            let v_modoEnvio;
            try {
                [v_modoEnvio] = await (0, api_1.api)('/Expedicion/ModoEnvio')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    ggmenvio_cod: this._datosTransporte.GGMENVIO_COD
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            assert.esUn(v_modoEnvio, 'COD', 'DESCRIP');
            this._datosTransporte.GGMENVIO_COD = v_modoEnvio.COD;
            this._datosTransporte.GGMENVIO_DESCRIP = v_modoEnvio.DESCRIP;
            this._datosTransporte.GGVEHICULO_COD = this._datosTransporte.GGVEHICULO_DESCRIP = null;
            this.actualizaVista();
        }
        async pideGgmenvio_cod() {
            let v_modosEnvio;
            try {
                [v_modosEnvio] = await (0, api_1.api)('/Expedicion/ModosEnvio')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD")
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            assert.esUnArrayDe(v_modosEnvio, 'COD', 'DESCRIP');
            let v_elegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaExpedicion.DialogoDatosTransporte.Lov.Transportista.Titulo)
                .columnas([
                { propiedad: 'COD', titulo: Resources.PantallaExpedicion.DialogoDatosTransporte.Lov.Transportista.Tabla.Columnas.Codigo, ancho: '6em' },
                { propiedad: 'DESCRIP', titulo: Resources.PantallaExpedicion.DialogoDatosTransporte.Lov.Transportista.Tabla.Columnas.Nombre }
            ])
                .elige(v_modosEnvio);
            if (typeof v_elegido !== 'undefined') {
                this._datosTransporte.GGMENVIO_COD = v_elegido.COD;
                this._datosTransporte.GGMENVIO_DESCRIP = v_elegido.DESCRIP;
                this._datosTransporte.GGVEHICULO_COD = this._datosTransporte.GGVEHICULO_DESCRIP = null;
                this.actualizaVista();
                return true;
            }
        }
        async validaGgvehiculo_cod({ valorHaCambiado: p_valorHaCambiado }) {
            this.actualizaModelo();
            if (!p_valorHaCambiado)
                return;
            let v_vehiculo;
            try {
                [v_vehiculo] = await (0, api_1.api)('/Expedicion/Vehiculo')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    ggmenvio_cod: this._datosTransporte.GGMENVIO_COD,
                    cod: this._datosTransporte.GGVEHICULO_COD
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            assert.esUn(v_vehiculo, 'COD', 'DESCRIP');
            this._datosTransporte.GGVEHICULO_COD = v_vehiculo.COD;
            this._datosTransporte.GGVEHICULO_DESCRIP = v_vehiculo.DESCRIP;
            this.actualizaVista();
        }
        async pideGgvehiculo_cod() {
            let v_vehiculos;
            try {
                [v_vehiculos] = await (0, api_1.api)('/Expedicion/Vehiculos')
                    .parametros({
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    ggmenvio_cod: this._datosTransporte.GGMENVIO_COD
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            assert.esUnArrayDe(v_vehiculos, 'COD', 'DESCRIP');
            let v_elegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaExpedicion.DialogoDatosTransporte.Lov.Vehiculo.Titulo)
                .columnas([
                { propiedad: 'COD', titulo: Resources.PantallaExpedicion.DialogoDatosTransporte.Lov.Vehiculo.Tabla.Columnas.Codigo, ancho: '6em' },
                { propiedad: 'DESCRIP', titulo: Resources.PantallaExpedicion.DialogoDatosTransporte.Lov.Vehiculo.Tabla.Columnas.Nombre }
            ])
                .elige(v_vehiculos);
            if (typeof v_elegido !== 'undefined') {
                this._datosTransporte.GGVEHICULO_COD = v_elegido.COD;
                this._datosTransporte.GGVEHICULO_DESCRIP = v_elegido.DESCRIP;
                this.actualizaVista();
                return true;
            }
        }
    }
    exports.DialogoDatosTransporte = DialogoDatosTransporte;
});
