define(["require", "exports", "Utilidades/assert", "componentes/api", "componentes/componentes", "expedicion/dialogos/js/dialogo-seleccion-tipo-bulto", "expedicion/dialogos/js/dialogo-detalle-bulto"], function (require, exports, assert, api_1, componentes_1, dialogo_seleccion_tipo_bulto_1, dialogo_detalle_bulto_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoSeleccionBulto = void 0;
    class DialogoSeleccionBulto {
        constructor(_modal, p_bultos, p_cantidad, p_ultimoBultoSeleccionado) {
            this._modal = _modal;
            this.bultos = p_bultos.map(this.mapeaBulto);
            this.cantidad = p_cantidad;
            this.tabla = _modal.tabla('bultos')
                .datos(this.bultos)
                .seleccionado(this.seleccionaBulto.bind(this))
                .columnas([
                { propiedad: 'N_BULTO', titulo: Resources.PantallaExpedicion.DialogoSeleccionBulto.Tabla.Columnas.N_Bulto, ancho: '2em', numerico: true },
                { propiedad: 'GVPALLETIP_COD', titulo: Resources.PantallaExpedicion.DialogoSeleccionBulto.Tabla.Columnas.Tipo_Bulto, ancho: '3em' },
                { propiedad: 'GVPALLETIP_DESCRIP', titulo: Resources.PantallaExpedicion.DialogoSeleccionBulto.Tabla.Columnas.Descripcion, ancho: 'auto' },
                { propiedad: 'CANTIDAD', titulo: Resources.PantallaExpedicion.DialogoSeleccionBulto.Tabla.Columnas.Cantidad, ancho: '6em', numerico: true, },
            ])
                .crea();
            this.botonSeleccionar = _modal.boton('seleccionar')
                .clic(this.seleccionar.bind(this))
                .tecla('Enter')
                .crea();
            this.botonNinguno = _modal.boton('ninguno')
                .clic(this.ninguno.bind(this))
                .crea();
            this.botonNuevo = _modal.boton('nuevo')
                .clic(this.nuevo.bind(this))
                .crea();
            this.botonDetalles = _modal.boton('detalles')
                .clic(this.detalles.bind(this))
                .crea();
            if (typeof p_ultimoBultoSeleccionado !== 'undefined')
                this.seleccionado = this.mapeaBulto(p_ultimoBultoSeleccionado);
        }
        mapeaBulto(p_bulto) {
            return {
                N_BULTO: p_bulto.N_BULTO,
                GVPALLETIP_COD: p_bulto.GVPALLETIP.COD,
                GVPALLETIP_DESCRIP: p_bulto.GVPALLETIP.DESCRIP,
                CANTIDAD: p_bulto.articulos.map(p_articulo => p_articulo.CANTIDAD).reduce((p_total, p_cantidad) => p_total + p_cantidad, 0),
                articulos: p_bulto.articulos
            };
        }
        seleccionaBulto(p_bulto) {
            this.seleccionado = p_bulto;
            this.actualizaVista();
        }
        actualizaVista() {
            this.tabla.datos(this.bultos);
            this.tabla.buscaYSelecciona(this.seleccionado, { seleccionaPrimero: true });
            this.botonSeleccionar.habilitado(typeof this.seleccionado !== 'undefined');
            this.botonDetalles.habilitado(typeof this.seleccionado !== 'undefined');
        }
        seleccionar() {
            let v_seleccionado = this.seleccionado;
            this._modal.cierra({ opcion: 'actualiza', bulto: v_seleccionado });
        }
        ninguno() {
            this._modal.cierra({ opcion: 'ninguno' });
        }
        async nuevo() {
            let v_tiposBulto;
            try {
                [v_tiposBulto] = await (0, api_1.api)('/Expedicion/TiposPale')
                    .parametros({
                    EMPGRUPO_COD: sessionStorage.getItem("EMPRGRUP_COD"),
                    EMP_COD: sessionStorage.getItem("EMPR_COD")
                })
                    .invoca();
            }
            catch ([v_mensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(v_mensaje);
                return;
            }
            assert.esUnArrayDe(v_tiposBulto, 'COD', 'DESCRIP');
            const v_resultadoSeleccionTipoBulto = await (0, componentes_1.dialogo)()
                .titulo(Resources.PantallaExpedicion.DialogoSeleccionTipoBulto.Titulo)
                .plantilla('expedicion/dialogos/plantillas/dialogo-seleccion-tipo-bulto')
                .construye(p_modal => new dialogo_seleccion_tipo_bulto_1.DialogoSeleccionTipoBulto(p_modal, v_tiposBulto, this.cantidad))
                .abre();
            if (typeof v_resultadoSeleccionTipoBulto === 'undefined')
                return;
            assert.esUn(v_resultadoSeleccionTipoBulto, 'numeroBultos', 'cantidadPorBulto', 'GVPALLETIP');
            this._modal.cierra({ opcion: 'nuevo', ...v_resultadoSeleccionTipoBulto });
        }
        async detalles() {
            let v_seleccionado = this.seleccionado;
            await (0, componentes_1.dialogo)()
                .titulo(Resources.PantallaExpedicion.DialogoDetalleBulto.Titulo)
                .plantilla('expedicion/dialogos/plantillas/dialogo-detalle-bulto')
                .construye(p_modal => new dialogo_detalle_bulto_1.DialogoDetalleBulto(p_modal, v_seleccionado))
                .abre();
        }
    }
    exports.DialogoSeleccionBulto = DialogoSeleccionBulto;
});
