define(["require", "exports", "Utilidades/assert", "componentes/api", "componentes/componentes", "componentes/foco", "componentes/boton", "componentes/campo", "componentes/pantalla"], function (require, exports, assert, api_1, componentes_1, foco_1, boton_1, campo_1, pantalla_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PantallaInventario = void 0;
    class PantallaInventario extends pantalla_1.Pantalla {
        constructor() {
            super();
            this.inicializa();
            this.asociaComponentes();
        }
        asociaComponentes() {
            const vComponentes = this.inicializaComponentes();
            Object.getOwnPropertyNames(vComponentes).forEach((pNombre) => (this[pNombre] = vComponentes[pNombre].crea()));
        }
        actualizaVista() {
            this._inputAlmacen.valor(this._modelo.GAALMNOM_COD);
            this._inputArticulo.valor(this._modelo.GAART_COD);
            this._inputArticuloDescrip.valor(this._modelo.GAART_DESCRIP);
            this._inputUbicacion.valor(this._modelo.GAUBIC_COD);
            this._inputLote.valor(this._modelo.Lote);
            this._inputExistencias.valor(this._modelo.Existencias);
            this._inputNuevas.valor(this._modelo.Cantidad);
            if (this._modelo.ControlUbicaciones === 'N') {
                this._inputUbicacion.deshabilita();
                this._btnUbic.deshabilita();
            }
            else {
                this._inputUbicacion.habilita();
                this._btnUbic.habilita();
            }
            if (this._modelo.ControlLotes === 'N') {
                this._inputLote.deshabilita();
                this._btnLote.deshabilita();
            }
            else {
                this._inputLote.habilita();
                this._btnLote.habilita();
            }
            this._botonConfirmar.habilitado(this.habilitarConfirmacion());
        }
        actualizaModelo() {
            this._modelo.GAALMNOM_COD = this._inputAlmacen.valor();
            this._modelo.GAART_COD = this._inputArticulo.valor();
            this._modelo.GAART_DESCRIP = this._inputArticuloDescrip.valor();
            this._modelo.GAUBIC_COD = this._inputUbicacion.valor();
            this._modelo.Lote = this._inputLote.valor();
            this._modelo.Existencias = this._inputExistencias.valor();
            this._modelo.Cantidad = this._inputNuevas.valor();
        }
        inicializa() {
            this._modelo = {
                EMP_COD: sessionStorage.getItem('EMPR_COD'),
                EMPGRUPO_COD: sessionStorage.getItem('EMPRGRUP_COD'),
                GAART_COD: '',
                GAART_DESCRIP: '',
                GAALMNOM_COD: '',
                GAALMNOM_DESCRIP: '',
                GAUBIC_COD: '',
                Lote: '',
                Existencias: null,
                Cantidad: null,
                ControlUbicaciones: 'S',
                ControlLotes: 'S',
            };
            this._loteUbicacionValido = false;
        }
        inicializaComponentes() {
            foco_1.foco.enPrimero(document);
            return {
                _botonNuevo: (0, boton_1.boton)('btnReset').clic(() => {
                    this.nuevo();
                }),
                _botonConfirmar: (0, boton_1.boton)('btnConfirmar')
                    .clic(async () => {
                    await this.confirmar();
                })
                    .tecla('Enter'),
                _inputAlmacen: (0, campo_1.campo)('txtAlmacen').lov(this.lovAlmacen.bind(this)),
                _btnBuscaAlmacen: (0, boton_1.boton)('btnBuscaAlmacen').clic(this.lovAlmacen.bind(this)),
                _inputArticulo: (0, campo_1.campo)('txtArt')
                    .cambio(async () => {
                    this.actualizaModelo();
                    await this.obtenerArticulo.bind(this)();
                })
                    .lov(this.lovArticulo.bind(this)),
                _btnArt: (0, boton_1.boton)('btnArt').clic(this.lovArticulo.bind(this)),
                _inputArticuloDescrip: (0, campo_1.campo)('txtArtDescr'),
                _inputUbicacion: (0, campo_1.campo)('txtUbic')
                    .cambio(async () => {
                    this.actualizaModelo();
                    await this.obtenerUbicacion();
                    if (this._modelo.ControlLotes === 'N')
                        await this.obtenerExistencias.bind(this)();
                })
                    .lov(this.lovUbicacion.bind(this)),
                _btnUbic: (0, boton_1.boton)('btnUbic').clic(this.lovUbicacion.bind(this)),
                _inputLote: (0, campo_1.campo)('txtLote')
                    .cambio(async () => {
                    this.actualizaModelo();
                    await this.obtenerExistencias.bind(this)();
                })
                    .lov(this.lovLote.bind(this)),
                _btnLote: (0, boton_1.boton)('btnLote').clic(this.lovLote.bind(this)),
                _inputExistencias: (0, campo_1.campo)('txtExistAct1'),
                _inputNuevas: (0, campo_1.campo)('txtNuev1').cambio(() => {
                    this.actualizaModelo.bind(this)();
                    this.actualizaVista.bind(this)();
                    if (this._botonConfirmar.habilitado())
                        foco_1.foco.en(this._botonConfirmar);
                }),
            };
        }
        nuevo() {
            this.inicializa();
            this.actualizaVista();
            foco_1.foco.enPrimero(document);
        }
        habilitarConfirmacion() {
            if ((this._modelo.Cantidad &&
                this._modelo.ControlLotes === 'N' &&
                this._modelo.ControlUbicaciones === 'N') ||
                (this._modelo.Cantidad && this._loteUbicacionValido))
                return true;
            return false;
        }
        async confirmar() {
            const almacen = this._modelo.GAALMNOM_COD;
            try {
                this.actualizaModelo();
                const [, vMensaje] = (await this.confirmarInventario());
                (0, componentes_1.notificacion)().muestra(vMensaje);
                this.nuevo();
                this._modelo.GAALMNOM_COD = almacen;
                this.actualizaVista();
            }
            catch (error) {
                console.error('Error confirming inventory:', error);
            }
        }
        async lovArticulo() {
            try {
                const vResultado = await this.obtenerArticulos();
                const vElegido = await (0, componentes_1.lov)()
                    .titulo(Resources.PantallaInventario.LovArticulo.Titulo)
                    .columnas([
                    { propiedad: 'COD', titulo: Resources.PantallaInventario.LovArticulo.Columna.Codigo, ancho: '6em' },
                    { propiedad: 'DESCRIP', titulo: Resources.PantallaInventario.LovArticulo.Columna.Descripcion },
                ])
                    .elige(vResultado);
                if (vElegido) {
                    this._modelo.GAART_COD = vElegido.COD;
                    await this.obtenerArticulo();
                    this.actualizaVista();
                }
            }
            catch (error) {
                console.error('Error in lovArticulo:', error);
            }
        }
        async lovAlmacen() {
            const vResultado = await this.obtenerAlmacenes();
            const vElegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaInventario.LovAlmacen.Titulo)
                .columnas([
                { propiedad: 'COD', titulo: Resources.PantallaInventario.LovAlmacen.Columna.Codigo, ancho: '6em' },
                { propiedad: 'DESCRIP', titulo: Resources.PantallaInventario.LovAlmacen.Columna.Nombre },
            ])
                .elige(vResultado);
            if (vElegido) {
                this._modelo.Existencias = null;
                this._modelo.GAUBIC_COD = null;
                this._modelo.Lote = null;
                this._modelo.Cantidad = null;
                this._modelo.ControlUbicaciones = 'S';
                this._modelo.ControlLotes = 'S';
                this._modelo.GAALMNOM_COD = vElegido.COD;
                this._modelo.GAALMNOM_DESCRIP = vElegido.DESCRIP;
                this.actualizaVista();
                foco_1.foco.enPrimero(document);
            }
        }
        async lovUbicacion() {
            const vResultado = await this.obtenerUbicaciones();
            const vElegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaInventario.LovUbicacion.Titulo)
                .columnas([{ propiedad: 'COD', titulo: Resources.PantallaInventario.LovUbicacion.Columna.Codigo }])
                .elige(vResultado);
            if (vElegido) {
                this._modelo.GAUBIC_COD = vElegido.COD;
                this._modelo.Lote = null;
                this._modelo.Existencias = null;
                this._modelo.Cantidad = null;
                if (this._modelo.ControlLotes === 'N')
                    await this.obtenerExistencias();
                this.actualizaVista();
                foco_1.foco.enSiguienteA(this._inputUbicacion);
            }
        }
        async lovLote() {
            const vResultado = await this.obtenerLotes();
            const vElegido = await (0, componentes_1.lov)()
                .titulo(Resources.PantallaInventario.LovLote.Titulo)
                .columnas([{ propiedad: 'COD', titulo: Resources.PantallaInventario.LovLote.Columna.Codigo }])
                .elige(vResultado);
            if (vElegido) {
                this._modelo.Lote = vElegido.COD;
                await this.obtenerExistencias();
            }
        }
        async obtenerArticulo() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Inventario/Articulo')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                    lecturaArticulo: this._modelo.GAART_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                this._inputArticulo.foco();
                return;
            }
            assert.esUn(vResultado, 'COD', 'DESCRIP', 'UBIC_SN', 'LOTES_SN', 'C_EXIST1', 'C_EXIST2');
            this._modelo.GAART_COD = vResultado.COD;
            this._modelo.GAART_DESCRIP = vResultado.DESCRIP;
            this._modelo.Existencias = null;
            this._modelo.GAUBIC_COD = null;
            this._modelo.Lote = null;
            this._modelo.Cantidad = null;
            this._modelo.ControlUbicaciones = vResultado.UBIC_SN;
            this._modelo.ControlLotes = vResultado.LOTES_SN;
            if (this._modelo.ControlUbicaciones === 'N' &&
                this._modelo.ControlLotes === 'N')
                this._modelo.Existencias = vResultado.C_EXIST1;
            this.actualizaVista();
            foco_1.foco.enSiguienteA(this._inputArticulo);
            return vResultado;
        }
        async obtenerArticulos() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Inventario/Articulos')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUnArrayDe(vResultado, 'COD', 'DESCRIP');
            return vResultado;
        }
        async obtenerExistencias() {
            let vResultado;
            let v_mensaje;
            try {
                [vResultado] = await (0, api_1.api)('/Inventario/Existencias')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                    LecturaArticulo: this._modelo.GAART_COD,
                    GAUBIC_COD: this._modelo.GAUBIC_COD,
                    Lote: this._modelo.Lote,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                this._loteUbicacionValido = false;
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUn(vResultado, 'C_EXIST1', 'C_EXIST2');
            this._loteUbicacionValido = true;
            this._modelo.Existencias = vResultado.C_EXIST1;
            this.actualizaVista();
            foco_1.foco.en(this._inputNuevas);
            return vResultado;
        }
        async obtenerAlmacenes() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Inventario/Almacenes')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUnArrayDe(vResultado, 'COD', 'DESCRIP');
            return vResultado;
        }
        async obtenerUbicaciones() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Inventario/Ubicaciones')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUnArrayDe(vResultado, 'COD');
            return vResultado;
        }
        async obtenerLotes() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Inventario/Lotes')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                    GAART_COD: this._modelo.GAART_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUnArrayDe(vResultado, 'COD');
            return vResultado;
        }
        async obtenerAlmacen() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Movimientos/Almacen')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUn(vResultado, 'COD', 'DESCRIP', 'GAUBIC_DEF', 'LOTE_DEF');
            return vResultado;
        }
        async obtenerUbicacion() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Movimientos/Ubicacion')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                    GAUBIC_COD: this._modelo.GAUBIC_COD,
                    Lote: this._modelo.Lote,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                foco_1.foco.en(this._inputUbicacion);
                return;
            }
            assert.esUn(vResultado, 'COD');
            if (vResultado?.COD && this._modelo.Lote)
                await this.obtenerExistencias();
            return vResultado;
        }
        async cambioArticulo() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Movimientos/Articulo')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                    GAUBIC_COD: this._modelo.GAUBIC_COD,
                    Lote: this._modelo.Lote,
                    LecturaArticulo: this._modelo.GAART_COD,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUn(vResultado, 'COD', 'DESCRIP', 'GAUBIC_DEF', 'LOTE_DEF');
            return vResultado;
        }
        async confirmarInventario() {
            let vResultado;
            let vMensaje;
            try {
                [vResultado, vMensaje] = await (0, api_1.api)('/Inventario/ActualizarRecuento')
                    .metodo('put')
                    .parametros({
                    EMPGRUPO_COD: this._modelo.EMPGRUPO_COD,
                    EMP_COD: this._modelo.EMP_COD,
                    GAALMNOM_COD: this._modelo.GAALMNOM_COD,
                    GAART_COD: this._modelo.GAART_COD,
                    GAUBIC_COD: this._modelo.GAUBIC_COD,
                    LOTE: this._modelo.Lote,
                    C_UNID1: this._modelo.Cantidad,
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            return [vResultado, vMensaje];
        }
    }
    exports.PantallaInventario = PantallaInventario;
});
