define(["require", "exports", "componentes/api", "componentes/componentes", "componentes/foco", "componentes/pantalla", "componentes/boton", "componentes/campo"], function (require, exports, api_1, componentes_1, foco_1, pantalla_1, boton_1, campo_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PantallaPesaje = void 0;
    class PantallaPesaje extends pantalla_1.Pantalla {
        constructor() {
            super();
            this.inicializa();
            this.asociaComponentes();
        }
        inicializa() {
            this._camposPesaje = {
                artRef: "",
                artDesc: "",
                peso: 0,
                pesoNuevo: 0,
            };
        }
        inicializaComponentes() {
            foco_1.foco.enPrimero(document);
            const vComponentes = {
                _btnReset: (0, boton_1.boton)('btnReset').clic(() => {
                    this.inicializa();
                    this.actualizaVista();
                    foco_1.foco.en(this._txtArtRef);
                }),
                _btnConfirm: (0, boton_1.boton)('btnConfirm').clic(async () => {
                    await this.confirmar();
                    this.actualizaVista();
                }),
                _txtArtRef: (0, campo_1.campo)('txtArtRef')
                    .notifica('con-pulsacion-retrasada')
                    .cambio(async () => {
                    this.actualizaModelo();
                    await this.obtenerArticulo();
                    this.actualizaVista();
                }),
                _txtArtDescrip: (0, campo_1.campo)('txtArtDescrip'),
                _txtPeso: (0, campo_1.campo)('txtPeso'),
                _txtPesoNuevo: (0, campo_1.campo)('txtPesoNuevo'),
            };
            return vComponentes;
        }
        asociaComponentes() {
            const vComponentes = this.inicializaComponentes();
            Object.getOwnPropertyNames(vComponentes).forEach(pNombre => this[pNombre] = vComponentes[pNombre].crea());
            this.actualizaVista();
            return;
        }
        actualizaVista() {
            this._txtArtRef.valor(this._camposPesaje.artRef);
            this._txtArtDescrip.valor(this._camposPesaje.artDesc);
            this._txtPeso.valor(this._camposPesaje.peso);
            this._txtPesoNuevo.valor(this._camposPesaje.pesoNuevo);
        }
        actualizaModelo() {
            this._camposPesaje.artRef = this._txtArtRef.valor();
            this._camposPesaje.artDesc = this._txtArtDescrip.valor();
            this._camposPesaje.peso = this._txtPeso.valor();
            this._camposPesaje.pesoNuevo = this._txtPesoNuevo.valor();
        }
        async obtenerArticulo() {
            let vResultado;
            let parametros;
            let GAART_COD;
            try {
                GAART_COD = this._camposPesaje.artRef;
                parametros = {
                    empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                    emp_cod: sessionStorage.getItem("EMPR_COD"),
                    GAART_COD: GAART_COD
                };
                [vResultado] = await (0, api_1.api)('/Inventario/Peso')
                    .metodo('get')
                    .parametros(parametros)
                    .invoca();
                if ([vResultado]) {
                    this._camposPesaje.artRef = vResultado.GAART_COD;
                    this._camposPesaje.artDesc = vResultado.GAART_DESCRIP;
                    this._camposPesaje.peso = vResultado.PESO;
                    this._camposPesaje.pesoNuevo = 0;
                    foco_1.foco.en(this._txtPesoNuevo);
                }
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                this.inicializa();
            }
        }
        async confirmar() {
            let vResultado;
            let parametros;
            let GAART_COD;
            let artDescVal = this._camposPesaje.artDesc;
            let pesoNuevoVal = this._txtPesoNuevo.valor();
            if (artDescVal.length > 0) {
                if (typeof pesoNuevoVal === 'number' && pesoNuevoVal > 0) {
                    try {
                        GAART_COD = this._camposPesaje.artRef;
                        parametros = {
                            empgrupo_cod: sessionStorage.getItem("EMPRGRUP_COD"),
                            emp_cod: sessionStorage.getItem("EMPR_COD"),
                            GAART_COD: GAART_COD,
                            PESO: pesoNuevoVal,
                        };
                        [vResultado] = await (0, api_1.api)('/Inventario/Peso')
                            .metodo('put')
                            .parametros(parametros)
                            .invoca();
                        if ([vResultado]) {
                            await (0, componentes_1.notificacion)().muestra('Peso actualizado! > GAART_COD-' + GAART_COD);
                            this.inicializa();
                            foco_1.foco.en(this._txtArtRef);
                        }
                        else {
                            await (0, componentes_1.notificacion)().error().muestra('No se ha podido actualizar el peso! > GAART_COD-' + GAART_COD + " || " + vResultado);
                        }
                    }
                    catch ([vMensaje]) {
                        await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                        foco_1.foco.en(this._txtArtRef);
                    }
                }
                else {
                    await (0, componentes_1.mensaje)().error().muestra('El campo "Peso Nuevo" no debe estar vacío, ser menor o igual a "0" o contener valores alfanuméricos!');
                    foco_1.foco.en(this._txtPesoNuevo);
                }
            }
            else {
                await (0, componentes_1.mensaje)().error().muestra('Se debe indicar un artículo');
            }
        }
    }
    exports.PantallaPesaje = PantallaPesaje;
});
