define(["require", "exports", "componentes/componentes", "componentes/foco", "componentes/api", "lib/lodash/lodash", "Recepcion/modelos/existencias", "Recepcion/dialogos/modelos/dialogo-lotes-ubicaciones-modelo"], function (require, exports, componentes_1, foco_1, api_1, _, existencias_1, dialogo_lotes_ubicaciones_modelo_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.DialogoLotesUbicaciones = void 0;
    class DialogoLotesUbicaciones {
        constructor(init) {
            this._modelo = new dialogo_lotes_ubicaciones_modelo_1.LotesUbicacionesPantallaModel({ Linea: init.Linea, Pedido: init.Pedido });
            this._modal = init.modal;
            this._lineaPreparada = new dialogo_lotes_ubicaciones_modelo_1.DetallesLineaPreparada();
            this.ConfiguraPantalla();
        }
        ConfiguraPantalla() {
            const columnas = [];
            if (this._modelo.ControlUbicaciones) {
                columnas.push({ propiedad: 'GAUBIC_COD', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.tabla.columna.GAUBIC_COD, ancho: 'auto' });
            }
            else {
                this._modelo.GAUBIC_COD = '*';
            }
            if (this._modelo.ControlLotes) {
                columnas.push({ propiedad: 'LOTE', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.tabla.columna.Lote, ancho: 'auto' });
            }
            else {
                this._modelo.LOTE = '*';
            }
            columnas.push({ propiedad: 'C_UNI1', titulo: Resources.PantallaRecepcion.DialogoLotesUbicaciones.tabla.columna.C_UNI1, ancho: 'auto', alineacion: 'centro' });
            this._tabla = this._modal.tabla('disponible')
                .datos(this._modelo.Linea.Existencias)
                .columnas(columnas)
                .seleccionado(pSeleccion => {
                this._modelo.GAUBIC_COD = pSeleccion?.GAUBIC_COD || null;
                this._modelo.LOTE = pSeleccion?.LOTE || null;
                this._modelo.C_UNI1 = this._modelo.Linea.C_PDTE || null;
                this.actualizaVista();
                this._inputLote.foco();
            })
                .crea();
            this._inputGaartCod = this._modal.campo('gaart_cod').crea();
            this._inputGaartDescrip = this._modal.campo('gaart_descrip').crea();
            this._inputPendiente = this._modal.campo('pendiente').crea();
            this.bloqueEdicion = this._modal.bloque('bloque-edicion').crea();
            this._inputGaubicCod = this._modal.campo('gaubic_cod')
                .valida(async (p_valor) => {
                const validado = await this.validarUbicacionesAlmacen(p_valor);
                if (validado && p_valor?.nuevoValor) {
                    const v_datosEcontrado = this._tabla.buscaYSelecciona({ GAUBIC_COD: p_valor.nuevoValor }, { seleccionaPrimero: true });
                    if (!v_datosEcontrado) {
                        this._modelo.GAUBIC_COD = p_valor.nuevoValor;
                        this.actualizaVista();
                    }
                    this._inputLote.foco();
                }
            })
                .crea();
            this._inputGaubicCod.visible(this._modelo.ControlUbicaciones);
            this._inputGaubicCod.habilitado(this._modelo.ControlUbicaciones);
            this._inputLote = this._modal.campo('lote').crea();
            this._inputLote.visible(this._modelo.ControlLotes);
            this._inputLote.habilitado(this._modelo.ControlLotes);
            this._inputCantidad = this._modal.campo('cantidad')
                .valida(p_valor => {
                const v_gaubic_cod = this._inputGaubicCod.valor();
                const v_lote = this._inputLote.valor();
                const v_cantidad = p_valor?.nuevoValor ? p_valor.nuevoValor : 0;
                this.insertaOActualizaLinea(v_gaubic_cod, v_lote, v_cantidad);
            })
                .crea();
            this._botonAceptar = this._modal.boton('aceptar')
                .clic(this.acepta.bind(this))
                .crea();
            setTimeout(() => {
                this.gestionFoco();
            }, 0);
        }
        async validarUbicacionesAlmacen(gaubic_cod) {
            let respuesta = null;
            try {
                [respuesta] = await (0, api_1.api)('/Recepcion/ValidaUbicacionAlmacen')
                    .metodo('get')
                    .parametros({
                    "empgrupo_cod": sessionStorage.getItem("EMPRGRUP_COD"),
                    "emp_cod": sessionStorage.getItem("EMPR_COD"),
                    "gaalmnom_cod": this._modelo.Linea.GAALMNOM_COD,
                    "gaubic_cod": gaubic_cod?.nuevoValor ? gaubic_cod.nuevoValor : null
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                this._inputGaubicCod.foco();
                return false;
            }
            return true;
        }
        gestionFoco() {
            foco_1.foco.en(this._inputGaubicCod);
        }
        puedeAceptar() {
            const vPuedeAceptar = !_.isEqual(this._modelo.Linea, this._modelo.LineaOriginal) || (this._modelo.GAUBIC_COD !== null && this._modelo.LOTE !== null && this._modelo.C_UNI1 !== null);
            return vPuedeAceptar;
        }
        async insertaOActualizaLinea(pGaubicCod, pLote, pCantidad) {
            if ((pGaubicCod || pLote) && pCantidad) {
                const vDisponible = this._modelo.Linea.Existencias.find(pDisponible => pDisponible.GAUBIC_COD === pGaubicCod && pDisponible.LOTE === pLote);
                if (typeof vDisponible !== 'undefined') {
                    vDisponible.C_UNI1 += pCantidad;
                }
                else {
                    this._modelo.Linea.Existencias.push(new existencias_1.Existencias({ GAUBIC_COD: pGaubicCod, LOTE: pLote, C_UNI1: pCantidad }));
                }
                this._tabla.refresca();
                this._modelo.Linea.C_PDTE -= pCantidad;
                this._lineaPreparada.C_PREP += pCantidad;
                const vLoteUbicPreparado = this._lineaPreparada.Detalles.find(pPreparado => pPreparado.GAUBIC_COD === pGaubicCod && pPreparado.LOTE === pLote);
                if (vLoteUbicPreparado) {
                    vLoteUbicPreparado.C_UNI1 += pCantidad;
                }
                else {
                    this._lineaPreparada.Detalles.push(new dialogo_lotes_ubicaciones_modelo_1.Detalle({
                        GAUBIC_COD: pGaubicCod,
                        LOTE: pLote,
                        C_UNI1: pCantidad
                    }));
                }
                this._modelo.GAUBIC_COD = this._modelo.LOTE = this._modelo.C_UNI1 = null;
                this.actualizaVista();
                return true;
            }
            return false;
        }
        actualizaVista() {
            this._inputGaartCod.valor(this._modelo.Linea.GAART_COD);
            this._inputGaartDescrip.valor(this._modelo.Linea.GAART_DESCRIP);
            this._inputPendiente.valor(this._modelo.Linea.C_PDTE);
            this._tabla.datos(null);
            this._tabla.datos(this._modelo.Linea.Existencias);
            this.bloqueEdicion.clase('ubicacion', this._modelo.Linea.GAART_UBIC_SN === 'S');
            this.bloqueEdicion.clase('lote', this._modelo.Linea.GAART_LOTES_SN === 'S');
            this._inputGaubicCod.valor(this._modelo.GAUBIC_COD);
            this._inputLote.valor(this._modelo.LOTE);
            this._inputCantidad.valor(this._modelo.C_UNI1);
            this._botonAceptar.habilitado(this.puedeAceptar());
        }
        actualizaModelo() {
            this._modelo.GAUBIC_COD = this._inputGaubicCod.valor();
            this._modelo.LOTE = this._inputLote.valor();
            this._modelo.C_UNI1 = this._inputCantidad.valor();
        }
        async acepta() {
            this.actualizaModelo();
            let vContinuar = true;
            if (this._modelo.GAUBIC_COD && this._modelo.LOTE && this._modelo.C_UNI1)
                vContinuar = await this.insertaOActualizaLinea(this._modelo.GAUBIC_COD, this._modelo.LOTE, this._modelo.C_UNI1);
            if (vContinuar)
                this._modal.cierra(new dialogo_lotes_ubicaciones_modelo_1.LineaPreparada({
                    Disponible: this._modelo.Linea.Existencias,
                    Preparado: this._lineaPreparada
                }));
        }
    }
    exports.DialogoLotesUbicaciones = DialogoLotesUbicaciones;
});
