﻿/**
 * Función para inicializar el diálogo de empresas.
 * Se ejecuta cuando el documento está listo y el botón de diálogo se hace clic.
 * @returns {void}
 */
$(document).ready(function () {
	$('#dialogButton').click(async function () {
		// URL para obtener datos de la empresa
		var v_url = 'menu/empresas';
		// Método HTTP para la solicitud
		var v_method = 'GET';
		// Datos para enviar con la solicitud
		var v_data = null;

		/**
		 * Función para manejar la respuesta exitosa de la solicitud de la API.
		 * Genera un diálogo con los datos de la empresa.
		 * @param {Object} response - La respuesta de la API.
		 * @returns {void}
		 */
		var v_success = function (response) {
			if (response.Ok) {
				var datosJson = response.Objeto;
				var url = 'Dialog/Dialog';
				var columnsJson = [
					{
						mDataProp: 'EMPGRUPO_COD',
						name: Resources.table_head_company_code,
						sClass: 'dt_col_hide',
					},
					{ mDataProp: 'COD', name: Resources.table_head_code },
					{ mDataProp: 'NOMBRE', name: Resources.table_head_name },
				];
				var columnDefs = [{ width: '6em', target: 1 }];
				var buttons = [
					generateSingleButton('selectEmpresa', Resources.button_accept),
				];

				generarDialogAuto(
					url,
					'#dialogLoV',
					'empresas',
					null,
					datosJson,
					columnsJson,
					buttons,
					null,
					null,
					empresas,
					Resources.title_company,
					null,
					null,
					null,
					Resources.label_company_empty,
					null,
					null,
					columnDefs,
					true
				);
			} else {
				lanzarMensaje(true, response.Mensaje);
			}
		};

		// Realizar la solicitud a la API
		await apiGeneral(v_url, v_method, v_data, v_success);
	});
});

function seleccionar() {
	var v_data = $('#empresas').DataTable().row('.selected').data();
	var v_grupo = v_data.EMPGRUPO_COD;
	var v_cod = v_data.COD;
	var v_nombre = v_data.NOMBRE;

	setEmpresa(v_grupo, v_nombre, v_cod);
	setEmpresaPred(v_grupo, v_cod);
	closeModal('#dialogLoV');
}

function empresas() {
	$('#selectEmpresa').on('click', function () {
		seleccionar();
	});
}

async function setEmpresaPred(grupo, cod) {
	var v_url = 'menu/empresapredeterminada';
	var v_method = 'PUT';
	var v_data = {
		empgrupo_cod: grupo,
		emp_cod: cod,
	};
	var v_success = function (response) {
		if (response.Ok) {
			//window.location.href = Router.action("menu", "submenu");
			window.location.href = 'Menu';
		} else {
			lanzarMensaje(false, response.Mensaje);
		}
	};

	await apiGeneral(v_url, v_method, v_data, v_success);
}

async function obtenerEmpresaPredeterminada(funcArr) {
	if (!sessionStorage.getItem('EMPRGRUP_COD')) {
		var v_url = 'menu/empresapredeterminada';
		var v_method = 'GET';
		var v_data = null;
		var v_success = function (response) {
			if (response.Ok) {
				var grupo = response.Objeto.EMPGRUPO_COD;
				var cod = response.Objeto.COD;
				var nombre = response.Objeto.NOMBRE;
				setEmpresa(grupo, nombre, cod);
			} else {
				lanzarMensaje(false, response.Mensaje);
			}
		};

		await apiGeneral(v_url, v_method, v_data, v_success);
	} else {
		$('#dialogButton #nombre_empresa').val(
			sessionStorage.getItem('EMPR_NOMBRE')
		);
	}

	if (funcArr) {
		funcArr.forEach(function (func) {
			func();
		});
	}
}

function setEmpresa(grupo, nombre, cod) {
	sessionStorage.setItem('EMPRGRUP_COD', grupo);
	sessionStorage.setItem('EMPR_COD', cod);
	sessionStorage.setItem('EMPR_NOMBRE', nombre);
	sessionStorage.getItem('EMPRGRUP_COD');
	$('#dialogButton #nombre_empresa').val(nombre);
}
