﻿var Resources;
var json;

/**
 * Función que se ejecuta cuando el documento está listo.
 * Realiza varias operaciones, como ajustar el tamaño de las tablas y configurar los eventos de los botones.
 */
$(document).ready(function () {
	// Ajusta el tamaño de las tablas cuando se redimensiona la ventana
	var body = this.body;
	var height = body.offsetHeight;
	this.defaultView.onresize = function () {
		var tables = $('.dataTable');
		tables.each(function (i, table) {
			if (table.id && $(table).is(':visible')) {
				SpyFilter().applyFilters('#' + table.id);
			}
		});
		var newHeight = body.offsetHeight;
		// Actualiza la altura si ha cambiado
		if (newHeight !== height) {
			height = newHeight;
		}
	};

	cambioTamanioMenu();
	cameraFunction();

	// Configura el evento de click del botón 'atras' para navegar hacia atrás en el historial del navegador
	$('#atras').on('click', function () {
		window.history.back();
	});

	toUpper();
	this.tabulado = tabulado();
	autoselect();
});

function toUpper() {
	$('.code').on('blur', function () {
		$(this).val($(this).val().toUpperCase());
	});

	$('.code').on('input', function (e) {
		//var input = $(this);
		var v_cursor = $(this)[0].selectionStart;
		$(this).val($(this).val().toUpperCase());

		if (v_cursor !== null) {
			$(this)[0].selectionStart = $(this)[0].selectionEnd = v_cursor;
		}
	});
}

$(window).resize(function () {
	cambioTamanioMenu();
	console.log('Resize');
});

/**
 * Función para abrir un diálogo y cargar datos desde una URL especificada.
 * Si se proporciona una tabla, crea una tabla con los datos obtenidos.
 * Si se proporciona una función, la ejecuta después de cargar los datos.
 *
 * @param {string} dialog - El selector del diálogo a mostrar.
 * @param {string} href - La URL desde donde cargar los datos.
 * @param {string} table - El selector de la tabla a crear (opcional).
 * @param {Function} func - La función a ejecutar después de cargar los datos (opcional).
 * @returns {boolean} - Siempre devuelve false.
 */
function dialog(dialog, href, table, func) {
	if (typeof dialog !== 'string' || typeof href !== 'string') {
		throw new Error(
			'Los parámetros "dialog" y "href" deben ser cadenas de texto.'
		);
	}

	if (table && typeof table !== 'string') {
		throw new Error('El parámetro "table" debe ser una cadena de texto.');
	}

	if (func && typeof func !== 'function') {
		throw new Error('El parámetro "func" debe ser una función.');
	}

	$.ajax({
		url: href,
		cache: false,
		dataType: 'json',
		success: function (response) {
			if (response.Ok) {
				$(dialog).html(response.Mensaje);
				showDialog(dialog);

				if (table) {
					createTable(table);
				}

				if (func) {
					func();
				}
			} else {
				lanzarMensaje(response.Ok, response.Mensaje);
			}

			$('.disabled').prop('disabled', true);
		},
		error: function () {
			lanzarMensaje(false, 'Error al cargar los datos.');
		},
	});

	return false;
}

/**
 * Función para abrir un diálogo automáticamente y cargar datos desde una URL especificada.
 * Si se proporciona una tabla, crea una tabla con los datos obtenidos.
 * Si se proporciona una función, la ejecuta después de cargar los datos.
 *
 * @param {string} dialog - El selector del diálogo a mostrar.
 * @param {string} href - La URL desde donde cargar los datos.
 * @param {string} table - El selector de la tabla a crear (opcional).
 * @param {Function} func - La función a ejecutar después de cargar los datos (opcional).
 * @returns {boolean} - Siempre devuelve false.
 */
function dialogAuto(dialog, href, table, func) {
	if (typeof dialog !== 'string' || typeof href !== 'string') {
		throw new Error(
			'Los parámetros "dialog" y "href" deben ser cadenas de texto.'
		);
	}

	if (table && typeof table !== 'string') {
		throw new Error('El parámetro "table" debe ser una cadena de texto.');
	}

	if (func && typeof func !== 'function') {
		throw new Error('El parámetro "func" debe ser una función.');
	}

	$.ajax({
		url: href,
		cache: false,
		dataType: 'json',
		success: function (response) {
			if (response.Ok) {
				$(dialog).html(response.Mensaje);
				showDialog(dialog);

				if (table) {
					$(dialog + ' table').prop('id', table.split('#')[1]);
					createTable(table);
				}

				if (func) {
					func();
				}

				toUpper();
			} else {
				lanzarMensaje(response.Ok, response.Mensaje);
			}

			$('.disabled').prop('disabled', true);
		},
		error: function () {
			lanzarMensaje(false, 'Error al cargar los datos.');
		},
	});

	return false;
}

function dialogMessage(href, message) {
	var dialog = '#dialogMessage';
	showDialog(dialog);
	var error = '{data:' + JSON.stringify(error) + '}';
	$(dialog).load(href, error);
}

/**
 * Añade la característica de la selección de filas a la tabla indicada.
 *
 * string table : "#ejemplo"
 * @param {string} table ID de la tabla
 * @param {boolean} multipleSelection Permitir seleccionar varios campos a la vez
 */
function selectRow(table, multipleSelection) {
	var v_datatable = $(table).dataTable();

	$(table).on('mousedown', 'tr', function (e) {
		var v_table = $(table).DataTable();
		if (v_table.rows().count() !== 0) {
			if (multipleSelection) {
				$(this).toggleClass('selected');
			} else {
				v_table.row('.selected').nodes().to$().removeClass('selected');
				$(this).addClass('selected');
			}
			return false;
		}
	});

	if (table !== '#empresas') {
		var firstRow = v_datatable.find('tbody tr:first');
		//firstRow.trigger('mousedown');
		firstRow.addClass('selected');
	}
}

function moveFilter(table, searchField) {
	//setTimeout(function () {
	var modal = $(table + '_filter').parents('.modal-bodyContainer')[0];
	$(table + '_filter label').addClass('col-xs-12');
	$(table + '_filter').prependTo(modal);
	//}, 0);
	searchField
		? $(table + '_filter').css('display', 'table-row')
		: $(table + '_filter').hide();
}

/**
 * Lanza una ventana modal con un mensaje, que al aceptar lanzará la función introducida.
 * Si el mensaje NO es de error, redirigirá al url indicado (redirectTo)
 * @param {boolean} Ok Tipo de mensaje  -  True => Mensaje  \\  False => Error
 * @param {string} Message Mensaje a mostrar
 * @param {string} RedirectTo URL de redireccion. Esto ocurre al pulsar el botón aceptar.
 * @param {function} functionName Función lanzada una vez se haga click en el botón aceptar.
 */
function lanzarMensajeRedirect(Ok, Message, RedirectTo, functionName) {
	showDialog('#dialogMessage');
	$('#dialogMessage #messageAccept').focus();
	$('#dialogMessage #messageAccept').unbind();
	var close = true;
	$('#dialogMessage #messageAccept').on('keydown click', function (e) {
		if (e.which === 13) {
			close = false;
			$('#dialogMessage #messageAccept').on('keyup', function (e) {
				if (e.which === 13) {
					close = true;
					closeMessageDialog();
				}
			});
		} else if (e.which === 1 && close === true) {
			closeMessageDialog();
		}
	});

	function closeMessageDialog() {
		$('#dialogMessage').modal('hide');
		$('.modal-header').css('background-color', '#2a75c2');
		if (functionName) {
			functionName();
		}
	}

	//$('.modal-close').on('click', function () {
	//    closeMessageDialog();
	//});

	Ok
		? $('#dialogMessage .modal-title').html(Resources.title_message)
		: $('#dialogMessage .modal-title').html(Resources.title_error);
	Ok
		? $('.modal-header').css('background-color', '#2a75c2')
		: $('.modal-header').css('background-color', '#b00020');
	$('#dialogMessage .txtMessage').html(
		'<div>' + Message.replace(/\n/g, '<br />') + '</div>'
	);

	// Busca la ventana modal con mayor zindex y suma 1
	var zindex = eval($($('.modal')[$('.modal').length - 1]).css('z-index')) + 1;
	$($('.modal-backdrop')[$('.modal-backdrop').length - 1]).css(
		'z-index',
		zindex
	);
	$('#dialogMessage').css('z-index', eval(zindex) + 1);
	if (RedirectTo && Ok) {
		$(
			'#dialogMessage .close, #dialogMessage, #dialogMessage #messageAccept'
		).on('click', function () {
			window.location.href = RedirectTo;
		});
		$('.modal-body, .modal-header').on('click', function (e) {
			e.stopPropagation();
			return false;
		});
	}
}

/**
 * Lanza un mensaje de confirmación; Si se acepta, se lanzará la función indicada.
 * @param {String} Message Mensaje a mostrar
 * @param {function} functionAccept Función lanzada una vez se haga click en el botón aceptar.
 * @param {function} functionCancel Función lanzada una vez se haga click en el botón cancelar.
 */
function lanzarConfirmacion(Message, functionAccept, functionCancel) {
	showDialog('#dialogConfirm');
	$('#dialogConfirm #messageAccept').unbind();
	$('#dialogConfirm #messageAccept').off();
	$('#dialogConfirm #messageAccept').on('click', function () {
		$('#dialogConfirm').modal('hide');
	});

	$('#dialogConfirm .txtMessage').html(
		'<div>' + Message.replace(/\n/g, '<br />') + '</div>'
	);
	var zindex = eval($($('.modal')[$('.modal').length - 1]).css('z-index')) + 1;
	$($('.modal-backdrop')[$('.modal-backdrop').length - 1]).css(
		'z-index',
		zindex
	);
	$('#dialogConfirm').css('z-index', eval(zindex) + 1);

	$('#btnConfirm, #btnCancel').unbind();
	//$('#btnConfirm').off();

	//Encapsulamos la función para que en el caso que utilicen "this" no cojan el botón de confirmar.
	$('#btnConfirm').on('click', function () {
		functionAccept();
	});
	if (functionCancel)
		$('#btnCancel').on('click', function () {
			functionCancel();
		});
	$('#btnConfirm, #btnCancel').on('click', function () {
		closeModal('#dialogConfirm');
	});
}

/**
 * Lanza una función u otra dependiendo del Ok recibido en response.
 * *Si el response tiene un mensaje se mostrará en pantalla ANTES de lanzarse la función.
 * *La función se lanzará al cerrar el dialogo del mensaje.
 * @param {any} response Objeto de respuesta del API => {Ok: false, Mensaje: <null || ""> , Objeto: {}}
 * @param {function} successFn Función si la respuesta es Ok
 * @param {function} errorFn Función si la respuesta NO es Ok
 * @param {boolean} bSuccessMessage False => Lanza una ventana de información || True => Muestra el mensaje en el título para no detener al usuario.
 */
function handleResponse(response, successFn, errorFn, bSuccessMessage) {
	var fn = response.Ok ? successFn : errorFn;
	if (!response.Ok || (response.Mensaje && !bSuccessMessage)) {
		lanzarMensaje(response.Ok, response.Mensaje, fn);
	} else if (response.Mensaje && bSuccessMessage) {
		successMessage(response.Mensaje);
		if (fn) fn();
	} else if (fn) fn();
}

function lanzarMensaje(Ok, Message, functionName) {
	lanzarMensajeRedirect(Ok, Message, null, functionName);
}

function createTable(
	table,
	displayLength,
	zeroRecordsMessage,
	responsive,
	multipleSelection
) {
	if (!displayLength) {
		var windowWidth = $(window).width();
		displayLength = windowWidth < 400 ? 4 : 7;
	}
	if (!zeroRecordsMessage) {
		zeroRecordsMessage = Resources.zeroRecordsMessage;
	}
	$(table).dataTable({
		ordering: false,
		autoWidth: true,
		responsive: responsive,
		displayLenght: 1,
		pagingType: 'simple',
		scroller: false,
		scrollY: true,
		scrollCollapse: true,
		scrollX: false,
		bInfo: false,
		paginate: {
			next: '&#8594;', // '→'
			previous: '&#8592;', // '←'
		},
		deferRender: false,
		aoColumnDefs: [{ sType: 'numeric' }],
		drawCallback: function () {
			$('.dataTable').each(function (e) {
				//applyFilters(this)
				//$(this).DataTable().columns.adjust().responsive.recalc();
			});
			navigateArrows(table);
		},
		initComplete: function (settings, json) {
			SpyFilter().applyFilters(table);
		},
		language: {
			sProcessing: 'Procesando...',
			sZeroRecords: 'No se encontraron resultados',
			sEmptyTable: zeroRecordsMessage,
			sInfo: Resources.table_sinfo,
			sInfoEmpty: Resources.table_sinfo_empty,
			sInfoFiltered: '(filtrado de un total de _MAX_ registros)',
			sInfoPostFix: '',
			sSearch: Resources.table_search + ':',
			sUrl: '',
			sInfoThousands: ',',
			sLoadingRecords: Resources.table_loading_records,
			oPaginate: {
				sFirst: 'Primero',
				sLast: 'Último',
				sNext: '<i class="glyphicon glyphicon-arrow-right gi-lg"/>',
				sPrevious: '<i class="glyphicon glyphicon-arrow-left gi-lg"/>',
			},
			oAria: {
				sSortAscending:
					': Activar para ordenar la columna de manera ascendente',
				sSortDescending:
					': Activar para ordenar la columna de manera descendente',
			},
		},
	}),
		$('.dataTable').each(function (e) {
			//$(this).DataTable().columns.adjust();
		});
	$('.dataTables_filter input').addClass('col-xs-7 input-xs');
	selectRow(table, multipleSelection);
	moveFilter(table);
}

/**
 * Genera una tabla a partir de un JSON:
 *
 * Ejemplo:
 * var table = "#ejemplo";
 * var data = [{ID: 1, Art: "Art1"}, {ID: 2, Art: "Art2"}, ...];
 *
 * // La clave de data debe coincidir con la id de la columna
 * var columns = [
 *      generateColumn('ID', 'ID', true),
 *      generateColumn('Art', 'Articulo', true)
 *  ];
 *
 * // Registros a mostrar
 * var displayLength = 6;
 *
 * var zeroRecordsMessage = "No se encuentran artículos";
 *
 * var columnsDefs = []   (https://datatables.net/reference/option/columnDefs)
 *
 * var responsive = true;
 *
 * // Se lanza cada vez que se pinta la tabla
 * var drawCallBackFN = function () {}
 * @param {string} table ID de la tabla
 * @param {JSON} data Array o JSON de registros a mostrar en la tabla
 * @param {Array} columns Array de columnas a mostrar en la tabla
 * @param {number} displayLength Número de registros a mostrar (Actualmente no utilizado por el alto dinámico de las tablas)
 * @param {string} zeroRecordsMessage Mensaje a mostrar en caso de no tener registros
 * @param {Array<JSON>} columnDefs Definiciones de columnas: https://datatables.net/reference/option/columnDefs
 * @param {boolean} responsive Tablas responsive (Actualmente no se utiliza)
 * @param {function} drawCallbackFn Funcion que se llama cada vez que se modifica la tabla
 * @param {boolean} searchField Mostrar campo de búsqueda
 * @param {boolean} multipleSelection Seleccionar un registro no quita la seleccion del resto. Seleccionar un registro ya seleccionado, quita la seleccion de este.
 * @param {Array} orderDef Indica el orden inicial de las columnas.
 */
function createTableJson(
	table,
	data,
	columns,
	displayLength,
	zeroRecordsMessage,
	columnDefs,
	responsive,
	drawCallbackFn,
	searchField,
	multipleSelection,
	orderDef
) {
	responsive = !responsive ? true : responsive;

	var ordering = orderDef ? true : false;

	if (!zeroRecordsMessage) {
		zeroRecordsMessage = Resources.table_no_data;
	}
	if (!drawCallbackFn) {
		drawCallbackFn = function () {};
	}

	$(table)
		.on('error.dt', function (e, settings, techNote, message) {
			lanzarMensaje(false, 'Ha ocurrido un error de DT: \n\n' + message);
		})
		.dataTable({
			ordering: ordering,
			order: orderDef,
			aaData: data,
			autoWidth: false,
			aoColumns: columns,
			columnDefs: columnDefs,
			displayLenght: displayLength,
			responsive: responsive,
			pagingType: 'simple',
			scrollY: false,
			scrollCollapse: true,
			scrollX: false,
			deferRender: true,
			bInfo: false,
			paginate: {
				next: '&#8594;', // or '→'
				previous: '&#8592;', // or '←'
			},
			drawCallback: function () {
				navigateArrows(table);
				drawCallbackFn();
			},
			initComplete: function (settings, json) {
				SpyFilter().applyFilters(table);
			},
			fnRowCallback: function (nRow, aData, iDisplayIndex) {
				//$(table).DataTable().columns.adjust().responsive.rebuild().responsive.recalc();
				return nRow;
			},
			language: {
				sProcessing: 'Procesando...',
				sLengthMenu: Resources.table_length_menu,
				sZeroRecords: 'No se encontraron resultados',
				sEmptyTable: zeroRecordsMessage,
				sInfo: Resources.table_sinfo,
				sInfoEmpty: Resources.table_sinfo_empty,
				sInfoFiltered: '(filtrado de un total de _MAX_ registros)',
				sInfoPostFix: '',
				sSearch: Resources.table_search + ':',
				sUrl: '',
				sInfoThousands: ',',
				sLoadingRecords: Resources.table_loading_records,
				oPaginate: {
					sFirst: 'Primero',
					sLast: 'Último',
					sNext: '<i class="glyphicon glyphicon-arrow-right gi-2x"/>',
					sPrevious: '<i class="glyphicon glyphicon-arrow-left gi-2x"/>',
				},
				oAria: {
					sSortAscending:
						': Activar para ordenar la columna de manera ascendente',
					sSortDescending:
						': Activar para ordenar la columna de manera descendente',
				},
			},
		});

	$('.dataTables_filter input').addClass('pull-right form-control input-xs');
	//$(table).css('table-layout', 'fixed')
	$(table).DataTable().page.len(1).draw();
	console.log($(table));
	selectRow(table, multipleSelection);
	moveFilter(table, searchField);
}

var v_code;
function obtenerCodBarras(JQInput) {
	v_code = leerEtiqueta(JQInput.val());
}

function getToken() {
	var v_token = sessionStorage.getItem('token');
	if (v_token === null) {
		sessionStorage.removeItem('token');
		redirectToLogin();
	} else {
		sessionStorage.setItem('token', v_token);
	}

	return v_token;
}

//function validationError(message) {
//    //lanzarMensajeRedirect(true, message, Router.action('Login', 'Logout'));
//    lanzarMensajeRedirect(true, message, "~/Login/Logout");
//}
//function redirectToLogin() {
//    //if (!sessionStorage.getItem('token') && window.location.pathname !== Router.action('Login', 'Login')) {
//    if (!sessionStorage.getItem('token') && window.location.pathname !== '/Login/Login' && window.location.pathname !== '/') {
//        var message = "La sesión ha expirado";
//        validationError(message);
//    }
//}

/**
 * @param {string} dialog ID del dialogo a cerrar.
 */
function closeModal(dialog) {
	$(dialog).modal('hide');
}

//Inicia la pantalla de carga
function startLoading() {
	$('#loadingScreen').show();
}

//Finaliza la pantalla de carga
function completedLoading() {
	$('#loadingScreen').hide();
}

async function logueaORedirecciona() {
	let v_usuario = sessionStorage.getItem('user');
	if (typeof v_usuario === 'undefined') {
		sessionStorage.clear();
		redirectToLogin();
		return;
	}

	let v_password = sessionStorage.getItem('pass'),
		v_bd = sessionStorage.getItem('db');

	let v_ok, v_sid, v_mensaje;

	try {
		({
			Ok: v_ok,
			Objeto: v_sid,
			Mensaje: v_mensaje,
		} = await invocaApi(
			null,
			'/login',
			'POST',
			`=${v_usuario}/${v_password}@${v_bd}`
		));
		if (!v_ok) throw new Error(v_mensaje);
		else sessionStorage.setItem('token', v_sid);
	} catch (ex) {
		sessionStorage.clear();
		redirectToLogin();
		return;
	}

	return v_sid;
}

// ------------------------------------ API ------------------------------------

/**
 * Realiza una llamada API general.
 *
 * @async
 * @param {string} url - La URL a la que llamar.
 * @param {string} method - El método HTTP a utilizar.
 * @param {Object} data - Los datos a enviar con la solicitud.
 * @param {Function} successFunction - La función a llamar si la solicitud es exitosa.
 * @param {Function} loguea - La función a llamar para obtener un token válido.
 * @returns {Promise<void>} Nada
 */
async function apiGeneral(url, method, data, successFunction, loguea) {
	let v_sid = await getValidToken(loguea);
	try {
		const v_respuesta = await invocaApi(v_sid, url, method, data);
		successFunction(v_respuesta);
	} catch ({ xhr, status, error }) {
		handleApiError(
			xhr,
			status,
			error,
			url,
			method,
			data,
			successFunction,
			loguea
		);
	}
}

/**
 * Obtiene un token válido.
 *
 * @async
 * @param {Function} loguea - La función a llamar para obtener un token válido.
 * @returns {Promise<string>} El token válido.
 */
async function getValidToken(loguea) {
	if (sessionStorage.getItem('token') !== null) {
		return getToken();
	} else if (typeof loguea !== 'undefined') {
		return await loguea();
	} else {
		return await logueaORedirecciona();
	}
}

/**
 * Maneja los errores de la API.
 *
 * @async
 * @param {Object} xhr - El objeto XMLHttpRequest.
 * @param {string} status - El estado de la respuesta.
 * @param {string} error - El mensaje de error.
 * @param {string} url - La URL a la que se llamó.
 * @param {string} method - El método HTTP que se utilizó.
 * @param {Object} data - Los datos que se enviaron con la solicitud.
 * @param {Function} successFunction - La función a llamar si la solicitud es exitosa.
 * @param {Function} loguea - La función a llamar para obtener un token válido.
 * @returns {Promise<void>} Nada
 */
async function handleApiError(
	xhr,
	status,
	error,
	url,
	method,
	data,
	successFunction,
	loguea
) {
	switch (xhr?.status) {
		case 401:
			sessionStorage.removeItem('token');
			if (typeof loguea !== 'undefined') {
				return await apiGeneral(url, method, data, successFunction, loguea);
			} else {
				return await apiGeneral(
					url,
					method,
					data,
					successFunction,
					logueaORedirecciona
				);
			}
		case 403:
			lanzarMensaje(false, error, function () {
				window.history.back();
			});
			break;
		default:
			//lanzarMensaje(false, error);
			break;
	}
}

/**
 * Invoca una API.
 *
 * @param {string} p_sid - El SID para la autenticación.
 * @param {string} p_url - La URL de la API a invocar.
 * @param {string} p_metodo - El método HTTP a utilizar.
 * @param {Object} p_datos - Los datos a enviar con la solicitud.
 * @returns {Promise<Object>} La respuesta de la API.
 */
function invocaApi(p_sid, p_url, p_metodo, p_datos) {
	let v_url = p_url.charAt(0) === '/' ? p_url.slice(1) : p_url;

	let v_config = {
		method: p_metodo,
		url: v_apiBaseUrl + v_url,
		headers: {
			app: 'RF',
			sid: p_sid,
		},
		beforeSend: startLoading,
		cache: false,
		complete: completedLoading,
		dataType: 'json',
		error: function (xhr, status, error) {
			completedLoading();
			return Promise.reject({ xhr, status, error });
		},
	};

	if (['POST', 'PUT'].includes(p_metodo)) {
		v_config.contentType = 'application/json';
		v_config.data = JSON.stringify(p_datos);
		if (p_metodo === 'POST') v_config.processData = false;
	} else {
		v_config.data = p_datos;
	}

	return $.ajax(v_config).then(function (p_respuesta) {
		return Promise.resolve(p_respuesta);
	});
}

// ------------------------------------^ API ^------------------------------------

function showDialog(dialog) {
	$(dialog).modal({
		backdrop: 'static',
		keyboard: false,
	});
	$(dialog).css('display', 'table');
}

/**
 * Vincula un elemento del DOM a una función.
 *
 * * Ej:
 * ---------------------------
 * var element = document.getElementById('ejemplo');
 *
 * function funcionEjemplo() {}
 *
 * addEvent(element, 'click', funcionEjemplo, false);
 *
 * @param {Element} elemento - El elemento HTML.
 * @param {string} nomevento - El nombre del evento que desencadena la acción ('click', 'mousedown', ...).
 * @param {function} funcion - El nombre de la función a lanzar o 'function () {funcionesALanzar(params);}'.
 * @param {boolean} captura - Usar captura.
 * @returns {boolean} - Retorna verdadero si el evento fue añadido, falso en caso contrario.
 * @throws {Error} - Lanza un error si no se proporciona una función.
 */
function addEvent(elemento, nomevento, funcion, captura) {
	if (typeof funcion === 'undefined') {
		throw new Error("Se esperaba una función: 'funcion'");
	}

	if (elemento.attachEvent) {
		elemento.attachEvent('on' + nomevento, funcion);
		return true;
	} else if (elemento.addEventListener) {
		elemento.addEventListener(nomevento, funcion, { capture: captura });
		return true;
	} else {
		return false;
	}
}

/**
 * Añade un evento onChange a un elemento del DOM.
 * @param {any} element HTMLElement, id o class de un elemento.
 * @param {any} onChangeFunction Nombre de la función a ejecutar al producirse el cambio.
 */
function addChangeEvent(element, onChangeFunction) {
	$(element).change(function () {
		onChangeFunction();
	});
}

function menuInit() {
	//Inicializa el menu cada vez que se introduce un nuevo pedido dejando el botón de desconexión
	$('#nav-user').find('li:not(:first)').remove();
}

/**
 * Crea un elemento que se añade al menú desplegable.
 *
 * @param {string} name - Nombre del elemento de menú a mostrar.
 * @param {string} icon - Clase del icono a mostrar (sólo el nombre del icono, sin 'glyphicon-').
 * @param {string} href - Ruta.
 * @param {string} value - Valor del input.
 * @param {string} valueIcon - Clase del icono a mostrar para el valor del elemento de menú (sólo el nombre del icono, sin 'glyphicon-').
 * @param {string} id - ID del elemento de menú.
 * @param {string} others - Otras propiedades que pueda necesitar el menú ('data="data"').
 */
function menuElement(name, icon, href, value, valueIcon, id, others) {
	const menuHtml = `
        <li id="${id}" class="menuElement">
            <div class="menuItem">
                <div class="icon-container">
                    <div class="rf-icon rf-${icon}"></div>
                </div>
                <div class="menuItemContent">
                    <label>${name}</label>
                    <input id="${id}-campo" class="invisible-input" readonly value="${value}" ${
		others || ''
	}>
                </div>
                <div class="icon-container">
                    <div class="rf-icon rf-${valueIcon}"></div>
                </div>
            </div>
        </li>
    `;

	$('#sessionElement').before(menuHtml);
}

//function fadeOut(el) {
//    el.style.opacity = 1;
//
//    (function fade() {
//        if ((el.style.opacity -= .1) < 0) {
//            el.style.display = "none";
//        } else {
//            requestAnimationFrame(fade);
//        }
//    })();
//};
//
//function fadeIn(el, display) {
//    el.style.opacity = 0;
//    el.style.display = display || "block";
//
//    (function fade() {
//        var val = parseFloat(el.style.opacity);
//        if (!((val += .1) > 1)) {
//            el.style.opacity = val;
//            requestAnimationFrame(fade);
//        }
//    })();
//};

function openMenu() {
	const v_menuCortina = document.getElementById('menu-cortina');
	//fadeIn(v_menuCortina);
	$(v_menuCortina).fadeIn(500, function () {
		v_menuCortina.classList.remove('desvanece');
	});
	$('.navbar-collapse').addClass('spyNav-open');
}

function closeMenu() {
	const v_menuCortina = document.getElementById('menu-cortina');
	//fadeOut(v_menuCortina);
	v_menuCortina.classList.add('desvanece');

	$(v_menuCortina).fadeOut(500, function () {});
	$('.navbar-collapse').removeClass('spyNav-open');
	tabulado();
}

/**
 * Muestra un mensaje transformando la cabecera de la página y devolviéndolo a la normalidad tras 3 segundos.
 * Esto permite que el usuario siga trabajando sin necesidad de cerrar ninguna ventana de mensaje.
 * @param {string} message - Mensaje a mostrar.
 */
function successMessage(message) {
	const successMessageContainer = $('#successMessage');
	const navbarSpyro = $('.navbar-spyro');
	const originalBackgroundColor = navbarSpyro.css('background-color');

	successMessageContainer.html(`<label>${message}</label>`);

	navbarSpyro.fadeOut(300, function () {
		successMessageContainer.addClass('in');
		navbarSpyro.css('background-color', '#44b549').fadeIn(300);
	});

	setTimeout(() => {
		navbarSpyro.fadeOut(300, function () {
			successMessageContainer.removeClass('in');
			navbarSpyro.css('background-color', originalBackgroundColor).fadeIn(300);
		});
	}, 3000);
}

/**
 * Modifica el color de la fila seleccionada en la tabla para indicar que se ha realizado un cambio
 * @param {string} tableId JQuery id de la tabla.
 */
function cambioColorPedido(tableId) {
	var v_table = $(tableId).DataTable();
	var row = v_table.row('.selected').node();
	$(row).addClass('green');
}

/**
 * Intenta transformar un string en Integer y devolverlo
 * Si no lo consigue, devuelve el número por defecto
 * @param {string} str String a transformar en Integer.
 * @param {number} defaultValue Valor que se devuelve en caso de error.
 *
 * @returns {number} Número entero
 */
function TryParseInt(str, defaultValue) {
	var retValue = defaultValue;
	if (str && !isNaN(str)) {
		retValue = parseInt(str);
	}
	return retValue;
}

/**
 * Intenta transformar un string en float y devolverlo.
 * Si no lo consigue, devuelve el número por defecto.
 * @param {string} str Cadena a transformar.
 * @param {number} defaultValue Número que devuelve en caso de error.
 *
 * @returns {number} Sin error: número obtenido del str. Con error: número por defecto.
 */
function TryParseFloat(str, defaultValue) {
	var retValue = defaultValue;
	if (str) {
		str = str.replace(',', '.');
		if (str.trim().length > 0) {
			if (!isNaN(str)) {
				retValue = parseFloat(str);
			}
		}
	}
	return retValue;
}

/**
 * Busca y selecciona una fila de la tabla mediante la variable JSON
 * de la columna y el valor a buscar.
 * @param {string} table ID de la tabla en formato jQuery <"#nombreTabla"> o elemento de tabla
 * @param {Object[]} jsonArray Array de objetos con la estructura {KEY: "clave", VALUE: "valor"}
 * @returns {boolean} Retorna verdadero si se encuentra la fila, falso en caso contrario
 */
function seleccionFila(table, jsonArray) {
	const v_table = $(table).DataTable();
	let encontrado = false;

	if (v_table.rows().count() !== 0) {
		v_table.row('.selected').nodes().to$().removeClass('selected');

		v_table
			.rows(function (idx, data) {
				encontrado = jsonArray.every((json) => data[json.KEY] === json.VALUE);

				if (encontrado) {
					const v_pag = Math.floor(idx / v_table.page.len());
					v_table.page(v_pag).draw('page');
				}

				return encontrado;
			})
			.nodes()
			.to$()
			.addClass('selected');

		if (v_table.row('.selected').nodes().count() !== 0) {
			const v_row = $(v_table.row('.selected').node());
			$(`${table}_wrapper .dataTables_scrollBody`).animate(
				{
					scrollTop:
						v_row.offset().top -
						$(`${table}_wrapper .dataTables_scrollBody`).offset().top,
				},
				300
			);
		}
	}

	console.log(encontrado);
	return encontrado;
}

/**
 * Obtiene los datos de una fila de la tabla mediante la variable JSON
 * de la columna y el valor a buscar.
 * @param {string} table ID de la tabla en formato jQuery <"#nombreTabla"> o elemento de tabla
 * @param {Object[]} jsonArray Array de objetos con la estructura {KEY: "clave", VALUE: "valor"}
 * @returns {Object|null} Retorna los datos de la fila si se encuentra, null en caso contrario
 */
function dameDatosFila(table, jsonArray) {
	const v_table = $(table).DataTable();
	let line = null;

	if (v_table.rows().count() !== 0) {
		v_table.rows(function (idx, data) {
			const encontrado = jsonArray.every(
				(json) => data[json.KEY] === json.VALUE
			);

			if (encontrado) {
				line = data;
				return false; // Stop processing further rows
			}
		});
	}

	return line;
}

/**
 * Obtiene el índice de una fila de la tabla mediante la variable JSON
 * de la columna y el valor a buscar.
 * @param {string} table ID de la tabla en formato jQuery <"#nombreTabla"> o elemento de tabla
 * @param {Object[]} jsonArray Array de objetos con la estructura {KEY: "clave", VALUE: "valor"}
 * @returns {number|null} Retorna el índice de la fila si se encuentra, null en caso contrario
 */
function dameIndexFila(table, jsonArray) {
	const v_table = $(table).DataTable();
	let line = null;

	if (v_table.rows().count() !== 0) {
		v_table.rows(function (idx, data) {
			const encontrado = jsonArray.every(
				(json) => data[json.KEY] === json.VALUE
			);

			if (encontrado) {
				line = idx;
				return false; // Stop processing further rows
			}
		});
	}

	return line;
}

/**
 * Hace focus en el siguiente elemento '.focusable' al presionar la tecla Enter
 * Sin 'dialogId': Obtiene todos los elementos '.focusable'
 * Con 'dialogId': Obtiene los elementos '.focusable' del dialog introducido
 * @param {string} dialogId JQuery id de la ventana modal
 * @returns {object} objeto para controlar la tabulación
 */
function tabulado(dialogId) {
	const v_tabulado = {
		siguiente: (function () {
			let v_siguiente,
				v_navegacion = null,
				v_temporizador;

			function estableceFoco() {
				if (v_siguiente) {
					console.log(
						'foco por ' + v_navegacion + ' en ' + v_siguiente.id ||
							v_siguiente.tagName
					);
					$(v_siguiente).focus();
					v_siguiente = v_navegacion = v_temporizador = null;
				}
			}

			return function (p_siguiente, p_teclado = false) {
				if (p_siguiente instanceof jQuery) {
					p_siguiente = p_siguiente[0];
				}

				if (p_siguiente) {
					if (v_navegacion === null) {
						v_navegacion = p_teclado ? 'teclado' : 'proceso';
						v_siguiente = p_siguiente;
					} else if (v_navegacion === 'teclado' && !p_teclado) {
						// Se cancela la navegacion por teclado
						v_navegacion = 'proceso';
						v_siguiente = p_siguiente;
					} else if (v_navegacion === 'proceso' && !p_teclado) {
						// Se navegaba por proceso y se quiere navegar por proceso a otro elemento
						v_siguiente = p_siguiente;
					}

					if (v_temporizador) {
						clearTimeout(v_temporizador);
					}
					v_temporizador = setTimeout(estableceFoco.bind(this), 0);
				} else {
					v_siguiente = null;
					v_navegacion = null;
				}
			};
		})(),
	};

	const focusableInputs = () =>
		!dialogId ? $('.focusable') : $(dialogId + ' .focusable');

	focusableInputs().attr('autocomplete', 'off');
	const getInputs = focusableInputs();
	if (getInputs.length > 0) {
		setTimeout(() => nextFocus(getInputs), 0);
	}

	document.addEventListener('keydown', function (e) {
		if (e.keyCode === 9 || e.keyCode === 13) {
			if (e.keyCode === 9) {
				if (e.shiftKey) {
					previousFocus($(e.target));
				} else {
					nextFocus($(e.target));
				}
				e.preventDefault();
			} else if (e.keyCode === 13) {
				nextFocus($(e.target));
			}
		}
	});

	document.addEventListener('keyup', function (e) {
		if (e.keyCode === 13 || e.keyCode === 9) {
			e.preventDefault();
		}
	});

	function determinaFocusables(p_contenedor) {
		return p_contenedor.querySelectorAll('*:not([disabled]).focusable');
	}

	function siguienteFocusable(p_elemento) {
		if (typeof p_elemento === 'undefined') {
			return determinaFocusables(document)[0];
		}

		const v_elemento = p_elemento.get(0) || p_elemento;
		let v_contenedor = v_elemento.closest('.formulario,.modal');

		if (v_contenedor === null) v_contenedor = document;

		const v_focusables = determinaFocusables(v_contenedor);

		const focussable = Array.prototype.filter.call(
			v_focusables,
			function (element) {
				return (
					element.offsetWidth > 0 ||
					element.offsetHeight > 0 ||
					element === v_elemento
				);
			}
		);
		const index = focussable.indexOf(v_elemento);
		if (index > -1) {
			const nextElement = focussable[index + 1] || focussable[0];
			return nextElement;
		}
	}

	function nextFocus(input) {
		input = siguienteFocusable(input);
		if (typeof input !== 'undefined') {
			v_tabulado.siguiente(input, true);
		}
	}

	document.siguienteFocusable = siguienteFocusable;

	function previousFocus(input) {
		const v_focusables = focusableInputs();
		input = input ? $(input) : $(v_focusables[v_focusables.length - 1]);
		const originInput = input.get(0);

		let currentInput;
		do {
			currentInput = input.get(0);
			const previousIndex = v_focusables.index(input) - 1;
			const adjustedIndex =
				previousIndex < 0 ? v_focusables.length - 1 : previousIndex;
			input = $(v_focusables[adjustedIndex]);
		} while (
			originInput !== input.get(0) &&
			currentInput !== input.get(0) &&
			(input.prop('readonly') ||
				input.prop('disabled') ||
				!input.is(':visible'))
		);

		v_tabulado.siguiente(input, true);
	}

	focusableInputs().on('focus', function () {
		$(this).select();
	});

	return v_tabulado;
}

function autoselect(dialogId) {
	var inputs = !dialogId
		? $('input:not(.invisible-input,:disabled)')
		: $(dialogId + ' input:not(.invisible-input,:disabled)');

	for (var i = 0; i < inputs.length; i++) {
		$(inputs[i]).on('focus', function () {
			$(this).select();
		});
	}
}

/**
 * Añade flechas que permiten navegar entre los registros
 *
 * @param {string} table - El id de la tabla en JQuery
 */
function navigateArrows(table) {
	const $dataTable = $(table).dataTable();
	const dataTable = $(table).DataTable();
	const nextButton =
		'<a class="paginate_button nextRow"><i class="glyphicon glyphicon-arrow-down gi-2x"/></a>';
	const previousButton =
		'<a class="paginate_button previousRow"><i class="glyphicon glyphicon-arrow-up gi-2x"/></a>';
	const paginationFirstButton = $(`${table}_paginate a:first-child`);

	paginationFirstButton.after(nextButton, previousButton);

	const previousRow = $(`${table}_paginate .previousRow`);
	const nextRow = $(`${table}_paginate .nextRow`);
	const nextPage = $(`${table}_paginate .next`);
	const prevPage = $(`${table}_paginate .previous`);

	$(`${table}_paginate`).appendTo($(`${table}_wrapper .dataTables_scroll`));

	// Botón linea siguiente
	nextRow.on('click', function () {
		if (!nextRow.hasClass('disabled')) {
			let selectedRowIndex = $dataTable.find('.selected').index();
			let nextIndex = selectedRowIndex + 1;
			let next = $dataTable.find(`tbody tr:eq(${nextIndex})`);

			disableRow(nextIndex);

			if (
				nextIndex === $dataTable.find('tbody').find('tr').length &&
				!nextPage.hasClass('disabled')
			) {
				nextPage.click();
				next = $dataTable.find('tbody tr:eq(0)');
				dataTable.row('.selected').nodes().to$().removeClass('selected');
				next.addClass('selected');
			} else {
				dataTable.row('.selected').nodes().to$().removeClass('selected');
				next.addClass('selected');
			}

			const v_row = $(dataTable.row('.selected').node());
			let scrollBody = $(`${table}_wrapper .dataTables_scrollBody`);
			let v_row_offset = v_row && v_row.offset ? v_row.offset() : null;
			let scrollBody_offset =
				scrollBody && scrollBody.offset ? scrollBody.offset() : null;

			if (v_row_offset && scrollBody_offset) {
				scrollBody.animate(
					{
						scrollTop:
							v_row_offset.top + v_row.height() - 2 * scrollBody_offset.top,
					},
					100
				);
			}
			next.trigger('mousedown');
		}
	});

	// Botón linea anterior
	previousRow.on('click', function () {
		if (!previousRow.hasClass('disabled')) {
			let selectedRowIndex = $dataTable.find('.selected').index();
			let prevIndex = selectedRowIndex - 1;
			let prev = $dataTable.find(`tbody tr:eq(${prevIndex})`);

			if (prevIndex < 0 && !prevPage.hasClass('disabled')) {
				prevPage.click();
				prev = $dataTable.find('tbody tr:eq(-1)');
				dataTable.row('.selected').nodes().to$().removeClass('selected');
				prev.addClass('selected');
				selectedRowIndex = $dataTable.find('.selected').index();
				prevIndex = selectedRowIndex - 1;
			} else {
				dataTable.row('.selected').nodes().to$().removeClass('selected');
				prev.addClass('selected');
			}

			disableRow(prevIndex);

			const v_row = $(dataTable.row('.selected').node());
			let scrollBody = $(`${table}_wrapper .dataTables_scrollBody`);
			let v_row_offset = v_row && v_row.offset ? v_row.offset() : null;
			let scrollBody_offset =
				scrollBody && scrollBody.offset ? scrollBody.offset() : null;

			if (v_row_offset && scrollBody_offset) {
				scrollBody.animate(
					{
						scrollTop: v_row_offset.top - scrollBody_offset.top,
					},
					100
				);
			}
			prev.trigger('mousedown');
		}
	});

	/**
	 * Deshabilita o habilita los botones de linea siguiente/anterior
	 * en funcion de los elementos que haya en la tabla.
	 *
	 * @param {number} rowIndex - El índice de la fila seleccionada
	 */
	function disableRow(rowIndex) {
		const pageLength = $dataTable.find('tbody').find('tr').length;
		const disablePrev = rowIndex <= 0;
		const disableNext = rowIndex === pageLength - 1;

		const nextPage = $(`${table}_paginate .next`).hasClass('disabled')
			? false
			: true;
		const prevPage = $(`${table}_paginate .previous`).hasClass('disabled')
			? false
			: true;

		if (disableNext && nextPage) {
			disableNext = false;
		}
		disableNext
			? nextRow.addClass('disabled')
			: nextRow.removeClass('disabled');

		if (disablePrev && prevPage) {
			disablePrev = false;
		}
		disablePrev
			? previousRow.addClass('disabled')
			: previousRow.removeClass('disabled');
	}
}

/**
 * Obtiene el número de registros que se deben mostrar en la tabla dependiendo del alto de esta.
 * Ajusta el alto al alto de pantalla teniendo en cuenta otros elementos que pueda haber de por medio.
 * Calcula la página que debería mostrarse al cambiar el número de registros.
 * TODO AAZ: Realizar el calculo con el registro seleccionado en lugar de hacerlo con el primer registro.
 *
 * @returns {Object} Objeto con la función applyFilters que se aplica a la tabla cada vez que ocurre un resize.
 **/
function SpyFilter() {
	return {
		applyFilters: function (tableId) {
			setTimeout(function () {
				var dataTable = $(tableId).DataTable();
				var oldLength = dataTable.page.len();

				// Si oldLength es null o undefined, no se ejecuta el resto del código
				if (!oldLength) return;

				var oldPage = dataTable.page.info().page + 1;
				var tableHeight = $(
					tableId + '_wrapper .dataTables_scrollBody'
				).height();
				var row = $(tableId + ' tbody tr').eq(0);
				var rowHeight = row ? row.height() : 11;

				// tableHeight - 0.1 => para ocultar el último registro cuando la tabla sea del mismo tamaño que la suma de los registros.
				// Sin esto, el alto mínimo de la tabla queda fijada al alto de los registros.
				var pageLength = Math.floor((tableHeight - 0.1) / rowHeight);
				pageLength = pageLength === 0 ? 1 : pageLength;

				if (oldLength !== pageLength) {
					dataTable.page.len(1).draw();

					// Cálculo de página (resize)

					// Obtiene el primer registro de la paginación antes del resize
					// TODO AAZ Sería preferible que coger el registro seleccionado
					var initialRegister = oldLength * oldPage - (oldLength - 1);

					// Obtiene la página en la que debería aparecer el registro con la nueva longitud de registros
					var newPage = Math.ceil(initialRegister / pageLength);
					dataTable.page.len(pageLength).draw();
					dataTable.page(newPage - 1).draw('page');
				}
			}, 0);
		},
	};
}

/**
 *
 * @param {number} pendingQty Cantidad disponible para sacar
 * @param {number} qty Cantidad total a servir
 *
 * @return {number} Cantidad a sacar de la ubicación seleccionada.
 */
function setOptimumQty(pendingQty, qty) {
	var v_num = pendingQty > qty ? qty : pendingQty;
	if (v_num < 0) v_num = 0;
	return v_num;
}

/**
 * @param {any} p_tabla Referencia a la tabla
 * @param {string} p_nombreColumna Nombre de la columna
 * Función para hacer modificables las columnas de las DataTable
 */
function ModifyCellValue(p_tabla, p_nombreColumna) {
	this.Active = true;
	this.Table = $(p_tabla).DataTable();
	this.Column = p_nombreColumna;
	this.AfterChangeArray = [];

	this.toggleActive = function () {
		this.Active = !this.Active;
		this.ChangeColumnColor();
	};

	this.AddDrawCallback = function () {
		var self = this;
		var DrawCallBack = {
			fn: function () {
				self.ChangeColumnColor();
			},
			sName: 'Colour',
		};
		$(p_tabla).dataTable().fnSettings().aoDrawCallback.push(DrawCallBack);
	};

	this.AfterChange = function (p_afterChangeFn) {
		this.AfterChangeArray.push(p_afterChangeFn);
	};

	this.ChangeColumnColor = function () {
		var _this = this;
		var columns = _this.Table.settings().init().aoColumns;

		columns.forEach(function (col, i) {
			if (col.mDataProp === _this.Column) {
				_this.Table.columns(i)
					.nodes()[0]
					.forEach(function (n) {
						$(n).toggleClass('green', _this.Active);
					});
			}
		});
	};

	this.AddDrawCallback();

	/**
	 * Hace que los valores de una columna se puedan modificar.
	 *
	 * @param {Function} p_onchange Función lanzada al modificar el valor (callback => newValue, oldValue, row).
	 * @param {string} p_type Indica el tipo de dato (String, Int, Float...)
	 * @param {boolean} UpperCase Indica si debe ir escrito en mayúsculas
	 */
	this.modCellValue = function (p_onchange, p_type = 'Int', UpperCase = false) {
		var self = this;
		var columns = self.Table.settings().init().aoColumns;

		$(self.Table.table().container()).on('click', 'td', function (e) {
			var cell = self.Table.cell(this);
			var index = cell.index();
			var $cell = $(this);
			var idCell = (setId = true) =>
				setId
					? '#modInput' + index.row + '' + index.column
					: 'modInput' + index.row + '' + index.column;

			if (columns[index.column]['mDataProp'] === self.Column && self.Active) {
				e.stopImmediatePropagation();
				if (!$(this).find('input').length) {
					var v_cellValue = $cell[0].innerHTML;
					var upperClass = UpperCase ? 'code' : '';
					var inputHTML = `<input id=${idCell(
						false
					)} class="col-xs-12 form-control input-sm input-xs ${upperClass} text-right" value="${v_cellValue}" />`;
					$cell[0].innerHTML =
						p_type.toUpperCase() !== 'STRING' ? inputHTML : $cell[0].innerHTML;
					$(idCell()).focus().select();
					$(idCell()).on('click', function () {
						$(this).select();
					});
					$(idCell()).keypress(function (e) {
						if (e.which === 13) {
							$(idCell()).blur();
						}
					});

					var blurFn = function () {
						if (RemovableFocus) {
							var data = self.Table.row(index.row).data();
							var oldValue = v_cellValue;
							var newValue = $(this).val();
							if (p_type.toUpperCase() === 'INT')
								data[self.Column] = TryParseInt(newValue, oldValue);
							else if (p_type.toUpperCase() === 'FLOAT')
								data[self.Column] = TryParseFloat(newValue, oldValue);
							else
								UpperCase
									? (data[self.Column] = newValue.toUpperCase())
									: (data[self.Column] = newValue);

							self.Table.row(index.row).invalidate();
							self.AfterChangeArray.forEach(function (fn) {
								fn();
							});
						}
					};
					$(idCell()).on('blur', blurFn);

					var resizeTimer;
					var RemovableFocus = true;
					window.onresize = function () {
						if (resizeTimer) {
							clearTimeout(resizeTimer);
						} else {
							RemovableFocus = false;
							$(idCell()).off('blur', blurFn);
						}
						resizeTimer = setTimeout(function () {
							resizeTimer = null;
							RemovableFocus = true;

							$(idCell()).on('blur', blurFn);
						}, 500);
					};

					$(idCell()).on('change', function () {
						var oldValue = v_cellValue;
						var newValue = $(this).val();
						data = self.Table.row(index.row).data();
						newValue = UpperCase ? newValue.toUpperCase() : newValue;
						if (p_onchange) p_onchange(newValue, oldValue, index.row);
					});
				}
			}
		});
	};
}
