define(["require", "exports", "Utilidades/assert", "componentes/foco"], function (require, exports, assert, foco_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.tabla = exports.Tabla = exports.DefinicionTabla = void 0;
    class FormateadorFila {
        constructor(p_dataTable, p_row, p_displayNum, p_displayIndex, p_dataIndex) {
            this.dataTable = p_dataTable;
            this.row = p_row;
            this.displayNum = p_displayNum;
            this.displayIndex = p_displayIndex;
            this.dataIndex = p_dataIndex;
        }
        fondo(p_color) {
            const v_colores = { 'verde': 'green', 'rojo': 'red' };
            const v_color = v_colores[p_color] || `fondo-${p_color}`;
            this.row.classList.add(v_color);
        }
    }
    class FormateadorCelda {
        constructor(p_td) {
            this.td = p_td;
        }
        fondo(p_color) {
            const v_colores = { 'verde': 'fondo-verde', 'rojo': 'fondo-rojo' };
            const v_color = v_colores[p_color] || `fondo-${p_color}`;
            this.td.classList.add(v_color);
        }
    }
    class DefinicionTabla {
        constructor(_selectorTabla, _valores) {
            this._selectorTabla = _selectorTabla;
            this._valores = _valores;
        }
        datos(p_datos) {
            if (arguments.length === 1) {
                const v_datos = Array.isArray(p_datos) ? p_datos : [];
                return new DefinicionTabla(this._selectorTabla, { ...this._valores, datos: v_datos });
            }
            else
                return this._valores.datos;
        }
        columnas(p_columnas) {
            if (arguments.length === 1) {
                const v_columnas = Array.isArray(p_columnas) ? p_columnas : [];
                return new DefinicionTabla(this._selectorTabla, { ...this._valores, columnas: v_columnas });
            }
            else
                return this._valores.columnas;
        }
        opciones(p_opciones) {
            if (arguments.length === 1)
                return new DefinicionTabla(this._selectorTabla, { ...this._valores, opciones: p_opciones });
            else
                return this._valores.opciones;
        }
        filtro(p_filtro) {
            if (arguments.length === 1)
                return new DefinicionTabla(this._selectorTabla, { ...this._valores, filtro: p_filtro });
            else
                return this._valores.filtro;
        }
        seleccionado(p_seleccionado) {
            if (arguments.length === 1)
                return new DefinicionTabla(this._selectorTabla, { ...this._valores, seleccionado: p_seleccionado });
            else
                return this._valores.seleccionado;
        }
        formatoFila(p_formato) {
            if (arguments.length === 1)
                return new DefinicionTabla(this._selectorTabla, { ...this._valores, formatoFila: p_formato });
            else
                return this._valores.formatoFila;
        }
        crea() {
            const v_tabla = new Tabla(this._selectorTabla, this._valores);
            v_tabla.filtro(this._valores.filtro);
            return v_tabla;
        }
    }
    exports.DefinicionTabla = DefinicionTabla;
    class Tabla {
        constructor(_selectorTabla, _valores) {
            this._selectorTabla = _selectorTabla;
            this._valores = _valores;
            this._nuevasFunciones = {};
            let v_opciones = typeof _valores.opciones === 'undefined' ? {} : _valores.opciones;
            let v_columnasDataTable = Tabla._creaColumnasDataTable(_valores.columnas, v_opciones);
            let v_definicionesDataTable = Tabla._creaDefinicionesDataTable(_valores.columnas);
            let v_opcionesDataTable = {
                "ordering": v_definicionesDataTable.ordenaciones ? true : false,
                "order": v_definicionesDataTable.ordenaciones,
                "pagingType": "simple",
                "data": _valores.datos,
                "autoWidth": true,
                "columns": v_columnasDataTable,
                "columnDefs": v_definicionesDataTable.columnas,
                "fixedHeader": true,
                "responsive": false,
                "scrollY": false,
                "scrollCollapse": true,
                "scrollX": false,
                "deferRender": true,
                "info": false,
                'select': {
                    style: 'single',
                    items: 'row',
                    toggleable: true,
                    className: 'selected'
                },
                "drawCallback": () => {
                    navigateArrows(`#${this._selectorTabla.replace(/\./g, '\\.')}`);
                },
                "initComplete": (settings, json) => {
                    SpyFilter().applyFilters(`#${this._selectorTabla.replace(/\./g, '\\.')}`);
                },
                "rowCallback": function (p_tr, p_objeto, p_displayNum, p_displayIndex, p_dataIndex) {
                    if (typeof _valores.formatoFila === 'function') {
                        const v_fila = new FormateadorFila(this.api(), p_tr, p_displayNum, p_displayIndex, p_dataIndex);
                        _valores.formatoFila(v_fila, p_objeto);
                    }
                    const vColumnasVisibles = _valores.columnas.filter(pColumna => pColumna.visible !== false);
                    for (let v_indice = 0; v_indice < vColumnasVisibles.length; v_indice++) {
                        const v_columna = vColumnasVisibles[v_indice];
                        const v_td = p_tr.cells[v_indice];
                        if (_valores.editable && v_columna.editable)
                            v_td.classList.add('green');
                        else
                            v_td.classList.remove('green');
                        if (typeof v_columna.formatoCelda === 'function') {
                            const v_celda = new FormateadorCelda(v_td);
                            const v_propiedad = v_columna.propiedad;
                            const v_valor = p_objeto[v_propiedad];
                            v_columna.formatoCelda(v_celda, p_objeto, v_propiedad, v_valor);
                        }
                    }
                    return;
                },
                "language": {
                    "sProcessing": "Procesando...",
                    "sLengthMenu": Resources.table_length_menu,
                    "sZeroRecords": "No se encontraron resultados",
                    "sEmptyTable": _valores.mensajeSinRegistros || Resources.table_no_data,
                    "sInfo": Resources.table_sinfo,
                    "sInfoEmpty": Resources.table_sinfo_empty,
                    "sInfoFiltered": "(filtrado de un total de _MAX_ registros)",
                    "sInfoPostFix": "",
                    "sSearch": Resources.table_search + ":",
                    "sUrl": "",
                    "sInfoThousands": ",",
                    "sLoadingRecords": Resources.table_loading_records,
                    "paginate": {
                        "first": "Primero",
                        "last": "Último",
                        "next": '<i class="glyphicon glyphicon-arrow-right gi-2x"/>',
                        "previous": '<i class="glyphicon glyphicon-arrow-left gi-2x"/>',
                    },
                    "aria": {
                        "sSortAscending": ": Activar para ordenar la columna de manera ascendente",
                        "sSortDescending": ": Activar para ordenar la columna de manera descendente"
                    }
                }
            };
            this._dataTable = $(`#${_selectorTabla.replace(/\./g, '\\.')}`)
                .on('error.dt', (e, settings, techNote, message) => {
                lanzarMensaje(false, "Ha ocurrido un error de DT: \n\n" + message);
            })
                .DataTable(v_opcionesDataTable);
            $('.dataTables_filter input').addClass('pull-right form-control input-xs');
            this._dataTable.page.len(1).draw();
            document.getElementById(_selectorTabla).addEventListener('click', p_evento => {
                const v_elementoTd = p_evento.target.closest('td'), vColumnasVisibles = this._valores.columnas.filter(pColumna => pColumna.visible !== false), v_columna = vColumnasVisibles[v_elementoTd.cellIndex];
                if (typeof v_columna?.opcion !== 'string')
                    return;
                const v_opcion = _valores.opciones[v_columna.opcion];
                if (typeof v_opcion.accion === 'function') {
                    const v_elementoTr = v_elementoTd.closest('tr'), v_fila = this._dataTable.row(v_elementoTr).data();
                    v_opcion.accion(v_fila);
                    p_evento.preventDefault();
                    p_evento.stopPropagation();
                }
            });
            this._dataTable.on('select', (p_evento, p_dataTable, p_tipo, p_indices) => {
                if (typeof this._valores.seleccionado === 'function') {
                    let v_objetos = this.seleccionado();
                    this._valores.seleccionado(v_objetos);
                }
            });
            this._dataTable.on('deselect', (p_evento, p_dataTable, p_tipo, p_indices) => {
                if (typeof this._valores.seleccionado === 'function')
                    this._valores.seleccionado(null);
            });
            this._dataTable.on('user-select', (p_evento, p_dataTable, p_tipo, p_celdas, p_eventoOriginal) => {
                let v_indice = p_celdas[0][0];
                const vColumnasVisibles = this._valores.columnas.filter(pColumna => pColumna.visible !== false);
                let v_definicionColumna = vColumnasVisibles[v_indice.column];
                if (this._valores.editable && v_definicionColumna?.editable) {
                    let v_celda = p_dataTable.cell(v_indice.row, v_indice.column);
                    this.configuraEditorCelda(v_celda.node(), v_celda);
                    p_evento.preventDefault();
                }
            });
            let vExisteAnchoColumna = false;
            v_definicionesDataTable.columnas.forEach(item => {
                if (item.width && item.width !== 'auto') {
                    vExisteAnchoColumna = true;
                    return;
                }
            });
            if (vExisteAnchoColumna) {
                $(document.getElementById(_selectorTabla + "_wrapper")).addClass('layoutFijo');
                window.dispatchEvent(new Event('resize'));
            }
        }
        datos(p_datos, extraRowFormatCallback = null) {
            let vCurrentPage = 0;
            let vSelectedRow = this.seleccionado();
            if (p_datos === this._arrayDatos) {
                this.refresca();
                return;
            }
            if (typeof this._arrayDatos !== 'undefined' && this._arrayDatos !== null && typeof this._funcionesSuplantadas !== 'undefined' && this._funcionesSuplantadas !== null) {
                Object.getOwnPropertyNames(this._funcionesSuplantadas)
                    .forEach(p_nombre => this._arrayDatos[p_nombre] = this._funcionesSuplantadas[p_nombre]);
                this._funcionesSuplantadas = void 0;
            }
            if (typeof p_datos !== 'undefined' && p_datos !== null) {
                this._funcionesSuplantadas = {};
                Object.getOwnPropertyNames(this._nuevasFunciones)
                    .forEach(p_nombre => {
                    this._funcionesSuplantadas[p_nombre] = p_datos[p_nombre];
                    p_datos[p_nombre] = this._nuevasFunciones[p_nombre];
                });
            }
            vCurrentPage = this._dataTable.page();
            this._dataTable.clear();
            if (Array.isArray(p_datos)) {
                for (let i = 0; i < p_datos.length; i++) {
                    this._dataTable.row.add(p_datos[i]);
                    if (extraRowFormatCallback) {
                        this._dataTable.row(i).child(extraRowFormatCallback(this._dataTable.row(i).data())).show();
                    }
                }
            }
            this._dataTable.draw();
            if (vCurrentPage > this._dataTable.page.info().pages - 1)
                vCurrentPage = this._dataTable.page.info().pages - 1;
            let vEncontrados = this.buscaYSelecciona(vSelectedRow, { notificar: false });
            this._dataTable.page(vCurrentPage).draw('page');
            this._arrayDatos = p_datos;
            return this;
        }
        refresca() {
            const v_pagina = this._dataTable.page();
            this._dataTable.rows().invalidate();
            this._dataTable.draw();
            this._dataTable.page(v_pagina).draw('page');
        }
        filtro(p_filtro) {
            if (typeof p_filtro !== 'undefined') {
                this._dataTable.search(p_filtro).draw();
            }
        }
        seleccionado() {
            let v_objetos = Array.from(this._dataTable.rows({ selected: true }).data());
            assert.assert(v_objetos.length <= 1);
            return v_objetos[0];
        }
        ;
        ocultarColumna(propiedad) {
            this.visualizarColumna(propiedad, false);
        }
        mostrarColumna(propiedad) {
            this.visualizarColumna(propiedad, true);
        }
        visualizarColumna(propiedad, visible) {
            const vIndex = this._valores.columnas.findIndex(pColumna => pColumna.propiedad === propiedad);
            const vColumn = this._dataTable.column(vIndex);
            if (vColumn)
                vColumn.visible(visible);
        }
        busca(p_objeto) {
            let v_filas = this._dataTable.rows()[0];
            let v_objeto = Object.getOwnPropertyNames(p_objeto)
                .reduce((p_anterior, p_propiedad) => {
                let v_valor = p_objeto[p_propiedad];
                if (typeof v_valor === 'string' && v_valor.length === 0) {
                }
                else
                    p_anterior[p_propiedad] = p_objeto[p_propiedad];
                return p_anterior;
            }, {});
            let v_encontrados = v_filas
                .filter(p_fila => Object.getOwnPropertyNames(v_objeto).every(p_propiedad => p_objeto[p_propiedad] === this._dataTable.row(p_fila).data()[p_propiedad]))
                .map(p_indiceEncontrado => ({ Indice: p_indiceEncontrado, Fila: this._dataTable.row(p_indiceEncontrado).data() }));
            return Array.from(v_encontrados);
        }
        selecciona(p_argumento, p_opciones) {
            const v_alSeleccionar = this._valores.seleccionado;
            this._valores.seleccionado = null;
            try {
                const v_opciones = p_opciones || {}, v_notificar = v_opciones.notificar !== null && typeof v_opciones.notificar !== 'undefined' ? v_opciones.notificar : true;
                this._dataTable.rows().deselect();
                this._dataTable.row('.selected').nodes().to$().removeClass('selected');
                let v_seleccionado;
                if (arguments.length !== 0) {
                    if (Array.isArray(p_argumento)) {
                        let v_indices = p_argumento;
                        v_indices.forEach(p_indice => this._dataTable.rows(p_indice).select());
                        if (v_notificar && typeof v_alSeleccionar === 'function')
                            v_seleccionado = this.seleccionado();
                    }
                    else if (p_argumento !== null) {
                        let v_indice = p_argumento;
                        this._dataTable.rows(v_indice).select();
                        if (v_notificar && typeof v_alSeleccionar === 'function')
                            v_seleccionado = this.seleccionado();
                    }
                }
                if (v_notificar && typeof v_alSeleccionar === 'function')
                    v_alSeleccionar(v_seleccionado);
            }
            finally {
                this._valores.seleccionado = v_alSeleccionar;
            }
        }
        buscaYSelecciona(p_objeto, p_opciones) {
            if (p_objeto) {
                const v_opciones = p_opciones || {}, v_seleccionaPrimero = v_opciones.seleccionaPrimero || false;
                let v_encontrados = this.busca(p_objeto);
                if (v_seleccionaPrimero) {
                    if (v_encontrados.length > 0) {
                        this.selecciona(v_encontrados[0].Indice, p_opciones);
                        return v_encontrados;
                    }
                    else {
                        this.selecciona(null, p_opciones);
                        return;
                    }
                }
                else {
                    if (v_encontrados.length > 0) {
                        this.selecciona(v_encontrados.map(p_encontrado => p_encontrado.Indice), p_opciones);
                        return v_encontrados;
                    }
                    return;
                }
            }
            else
                this.selecciona(null, p_opciones);
        }
        editable(p_editable) {
            if (arguments.length === 0)
                return this._valores.editable;
            else {
                this._valores.editable = p_editable;
                this.refresca();
            }
        }
        configuraEditorCelda(p_td, p_celda) {
            let v_indice = p_celda.index();
            const vColumnasVisibles = this._valores.columnas.filter(pColumna => pColumna.visible !== false);
            let v_definicionColumna = vColumnasVisibles[v_indice.column];
            let v_valorAnterior = p_celda.data();
            function actualizaDatos(p_input) {
                let v_valor;
                switch (v_input.type) {
                    case 'number':
                        v_valor = Number(p_input.value);
                        break;
                    default:
                        v_valor = p_input.value;
                        break;
                }
                let v_fila = this._dataTable.row(v_indice.row).data();
                let v_propiedad = v_definicionColumna.propiedad;
                v_fila[v_propiedad] = v_valor;
                if (typeof v_definicionColumna.cambio === 'function' && v_valor !== v_valorAnterior) {
                    v_definicionColumna.cambio(v_valor);
                    v_valorAnterior = v_valor;
                }
            }
            ;
            let v_input = document.createElement('input');
            v_input.style = "width:100%;height:100%;";
            v_input.value = p_celda.data();
            if (v_definicionColumna.numerico)
                v_input.type = 'number';
            v_input.classList.add('form-control', 'input-sm', 'input-xs');
            v_input.classList.add('focusable');
            if (v_definicionColumna.numerico)
                v_input.classList.add('derecha');
            const v_seleccionado = this._valores.seleccionado;
            this._valores.seleccionado = void 0;
            const v_manejador = {
                handleEvent: function (p_evento) {
                    switch (p_evento.type) {
                        case 'blur': {
                            p_evento.target.remove();
                            actualizaDatos.bind(this)(p_evento.target);
                            let v_valor = p_evento.target.value;
                            p_td.innerText = v_valor;
                            this._valores.seleccionado = v_seleccionado;
                            break;
                        }
                        case 'keydown': {
                            if (p_evento.key === 'Enter') {
                                p_evento.target.blur();
                                p_evento.stopPropagation();
                            }
                            break;
                        }
                    }
                }.bind(this)
            };
            v_input.addEventListener('blur', v_manejador);
            v_input.addEventListener('keydown', v_manejador);
            p_td.firstChild.remove();
            p_td.appendChild(v_input);
            foco_1.foco.en(v_input);
        }
        static _generateColumn(p_propiedad, p_titulo, p_visible, p_numerico, p_clase, p_opcion) {
            p_clase = p_clase || '';
            if (p_numerico)
                p_clase += ' dt-right';
            p_visible = typeof p_visible === 'undefined' || p_visible === null || p_visible ? true : false;
            const data = { 'name': p_titulo, 'title': p_titulo, "bVisible": p_visible };
            if (p_propiedad)
                data.mDataProp = p_propiedad;
            if (p_clase !== "")
                data.sClass = p_clase;
            if (p_opcion) {
                const v_boton = Tabla._botones[p_opcion.boton];
                data.defaultContent = v_boton.contenido;
            }
            return data;
        }
        static _creaColumnasDataTable(p_columnas, p_opciones) {
            const v_clasesAlineacion = {
                derecha: 'dt-right',
                centro: 'dt-center',
                izquierda: 'dt-left'
            };
            const v_columnasDataTable = p_columnas
                .map(function (p_columna) {
                const v_propiedad = typeof p_columna.propiedad === 'string' ? p_columna.propiedad : null;
                const v_opcion = p_columna.opcion;
                const v_titulo = typeof p_columna.titulo === 'string' ? p_columna.titulo : p_columna.propiedad;
                const v_visible = typeof p_columna.visible === 'boolean' ? p_columna.visible : true;
                const v_numerico = typeof p_columna.numerico === 'boolean' ? p_columna.numerico : false;
                const v_alineacion = typeof (p_columna.alineacion) === 'string' ? p_columna.alineacion : null;
                let v_clase = "";
                if (v_alineacion !== null && typeof v_clasesAlineacion[v_alineacion] === 'string')
                    v_clase = v_clasesAlineacion[p_columna.alineacion];
                if (!p_columna.ancho)
                    v_clase += " autoajustado";
                return Tabla._generateColumn(v_propiedad, v_titulo, v_visible, v_numerico, v_clase, p_opciones[v_opcion]);
            });
            return v_columnasDataTable;
        }
        static _creaDefinicionesDataTable(p_columnas) {
            const v_definicionesColumna = [];
            let v_definicionesOrden;
            p_columnas.forEach(function (p_columna, p_indice) {
                if ((typeof p_columna.ancho !== 'undefined' && p_columna.ancho !== null) ||
                    (typeof p_columna.orden !== 'undefined' && p_columna.orden !== null)) {
                    const v_definicionColumna = { targets: p_indice };
                    if (typeof p_columna.ordenColumnas !== 'undefined' && p_columna.ordenColumnas !== null) {
                        const v_indices = p_columna.ordenColumnas.map(p_ordenColumna => p_columnas.findIndex(p => p.propiedad === p_ordenColumna));
                        v_definicionColumna.orderData = v_indices;
                    }
                    if (typeof p_columna.ancho !== 'undefined' && p_columna.ancho !== null)
                        v_definicionColumna.width = p_columna.ancho;
                    if (typeof p_columna.orden !== 'undefined' && p_columna.orden !== null) {
                        v_definicionColumna.orderable = true;
                        const v_definicionOrden = [p_indice, p_columna.orden === '*' ? null : p_columna.orden];
                        v_definicionesOrden = (v_definicionesOrden || []).concat([v_definicionOrden]);
                    }
                    else
                        v_definicionColumna.orderable = false;
                    if (p_columna.tipoDato === Date) {
                        v_definicionColumna.render = $.fn.dataTable.render.moment('DD/MM/YYYY');
                    }
                    if (!p_columna.ancho || p_columna.ancho === 'auto')
                        v_definicionColumna.className = 'autoajustado';
                    v_definicionesColumna.push(v_definicionColumna);
                }
            });
            return {
                columnas: v_definicionesColumna,
                ordenaciones: v_definicionesOrden
            };
        }
    }
    exports.Tabla = Tabla;
    Tabla._botones = {
        elimina: { contenido: '<button class="opcion-elimina boton boton-delete">X</button>' }
    };
    function tabla(p_elemento) {
        let v_idElemento = p_elemento instanceof HTMLElement ? p_elemento.id : p_elemento;
        return new DefinicionTabla(v_idElemento, {
            datos: [],
            columnas: [],
            mensajeSinRegistros: null,
            editable: true
        });
    }
    exports.tabla = tabla;
});
