define(["require", "exports", "Utilidades/assert", "componentes/api", "componentes/componentes", "componentes/foco", "componentes/pantalla", "componentes/boton", "componentes/campo", "consultas/consulta-articulo/modelos/modelo", "componentes/tabla"], function (require, exports, assert, api_1, componentes_1, foco_1, pantalla_1, boton_1, campo_1, modelo_1, tabla_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PantallaConsultaArticulo = void 0;
    class PantallaConsultaArticulo extends pantalla_1.Pantalla {
        constructor() {
            super();
            this.inicializa();
            this.asociaComponentes();
        }
        async asociaComponentes() {
            await this.inicioArticulo();
            const vComponentes = this.inicializaComponentes();
            Object.getOwnPropertyNames(vComponentes).forEach(pNombre => this[pNombre] = vComponentes[pNombre].crea());
            if (this._config.PermiteRegularizar) {
                this._botonRegularizar.habilita().visible(true);
            }
            else {
                this._botonRegularizar.deshabilita().visible(false);
            }
            this._inputArticulo.foco();
            this.actualizaVista();
            return;
        }
        actualizaVista() {
            this._inputArticulo.valor(this._modelo.GAART_COD);
            this._inputArticuloDescrip.valor(this._modelo.GAART_DESCRIP);
            this._inputExistencias.valor(this._modelo.C_EXIST1);
            this._inputAgrupaciones.valor(this._modelo.C_EXIST1_ALMAGRU);
            this._botonRegularizar.habilitado(this.habilitarConfirmacion());
            this._tablaConsulta.editable(this._config.PermiteRegularizar);
            this._tablaConsulta.datos(this._modelo.Existencias);
        }
        actualizaModelo() {
            this._modelo.GAART_COD = this._inputArticulo.valor();
            this._modelo.GAART_DESCRIP = this._inputArticuloDescrip.valor();
            this._modelo.C_EXIST1 = this._inputExistencias.valor();
            this._modelo.C_EXIST1_ALMAGRU = this._inputAgrupaciones.valor();
        }
        async inicializa() {
            if (!this._config)
                this._config = new modelo_1.ConfigConsultaArticulo();
            if (!this._modelo)
                this._modelo = new modelo_1.ExistenciasArticulo();
            this._modelo.Init();
        }
        inicializaComponentes() {
            foco_1.foco.enPrimero(document);
            let vComponentes = {
                _botonNuevo: (0, boton_1.boton)('btnReset')
                    .clic(() => {
                    this.nuevo();
                }),
                _botonRegularizar: (0, boton_1.boton)('btnUpdate')
                    .clic(async () => {
                    this.Regularizar();
                })
                    .tecla('Enter'),
                _inputArticulo: (0, campo_1.campo)('articulo')
                    .cambio(async () => {
                    this.actualizaModelo();
                    await this.ConsultaArticulo.bind(this)();
                    if (!this._config.PermiteRegularizar)
                        this._inputArticulo.foco();
                })
                    .lov(this.lovArticulo.bind(this)),
                _inputArticuloDescrip: (0, campo_1.campo)('desc'),
                _inputExistencias: (0, campo_1.campo)('exist'),
                _inputAgrupaciones: (0, campo_1.campo)('agrup'),
                _btnBusquedaArticulo: (0, boton_1.boton)('btnConsArt').clic(this.lovArticulo.bind(this)),
                _tablaConsulta: (0, tabla_1.tabla)('tablaConsultas')
                    .columnas([
                    { propiedad: 'GAUBIC_COD', titulo: Resources.table_head_location },
                    { propiedad: 'LOTE', titulo: Resources.table_head_batch },
                    { propiedad: 'C_UNI1', titulo: Resources.table_head_shrt_quantity, numerico: true, editable: true },
                ])
            };
            if (this._config.Reservas) {
                const vColumnaReservas = { propiedad: 'C_RESER1', titulo: "C. Res.", numerico: true };
                vComponentes._tablaConsulta.columnas().splice(2, 0, vColumnaReservas);
            }
            return vComponentes;
        }
        nuevo() {
            this.inicializa();
            this.actualizaVista();
            this._inputArticulo.foco();
        }
        habilitarConfirmacion() {
            return this._config.PermiteRegularizar;
        }
        async Regularizar() {
            this.actualizaModelo();
            let resultado = null;
            resultado = await this.apiRegularizar();
            if (resultado) {
                (0, componentes_1.notificacion)().muestra("Regularizado");
                this.nuevo();
            }
        }
        async lovArticulo() {
            const vResultado = await this.ConsultaArticulos();
            const vElegido = await (0, componentes_1.lov)()
                .titulo('Artículos')
                .columnas([
                { propiedad: 'COD', titulo: 'Código', ancho: '6em' },
                { propiedad: 'DESCRIP', titulo: 'Nombre' }
            ])
                .elige(vResultado);
            this._modelo.GAART_COD = vElegido.COD;
            this.actualizaVista();
            await this.ConsultaArticulo();
            this.actualizaVista();
        }
        async ConsultaArticulo() {
            this.actualizaModelo();
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Consultas/ExistenciasArticulo')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._config.EMPGRUPO_COD,
                    EMP_COD: this._config.EMP_COD,
                    LecturaArticulo: this._modelo.GAART_COD
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            finally {
                this._inputArticulo.foco();
            }
            assert.esUn(vResultado, "GAART_COD", "GAART_DESCRIP", "C_EXIST1", "C_EXIST2", "C_EXIST1_ALMAGRU", "C_EXIST2_ALMAGRU", "Existencias");
            vResultado.Existencias = vResultado.Existencias.map((i) => {
                i.C_UNI1_aux = i.C_UNI1;
                return i;
            });
            Object.assign(this._modelo, vResultado);
            this.actualizaVista();
        }
        async ConsultaArticulos() {
            this.actualizaModelo();
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Consultas/Articulos')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._config.EMPGRUPO_COD,
                    EMP_COD: this._config.EMP_COD
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                this._inputArticulo.foco();
                return;
            }
            assert.esUnArrayDe(vResultado, "COD", "DESCRIP");
            return vResultado;
        }
        async apiRegularizar() {
            try {
                await (0, api_1.api)('/Consultas/Regularizar')
                    .metodo('put')
                    .parametros({
                    EMPGRUPO_COD: this._config.EMPGRUPO_COD,
                    EMP_COD: this._config.EMP_COD,
                    Movimientos: this._modelo.Existencias.filter((item) => {
                        return item.C_UNI1 !== item.C_UNI1_aux;
                    }).map((item) => {
                        let movimiento = new modelo_1.Movimiento();
                        movimiento.GAART_COD = this._modelo.GAART_COD;
                        movimiento.GAUBIC_COD = item.GAUBIC_COD;
                        movimiento.LOTE = item.LOTE;
                        movimiento.C_UNID1 = item.C_UNI1;
                        return movimiento;
                    })
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return false;
            }
            return true;
        }
        async inicioArticulo() {
            let vResultado;
            try {
                [vResultado] = await (0, api_1.api)('/Consultas/InicioArticulo')
                    .metodo('get')
                    .parametros({
                    EMPGRUPO_COD: this._config.EMPGRUPO_COD,
                    EMP_COD: this._config.EMP_COD
                })
                    .invoca();
            }
            catch ([vMensaje]) {
                await (0, componentes_1.mensaje)().error().muestra(vMensaje);
                return;
            }
            assert.esUn(vResultado, "MostrarAgrupacion", "PermiteRegularizar", "Reservas");
            this._config.MostrarAgrupacion = vResultado.MostrarAgrupacion;
            this._config.PermiteRegularizar = vResultado.PermiteRegularizar;
            this._config.Reservas = vResultado.Reservas;
            return vResultado;
        }
    }
    exports.PantallaConsultaArticulo = PantallaConsultaArticulo;
});
